@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,12 +21,20 @@
 #include "alloca.h"
 #include "analyzer.h"
 #include "currenttime.h"
-#include "debug.h"
 #include "tree.h"
+#include "debug.h"
 
 #define SYMPRIME 65519
 #define WAVE_DECOMPRESSOR "gzip -cd "	/* zcat alone doesn't cut it for AIX */
 
+#ifndef _MSC_VER
+#include <unistd.h>
+#include <inttypes.h>
+#else
+typedef long off_t;
+#include <windows.h>
+#include <io.h>
+#endif
 
 struct fac
 {
@@ -65,6 +73,7 @@ struct symbol *symbol;
 
 struct symbol *symfind(char *);
 struct symbol *symadd(char *, int);
+struct symbol *symadd_name_exists(char *name, int hv);
 int hash(char *s);
 
 void facsplit(char *, int *, int *);
@@ -74,6 +83,7 @@ void quicksort(struct symbol **, int, int);
 void wave_heapsort(struct symbol **a, int num);
 
 struct Bits *makevec(char *, char *);
+struct Bits *makevec_annotated(char *, char *);
 int maketraces(char *);
 
 int parsewavline(char *);
@@ -99,4 +109,8 @@ struct Bits *makevec_chain(char *vec, struct symbol *sym, int len);
 int add_vector_chain(struct symbol *s, int len);
 char *makename_chain(struct symbol *sym);
 
+/* splash screen activation (version >= GTK2 only) */
+void splash_create(void);
+void splash_sync(off_t current, off_t total);  
+
 #endif
