@@ -24,7 +24,7 @@
 
 #ifndef __FreeBSD__
 #ifndef	__NetBSD__
-#ifndef __sun__
+#if !defined __sun__ && !defined _AIX
 	#ifndef alloca
 	#define alloca __alloca
 	#endif
