@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,6 +14,8 @@
 #include "debug.h"
 #include "bsearch.h"
 #include "strace.h"
+#include "translate.h"
+#include "ptranslate.h"
 
 #ifdef _MSC_VER
 #define strcasecmp _stricmp
@@ -21,7 +23,7 @@
 
 unsigned int default_flags=TR_RJUSTIFY;
 Times        tims = {0, 0, 0, 0, 0, 0, 0};
-Traces       traces = {0, NULL, NULL, NULL, NULL, 0};
+Traces       traces = {0, 0, NULL, NULL, NULL, NULL, 0};
 
 /*
  * extract last n levels of hierarchy
@@ -79,6 +81,17 @@ static void AddTrace( Trptr t )
 if(default_flags&TR_NUMMASK) t->flags=default_flags;
 	else t->flags=(t->flags&TR_NUMMASK)|default_flags;
 
+if(default_flags & TR_FTRANSLATED)
+	{
+	t->f_filter = current_translate_file;
+	}
+else
+if(default_flags & TR_PTRANSLATED)
+	{
+	t->p_filter = current_translate_proc;
+	}
+
+
 if(shift_timebase_default_for_add)
 	t->shift=shift_timebase_default_for_add;
 
@@ -86,14 +99,14 @@ if(!shadow_active)
 	{
 	if( traces.first == NULL )
 		{
-		t->next = t->prev = NULL;
+		t->t_next = t->t_prev = NULL;
 		traces.first = traces.last = t;
 	      	}
 	    	else
 	      	{
-		t->next = NULL;
-		t->prev = traces.last;
-		traces.last->next = t;
+		t->t_next = NULL;
+		t->t_prev = traces.last;
+		traces.last->t_next = t;
 		traces.last = t;
 	      	}
 	traces.total++;
@@ -109,7 +122,6 @@ if(!shadow_active)
 	shadow_string = NULL;
 
 	shadow_straces = st;
-
 	}
 }
 
@@ -146,7 +158,11 @@ if( (t = (Trptr) calloc_2( 1, sizeof( TraceEnt ))) == NULL )
 	return( 0 );
       	}
 AddTrace(t);
-t->flags=TR_BLANK;
+t->flags = TR_BLANK | (default_flags & (TR_COLLAPSED|TR_ANALOG_BLANK_STRETCH));
+if(t->flags & TR_ANALOG_BLANK_STRETCH)
+	{
+	t->flags &= ~TR_BLANK;
+	}
 
 if((comment=precondition_string(commentname)))
 	{
@@ -161,7 +177,7 @@ return(1);
 /*
  * Insert a blank [or comment] trace into the display...
  */
-int InsertBlankTrace(char *comment)
+int InsertBlankTrace(char *comment, int different_flags)
 {
 TempBuffer tb;
 char *comm;
@@ -172,7 +188,15 @@ if( (t = (Trptr) calloc_2( 1, sizeof( TraceEnt ))) == NULL )
 	fprintf( stderr, "Out of memory, can't insert blank trace to analyzer\n");
 	return( 0 );
       	}
-t->flags=TR_BLANK;
+
+if(!different_flags)
+	{
+	t->flags=TR_BLANK;
+	}
+	else
+	{
+	t->flags = different_flags;
+	}
 
 if((comm=precondition_string(comment)))
 	{
@@ -208,7 +232,7 @@ if(!traces.first)
 /*
  * Adds a single bit signal to the display...
  */
-int AddNode(nptr nd, char *aliasname)
+int AddNodeTraceReturn(nptr nd, char *aliasname, Trptr *tret)
   {
     Trptr  t;
     hptr histpnt;
@@ -299,11 +323,18 @@ if(aliasname)
 	}
     t->vector = FALSE;
     t->n.nd = nd;
+    if(tret) *tret = t;		/* for expand */
     AddTrace( t );
     return( 1 );
   }
 
 
+int AddNode(nptr nd, char *aliasname)
+{
+return(AddNodeTraceReturn(nd, aliasname, NULL));
+}
+
+
 /*
  * Adds a vector to the display...
  */
@@ -370,6 +401,7 @@ if(t->vector)
 	if(bv->bits)
 		{
 		if(bv->bits->name) free_2(bv->bits->name);
+		if(bv->bits->attribs) free_2(bv->bits->attribs);
 		for(i=0;i<bv->nbits;i++)
 			{
 			DeleteNode(bv->bits->nodes[i]);
@@ -404,19 +436,19 @@ void RemoveTrace( Trptr t, int dofree )
     traces.total--;
     if( t == traces.first )
       {
-	traces.first = t->next;
-	if( t->next )
-            t->next->prev = NULL;
+	traces.first = t->t_next;
+	if( t->t_next )
+            t->t_next->t_prev = NULL;
         else
             traces.last = NULL;
       }
     else
       {
-        t->prev->next = t->next;
-        if( t->next )
-            t->next->prev = t->prev;
+        t->t_prev->t_next = t->t_next;
+        if( t->t_next )
+            t->t_next->t_prev = t->t_prev;
         else
-            traces.last = t->prev;
+            traces.last = t->t_prev;
       }
     
     if(dofree)
@@ -437,7 +469,7 @@ t=traces.buffer;
 
 while(t)
 	{
-	t2=t->next;
+	t2=t->t_next;
 	FreeTrace(t);
 	t=t2;	
 	}
@@ -463,18 +495,54 @@ t=traces.first;
 while(t)
 	{
 	if((t->flags)&(TR_HIGHLIGHT)) break;
-	t=t->next;
+	t=t->t_next;
 	}
 if(!t) return(NULL);	/* keeps a double cut from blowing out the buffer */
 
 signalwindow_width_dirty=1;
 
 FreeCutBuffer();
-t=traces.first;
 
+/*
+ * propagate cut for whole comment group if comment selected and collapsed...
+ */
+t=traces.first;
 while(t)
 	{
-	tnext=t->next;
+	top_of_cut:
+	if( (t->name) && ((t->flags&(TR_BLANK|TR_HIGHLIGHT))==(TR_BLANK|TR_HIGHLIGHT)) )
+		{
+		if(t->flags&TR_COLLAPSED)
+			{
+			t=t->t_next;
+			while(t)
+				{
+				if(t->flags & TR_BLANK) goto top_of_cut;
+				t->flags |= TR_HIGHLIGHT;
+				t=t->t_next;
+				}
+			break;
+			}	
+			else
+			{
+			t=t->t_next; /* for sanity, ensure we're uncollapsed */
+			while(t)
+				{
+				if(t->flags & TR_BLANK) goto top_of_cut;
+				t->flags &= ~TR_COLLAPSED;
+				t=t->t_next;
+				}
+			break;
+			}
+		}
+
+	t=t->t_next;
+	}
+
+t=traces.first;
+while(t)
+	{
+	tnext=t->t_next;
 	if(t->flags&TR_HIGHLIGHT)
 		{
 		traces.bufferlast=t;
@@ -485,15 +553,15 @@ while(t)
 		if(!current)
 			{
 			first=current=t;
-			t->prev=NULL;
-			t->next=NULL;
+			t->t_prev=NULL;
+			t->t_next=NULL;
 			}
 			else
 			{
-			current->next=t;
-			t->prev=current;
+			current->t_next=t;
+			t->t_prev=current;
 			current=t;
-			t->next=NULL;
+			t->t_next=NULL;
 			}
 		}
 	t=tnext;
@@ -522,7 +590,7 @@ if(!(t=traces.first))
 		{
 		traces.last=t;
 		traces.total++;
-		t=t->next;
+		t=t->t_next;
 		}	
 
 	traces.buffer=traces.bufferlast=NULL;
@@ -533,15 +601,35 @@ if(!(t=traces.first))
 
 while(t)
 	{
-	if(t->flags&TR_HIGHLIGHT) tinsert=t;
-	t=t->next;
+	if(t->flags&TR_HIGHLIGHT) 
+		{
+		if((t->flags & (TR_BLANK|TR_COLLAPSED)) && (t->name))
+			{
+			tinsert=t;
+			t=t->t_next;
+			while(t)
+				{
+				if(t->flags & TR_BLANK) goto nxtl;
+				tinsert=t;
+				t=t->t_next;
+				}
+			break;
+			}
+			else
+			{
+			tinsert=t;
+			}
+		}
+
+nxtl:	t=t->t_next;
 	}
 
 if(!tinsert) tinsert=traces.last;
-tinsertnext=tinsert->next;
-tinsert->next=traces.buffer;
-traces.buffer->prev=tinsert;
-traces.bufferlast->next=tinsertnext;
+
+tinsertnext=tinsert->t_next;
+tinsert->t_next=traces.buffer;
+traces.buffer->t_prev=tinsert;
+traces.bufferlast->t_next=tinsertnext;
 traces.total+=traces.buffercount;
 
 if(!tinsertnext)
@@ -550,7 +638,7 @@ if(!tinsertnext)
 	}
 	else
 	{
-	tinsertnext->prev=traces.bufferlast;
+	tinsertnext->t_prev=traces.bufferlast;
 	}
 
 traces.buffer=traces.bufferlast=NULL;
@@ -579,13 +667,13 @@ while(t)
 	prev=t;
 	t->flags&=(~TR_HIGHLIGHT);
 	traces.total++;
-	t=t->next;
+	t=t->t_next;
 	}
 
-if((prev->next=traces.first))
+if((prev->t_next=traces.first))
 	{
 	/* traces.last current value is ok as it stays the same */
-	traces.first->prev=prev; /* but we need the reverse link back up */
+	traces.first->t_prev=prev; /* but we need the reverse link back up */
 	}
 	else
 	{
@@ -623,7 +711,7 @@ for(i=0;i<traces.total;i++)
                 }
         *(tsort_pnt++)=t;
 
-        t=t->next;
+        t=t->t_next;
         }
 
 traces.first=*(--tsort_pnt);
@@ -634,21 +722,21 @@ for(i=traces.total-1;i>=0;i--)
 
 	if(i==traces.total-1)
 		{
-		t->prev=NULL;
+		t->t_prev=NULL;
 		}
 		else
 		{
-		t->prev=*(tsort_pnt+1);
+		t->t_prev=*(tsort_pnt+1);
 		}
 
 	if(i)
 		{
-		t->next=*(--tsort_pnt);
+		t->t_next=*(--tsort_pnt);
 		}
         }
 
 traces.last=*tsort;
-traces.last->next=NULL;
+traces.last->t_next=NULL;
 
 return(1);
 }  
@@ -748,7 +836,7 @@ return(sigcmp(str1, str2));
  */
 int TracesAlphabetize(int mode)
 {
-Trptr t, prev;
+Trptr t, prev = NULL;
 Trptr *tsort, *tsort_pnt;
 char *subst, ch;
 int i;
@@ -774,7 +862,7 @@ for(i=0;i<traces.total;i++)
         	        subst++;
         	        }
 
-        t=t->next;
+        t=t->t_next;
         }
 
 switch(mode)
@@ -794,12 +882,12 @@ for(i=0;i<traces.total;i++)
 	if(!i)
 		{
 		traces.first=t;
-		t->prev=NULL;
+		t->t_prev=NULL;
 		}
 		else
 		{
-		prev->next=t;
-		t->prev=prev;
+		prev->t_next=t;
+		t->t_prev=prev;
 		}
 
 	prev=t;
@@ -813,8 +901,154 @@ for(i=0;i<traces.total;i++)
         }
 
 traces.last=prev;
-prev->next=NULL;
+prev->t_next=NULL;
 
 return(1);
 }  
 
+
+/*
+ * trace traversal with collapsed groups
+ */
+Trptr GiveNextTrace(Trptr t)
+{
+if((t->name)&&((t->flags & TR_ISCOLLAPSED)==TR_ISCOLLAPSED))
+	{
+	t=t->t_next;
+	while(t)
+		{
+		if(t->flags & TR_BLANK) break;
+		t=t->t_next;
+		}
+	}
+	else
+	{
+	t=t->t_next;
+	}
+
+return(t);
+}
+
+
+Trptr GivePrevTrace(Trptr t)
+{
+Trptr t2 = t;
+int iter = 0;
+
+while(t2)
+	{
+	if((t2->flags & TR_COLLAPSED) && (!(t2->flags & TR_BLANK)))
+		{
+		if(!iter)
+			{
+			t2=t2->t_prev;
+			iter=1;
+			continue;
+			}
+			else
+			{
+			break;
+			}
+		}
+		else
+		{
+		if(iter) break;
+		return(t2->t_prev);
+		}
+	}
+
+return(t2);
+}
+
+
+int CollapseTrace(Trptr t)
+{
+int rc;
+int flg;
+
+if((rc=((t->name)&&(t->flags&TR_BLANK))))
+	{
+        Trptr t2 = t;
+
+        t2->flags ^= TR_COLLAPSED;
+	flg = t2->flags & TR_COLLAPSED;
+	t2->flags &= ~TR_HIGHLIGHT;
+
+        t2=t->t_next;
+        while(t2)
+        	{
+                if(t2->flags & TR_BLANK) break;
+                t2->flags &= ~(TR_COLLAPSED|TR_HIGHLIGHT);
+		t2->flags |= flg;
+                t2=t2->t_next;
+                }
+
+	UpdateTracesVisible();
+	}
+
+return(rc);
+}
+
+
+int UpdateTracesVisible(void)
+{
+Trptr t = traces.first;
+int cnt = 0;
+
+while(t)
+	{
+	if( (t->flags & TR_BLANK) || (!(t->flags & TR_COLLAPSED)) )
+		{
+		cnt++;
+		}
+
+	t=t->t_next;
+	}
+
+traces.visible = cnt;
+return(cnt);
+}
+
+
+void CollapseAllGroups(void)
+{
+Trptr t = traces.first;
+int mode = 0;
+
+while(t)
+	{
+	t->flags &= ~TR_HIGHLIGHT;
+
+	if(t->flags & TR_BLANK)
+		{
+		mode = (t->name != NULL);
+		}
+
+	if(mode)
+		{
+		t->flags |= TR_COLLAPSED;
+		}
+		else
+		{
+		t->flags &= ~TR_COLLAPSED;
+		}
+
+	t=t->t_next;
+	}
+
+UpdateTracesVisible();
+}
+
+
+void ExpandAllGroups(void)
+{
+Trptr t = traces.first;
+
+while(t)
+	{
+	t->flags &= ~(TR_HIGHLIGHT|TR_COLLAPSED);
+	t=t->t_next;
+	}
+
+UpdateTracesVisible();
+}
