@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2003.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,6 +16,8 @@
 #include "debug.h"
 #include "bsearch.h"
 #include "strace.h"
+#include "translate.h"
+#include "ptranslate.h"
 #include <stdlib.h>
 
 /*
@@ -53,6 +55,7 @@ vptr vhead=NULL, vcurr=NULL, vadd;
 int numextrabytes;
 TimeType mintime, lasttime=-1;
 bvptr bitvec=NULL;
+TimeType tshift, tmod;
 
 if(!b) return(NULL);
 
@@ -74,10 +77,25 @@ while(h[0])	/* should never exit through this point the way we set up histents w
 	
 	for(i=0;i<b->nbits;i++)	/* was 1...big mistake */
 		{
+		tshift = (b->attribs) ? b->attribs[i].shift : 0;
+
 		if(h[i]->next)
-		if(h[i]->next->time < mintime)
-			{	
-			mintime = h[i]->next->time;
+			{
+			if((h[i]->next->time >= 0) && (h[i]->next->time < MAX_HISTENT_TIME-2))
+				{
+				tmod = h[i]->next->time+tshift;
+				if(tmod < 0) tmod = 0;
+				if(tmod > MAX_HISTENT_TIME-2) tmod = MAX_HISTENT_TIME-2;
+				}
+				else
+				{
+				tmod = h[i]->next->time;	/* don't timeshift endcaps */
+				}
+
+			if(tmod < mintime)
+				{	
+				mintime = tmod;
+				}
 			}
 		}
 
@@ -90,13 +108,69 @@ while(h[0])	/* should never exit through this point the way we set up histents w
 		{
 		unsigned char enc;
 
-		enc  = ((unsigned char)(h[i]->v.val))&3;
+		tshift = (b->attribs) ? b->attribs[i].shift : 0;
+
+		if((b->attribs)&&(b->attribs[i].flags & TR_INVERT))
+			{
+			enc  = ((unsigned char)(h[i]->v.h_val));
+			switch(enc)	/* don't remember if it's preconverted in all cases; being conservative is OK */
+				{
+				case AN_0: case '0':
+					enc = AN_1; break;
+
+				case AN_1: case '1':
+					enc = AN_0; break;
+
+				case AN_H: case 'h': case 'H':
+					enc = AN_L; break;
+
+				case AN_L: case 'l': case 'L':
+					enc = AN_H; break;
+
+				case 'x': case 'X':
+					enc = AN_X; break;
+
+				case 'z': case 'Z':
+					enc = AN_Z;
+
+				case 'u': case 'U':
+					enc = AN_U;
+
+				case 'w': case 'W':
+					enc = AN_W;
+
+				default: enc = enc & AN_MSK; break;
+				}
+			}
+			else
+			{
+			enc  = ((unsigned char)(h[i]->v.h_val)) & AN_MSK;
+			}
+
 		vadd->v[i] = enc;
 
 		if(h[i]->next)
-		if(h[i]->next->time == mintime)
 			{
-			h[i]=h[i]->next;
+			if((h[i]->next->time >= 0) && (h[i]->next->time < MAX_HISTENT_TIME-2))
+				{
+				tmod = h[i]->next->time+tshift;
+				if(tmod < 0) tmod = 0;
+				if(tmod > MAX_HISTENT_TIME-2) tmod = MAX_HISTENT_TIME-2;
+				}
+				else
+				{
+				tmod = h[i]->next->time;	/* don't timeshift endcaps */
+				}
+
+			if(tmod < mintime)
+				{	
+				mintime = tmod;
+				}
+
+			if(tmod == mintime)
+				{
+				h[i]=h[i]->next;
+				}
 			}
 		}
 
@@ -355,6 +429,125 @@ if(nodepnt)
 return(b);
 }
 
+/*
+ * Create an annotated (b->attribs) vector from stranded signals...
+ */
+struct Bits *makevec_annotated(char *vec, char *str)
+{
+char *pnt, *wild=NULL;
+char ch;
+int len, nodepnt=0;
+int i;
+struct Node *n[512];
+struct BitAttributes ba[512];
+struct Bits *b=NULL;
+int state = 0;
+
+while(1)
+{
+pnt=str;
+len=0;
+
+while(1)
+	{
+	ch=*pnt++;
+	if(isspace(ch)||(!ch)) break;
+	len++;
+	}
+
+if(len)
+	{
+	wild=(char *)calloc_2(1,len+1);
+	memcpy(wild,str,len);
+
+	DEBUG(printf("WILD: %s\n",wild));
+
+	if(state==1)
+		{
+		ba[nodepnt-1].shift = atoi_64(wild);
+		state++;
+		goto fw;		
+		}
+	else
+	if(state==2)
+		{
+		sscanf(wild, "%x", &ba[nodepnt-1].flags);
+		state = 0;
+		goto fw;
+		}
+
+	state++;
+
+	/* no wildcards for annotated! */
+		{
+		struct symbol *s;
+
+		if(nodepnt==512) { free_2(wild); goto ifnode; }
+
+		if(wild[0]=='(')
+			{
+			nptr nexp;
+			
+			for(i=1;;i++)
+				{
+				if(wild[i]==0) break;
+				if((wild[i]==')')&&(wild[i+1])) 
+					{
+					i++; 
+					s=symfind(wild+i);
+					if(s)
+						{
+						nexp = ExtractNodeSingleBit(s->n, atoi(wild+1));
+						if(nexp)
+							{
+							n[nodepnt++]=nexp;
+							if(nodepnt==512) { free_2(wild); goto ifnode; }
+							}		
+						}
+					break;
+					}
+				}
+			}
+			else
+			{
+			if((s=symfind(wild)))	
+				{
+				n[nodepnt++]=s->n;
+				}
+			}
+		}
+
+fw:	free_2(wild);
+	}
+
+if(!ch) break;
+str=pnt;
+}
+
+ifnode:
+if(nodepnt)
+	{
+	b=(struct Bits *)calloc_2(1,sizeof(struct Bits)+(nodepnt-1)*
+				  sizeof(struct Node *));
+
+	b->attribs = calloc_2(nodepnt, sizeof(struct BitAttributes));
+
+	for(i=0;i<nodepnt;i++)
+		{
+		b->nodes[i]=n[i];
+		if(n[i]->mvlfac) import_trace(n[i]);
+
+		b->attribs[i].shift = ba[i].shift;
+		b->attribs[i].flags = ba[i].flags;
+		}
+
+	b->nbits=nodepnt;
+	strcpy(b->name=(char *)malloc_2(strlen(vec)+1),vec);
+	}
+
+return(b);
+}
+
 
 /*
  * Create a vector from selected_status signals...
@@ -424,6 +617,7 @@ if((b=makevec_selected(alias, numrows, direction)))
                 else
                 {
                 free_2(b->name);
+		if(b->attribs) free_2(b->attribs);
                 free_2(b);
                 }
         }
@@ -443,7 +637,7 @@ int nodepnt=0, nodepnt_rev;
 int i;
 struct Node **n;
 struct Bits *b=NULL;
-struct symbol *symhi, *symlo;
+struct symbol *symhi = NULL, *symlo = NULL;
 char hier_delimeter2;
 
 if(!vcd_explicit_zero_subscripts)	/* 0==yes, -1==no */
@@ -524,7 +718,14 @@ if(nodepnt)
 			{
 			if(symlo!=symhi)
 				{
-				strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+				if(!b->attribs)
+					{
+					strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+					}
+					else
+					{
+					strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>");
+					}
 				}
 				else
 				{
@@ -544,7 +745,7 @@ if(nodepnt)
 			
 			if(symlo!=symhi)
 				{
-				unsigned char fixup1, fixup2;
+				unsigned char fixup1 = 0, fixup2 = 0;
 
 				totallen=
 					root1len
@@ -575,7 +776,7 @@ if(nodepnt)
 				}
 				else
 				{
-				unsigned char fixup1;
+				unsigned char fixup1 = 0;
 
 				totallen=
 					root1len
@@ -630,6 +831,7 @@ if(len>1)
 	                else
 	                {
 	                free_2(b->name);
+			if(b->attribs) free_2(b->attribs);
 	                free_2(b);
 	                }
 	        }
@@ -720,7 +922,14 @@ if(nodepnt)
 			{
 			if(lo!=hi)
 				{
-				strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+				if(!b->attribs)
+					{
+					strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+					}
+					else
+					{
+					strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>");
+					}
 				}
 				else
 				{
@@ -795,6 +1004,7 @@ if(lo!=hi)
 	                else
 	                {
 	                free_2(b->name);
+			if(b->attribs) free_2(b->attribs);
 	                free_2(b);
 	                }
 	        }
@@ -1070,7 +1280,7 @@ qsort(a, num, sizeof(struct symbol *), qssigcomp);
 char *makename_chain(struct symbol *sym)
 {
 int i;
-struct symbol *symhi, *symlo;
+struct symbol *symhi = NULL, *symlo = NULL;
 char hier_delimeter2;
 char *name=NULL;
 char *s1, *s2;
@@ -1147,7 +1357,7 @@ if((root1len!=root2len)||(!root1len)||(!root2len)||
 			
 	if(symlo!=symhi)
 		{
-		unsigned char fixup1, fixup2;
+		unsigned char fixup1 = 0, fixup2 = 0;
 
 		totallen=
 			root1len
@@ -1178,7 +1388,7 @@ if((root1len!=root2len)||(!root1len)||(!root2len)||
 		}
 		else
 		{
-		unsigned char fixup1;
+		unsigned char fixup1 = 0;
 
 		totallen=
 			root1len
@@ -1216,8 +1426,7 @@ int width;
 int msb, lsb, delta;
 int actual;
 hptr h, htemp;
-hptr *harray;
-int i, j, k;
+int i, j;
 nptr *narray;
 char *nam;
 int offset, len;
@@ -1332,14 +1541,14 @@ if(!n->ext)
 				if(narray[j]->curr)
 					{
 					htemp = (hptr) calloc_2(1, sizeof(struct HistEnt));				
-					htemp->v.val = 1;			/* 'x' */
+					htemp->v.h_val = AN_X;			/* 'x' */
 					htemp->time = h->time;
 					narray[j]->curr->next = htemp;
 					narray[j]->curr = htemp;
 					}
 					else
 					{
-					narray[j]->head.v.val = 1; 		/* 'x' */
+					narray[j]->head.v.h_val = AN_X;		/* 'x' */
 					narray[j]->head.time  = h->time;
 					narray[j]->curr = &(narray[j]->head);
 					}
@@ -1351,22 +1560,25 @@ if(!n->ext)
 			{
 			for(j=0;j<width;j++)
 				{
-				unsigned char val = h->v.vector[j];
+				unsigned char val = h->v.h_vector[j];
 				switch(val)
 					{
-					case '0':	val = 0; break;
-					case 'x':
-					case 'X':	val = 1; break;
-					case 'z':
-					case 'Z':	val = 2; break;
-					case '1':	val = 3; break;
+					case '0':		val = AN_0; break;
+					case '1':		val = AN_1; break;
+					case 'x': case 'X':	val = AN_X; break;
+					case 'z': case 'Z':	val = AN_Z; break;
+					case 'h': case 'H':	val = AN_H; break;
+					case 'l': case 'L':	val = AN_L; break;
+					case 'u': case 'U':	val = AN_U; break;
+					case 'w': case 'W':	val = AN_W; break;
+					case '-':		val = AN_DASH; break;
 					default:	break;			/* leave val alone as it's been converted already.. */
 					}
 
-				if(narray[j]->curr->v.val != val) 		/* curr will have been established already by 'x' at time: -1 */
+				if(narray[j]->curr->v.h_val != val) 		/* curr will have been established already by 'x' at time: -1 */
 					{
 					htemp = (hptr) calloc_2(1, sizeof(struct HistEnt));				
-					htemp->v.val = val;
+					htemp->v.h_val = val;
 					htemp->time = h->time;
 					narray[j]->curr->next = htemp;
 					narray[j]->curr = htemp;
@@ -1398,8 +1610,7 @@ nptr ExtractNodeSingleBit(nptr n, int bit)
 {
 int lft, rgh;
 hptr h, htemp;
-hptr *harray;
-int i, j, k;
+int i, j;
 int actual;
 nptr np;
 char *nam;
@@ -1505,14 +1716,14 @@ if(!n->ext)
 			if(np->curr)
 				{
 				htemp = (hptr) calloc_2(1, sizeof(struct HistEnt));				
-				htemp->v.val = 1;			/* 'x' */
+				htemp->v.h_val = AN_X;			/* 'x' */
 				htemp->time = h->time;
 				np->curr->next = htemp;
 				np->curr = htemp;
 				}
 				else
 				{
-				np->head.v.val = 1; 			/* 'x' */
+				np->head.v.h_val = AN_X;		/* 'x' */
 				np->head.time  = h->time;
 				np->curr = &(np->head);
 				}
@@ -1521,22 +1732,25 @@ if(!n->ext)
 			}
 			else
 			{
-			unsigned char val = h->v.vector[bit];
+			unsigned char val = h->v.h_vector[bit];
 			switch(val)
 				{
-				case '0':	val = 0; break;
-				case 'x':
-				case 'X':	val = 1; break;
-				case 'z':
-				case 'Z':	val = 2; break;
-				case '1':	val = 3; break;
+				case '0':		val = AN_0; break;
+				case '1':		val = AN_1; break;
+				case 'x': case 'X':	val = AN_X; break;
+				case 'z': case 'Z':	val = AN_Z; break;
+				case 'h': case 'H':	val = AN_H; break;
+				case 'l': case 'L':	val = AN_L; break;
+				case 'u': case 'U':	val = AN_U; break;
+				case 'w': case 'W':	val = AN_W; break;
+				case '-':		val = AN_DASH; break;
 				default:	break;			/* leave val alone as it's been converted already.. */
 				}
 
-			if(np->curr->v.val != val) 		/* curr will have been established already by 'x' at time: -1 */
+			if(np->curr->v.h_val != val) 		/* curr will have been established already by 'x' at time: -1 */
 				{
 				htemp = (hptr) calloc_2(1, sizeof(struct HistEnt));				
-				htemp->v.val = val;
+				htemp->v.h_val = val;
 				htemp->time = h->time;
 				np->curr->next = htemp;
 				np->curr = htemp;
@@ -1595,7 +1809,7 @@ if(n->expansion)
  */
 int parsewavline(char *w)
 {
-int i, j;
+int i;
 int len;
 char *prefix, *suffix;
 char *w2;
@@ -1673,6 +1887,11 @@ if(*w2=='@')
 	{
 	/* handle trace flags */
 	sscanf(w2+1, "%x", &default_flags);
+	if( (default_flags & (TR_FTRANSLATED|TR_PTRANSLATED)) == (TR_FTRANSLATED|TR_PTRANSLATED) )
+		{
+		default_flags &= ~TR_PTRANSLATED; /* safest bet though this is a cfg file error */
+		}
+
 	return(~0);
 	}
 else
@@ -1718,11 +1937,12 @@ aliaspt2:
 		}
 	}
 else
-if(*w2=='#')
+if((*w2=='#')||(*w2==':'))
 	{
 	/* handle bitvec */
 	bvptr v=NULL;
 	bptr b=NULL;
+	int maketyp = (*w2=='#');
 
 	w2=w2+strlen(prefix);
 	while(1)
@@ -1732,8 +1952,9 @@ if(*w2=='#')
 		break;			/* start grabbing chars from here */
 		}
 
+	b = maketyp ? makevec(prefix+1,w2) : makevec_annotated(prefix+1,w2);	/* '#' vs ':' cases... */
 
-	if((b=makevec(prefix+1,w2)))
+	if(b)
 		{
 		if((v=bits2vector(b)))
 			{
@@ -1746,6 +1967,7 @@ if(*w2=='#')
 			else
 			{
 			free_2(b->name);
+			if(b->attribs) free_2(b->attribs);
 			free_2(b);
 			}
 		}
@@ -1783,6 +2005,26 @@ if(*w2=='!')
 			}
 		}
 
+	shadow_mark_idx_start = 0;
+	shadow_mark_idx_end = 0;
+
+	if(i==6)
+		{
+		ch = *(w2+7);
+		if(ch != 0)
+			{
+			if (isupper(ch) || ch=='@')
+				shadow_mark_idx_start = ch - '@';
+		
+			ch = *(w2+8);
+			if(ch != 0)
+				{
+				if (isupper(ch) || ch=='@')
+					shadow_mark_idx_end = ch - '@';
+				}
+			}
+		}
+
 	shadow_active = 1;
 	return(~0);
 	}
@@ -1813,6 +2055,43 @@ if(*w2=='?')
 
 	return(~0);
 	}
+else if(*w2=='^')
+	{
+	if(*(w2+1) == '>')
+		{
+		current_translate_proc = 0;	/* will overwrite if loadable/translatable */
+
+		if(*(w2+2) != '0')
+			{
+			char *fn = strstr(w2+3, " ");
+			if(fn)
+				{
+				while(*fn && isspace(*fn)) fn++;
+				if(*fn && !isspace(*fn)) 
+					{
+					set_current_translate_proc(fn);
+					}
+				}
+			}
+		}
+		else
+		{
+		current_translate_file = 0;	/* will overwrite if loadable/translatable */
+
+		if(*(w2+1) != '0')
+			{
+			char *fn = strstr(w2+2, " ");
+			if(fn)
+				{
+				while(*fn && isspace(*fn)) fn++;
+				if(*fn && !isspace(*fn)) 
+					{
+					set_current_translate_file(fn);
+					}
+				}
+			}
+		}
+	}
 	else
 	{
 	return(maketraces(w));
@@ -1850,7 +2129,6 @@ while((ch=*pnt))
 if(!wild_active)	/* short circuit wildcard evaluation with bsearch */
 	{
 	struct symbol *s;
-	nptr nexp;
 
 	if(str[0]=='(')
 		{
@@ -1921,7 +2199,7 @@ void makevec_lx2(char *str)
 {
 char *pnt, *pnt2, *wild=NULL;
 char ch, ch2, wild_active;
-int len, nodepnt=0;
+int len;
 int i;
 
 while(1)
@@ -1960,8 +2238,6 @@ if(len)
 		struct symbol *s;
 		if(wild[0]=='(')
 			{
-			nptr nexp;
-			
 			for(i=1;;i++)
 				{
 				if(wild[i]==0) break;
@@ -2011,11 +2287,10 @@ str=pnt;
  */
 void parsewavline_lx2(char *w)
 {
-int i, j;
+int i;
 int len;
 char *prefix, *suffix;
 char *w2;
-nptr nexp;
 
 if(!(len=strlen(w))) return;
 if(*(w+len-1)=='\n')
@@ -2086,12 +2361,9 @@ aliaspt2:
 		}
 	}
 else
-if(*w2=='#')
+if((*w2=='#')||(*w2==':'))
 	{
-	/* handle bitvec */
-	bvptr v=NULL;
-	bptr b=NULL;
-
+	/* handle bitvec, parsing extra time info and such is inefficient but ok for ":" case */
 	w2=w2+strlen(prefix);
 	while(1)
 		{
@@ -2111,6 +2383,9 @@ if(*w2=='!')
 if(*w2=='?')
 	{
 	}
+else if(*w2=='^')
+	{
+	}
 	else
 	{
 	maketraces_lx2(w);
