@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -11,6 +11,7 @@
 #include "debug.h"
 
 int color_back=-1;
+int color_baseline=-1;
 int color_grid=-1;
 int color_high=-1;
 int color_low=-1;
@@ -26,6 +27,16 @@ int color_vtrans=-1;
 int color_x=-1;
 int color_xfill=-1;
 
+int color_0=-1;
+int color_1=-1;
+int color_ufill=-1;
+int color_u=-1;
+int color_wfill=-1;
+int color_w=-1;
+int color_dashfill=-1;
+int color_dash=-1;
+
+
 
 /* 
  * return graphics context with tuple's color or
