@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,6 +14,7 @@
 #include <gtk/gtk.h>
 
 extern int color_back;
+extern int color_baseline;
 extern int color_grid;
 extern int color_high;
 extern int color_low;
@@ -29,6 +30,15 @@ extern int color_vtrans;
 extern int color_x;
 extern int color_xfill;
 
+extern int color_0;
+extern int color_1;
+extern int color_ufill;
+extern int color_u;
+extern int color_wfill;
+extern int color_w;
+extern int color_dashfill;
+extern int color_dash;
+
 GdkGC *alloc_color(GtkWidget *widget, int tuple, GdkGC *fallback);	/* tuple is encoded as 32bit: --RRGGBB (>=0 is valid) */
 
 #endif
