@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +15,15 @@ char is_vcd=0;
 
 char use_maxtime_display=1;
 static GtkWidget *max_or_marker_label=NULL;
+static GtkWidget *base_or_curtime_label=NULL;
    
+static TimeType cached_currenttimeval=0;
 TimeType currenttime=0;
 TimeType max_time=0;
 TimeType min_time=-1;
 char display_grid=~0;       /* default to displaying grid */
-char time_dimension='n';
+TimeType time_scale=1;      /* multiplier is 1, 10, 100   */
+char time_dimension='n';    /* nsec, psec, etc...         */
 static char *time_prefix=" munpf";
 static GtkWidget *maxtimewid;
 static GtkWidget *curtimewid;
@@ -44,10 +47,11 @@ if(use_maxtime_display)
 	}
 }
 
-TimeType unformat_time(char *buf, char dim)
+TimeType unformat_time(const char *buf, char dim)
 {
 TimeType rval;
-char *pnt, *offs=NULL, *doffs;
+const char *pnt;
+char *offs=NULL, *doffs;
 char ch;
 int i, ich, delta;
 
@@ -158,14 +162,27 @@ if(!use_maxtime_display)
 	{
 	if(val>=0)
 		{
-		if(tims.lmbcache>=0) val-=tims.lmbcache; /* do delta instead */
-
-		if((tims.lmbcache>=0)&&(val>=0))
+		if(tims.baseline>=0)
 			{
+			val-=tims.baseline; /* do delta instead */
+			*maxtext='B';
+			if(val>=0)
+				{
+				*(maxtext+1)='+';
+				reformat_time(maxtext+2, val, time_dimension);
+				}
+				else
+				{
+				reformat_time(maxtext+1, val, time_dimension);
+				}
+			}
+		else if(tims.lmbcache>=0) 
+			{
+			val-=tims.lmbcache; /* do delta instead */
 			*maxtext='+';
 			reformat_time(maxtext+1, val, time_dimension);
 			}
-			else
+		else
 			{
 			reformat_time(maxtext, val, time_dimension);
 			}
@@ -180,6 +197,23 @@ if(!use_maxtime_display)
 }
 
 
+void update_basetime(TimeType val)
+{
+if(val>=0)
+	{
+	gtk_label_set(GTK_LABEL(base_or_curtime_label), "Base Marker");
+	reformat_time(curtext, val, time_dimension);
+	}
+	else
+	{
+	gtk_label_set(GTK_LABEL(base_or_curtime_label), "Current Time");
+	reformat_time_blackout(curtext, cached_currenttimeval, time_dimension);
+	}
+
+gtk_label_set(GTK_LABEL(curtimewid), curtext);
+}
+
+
 void update_maxtime(TimeType val)
 {
 max_time=val;
@@ -194,9 +228,14 @@ if(use_maxtime_display)
 
 void update_currenttime(TimeType val)
 {
-currenttime=val;
-reformat_time_blackout(curtext, val, time_dimension);
-gtk_label_set(GTK_LABEL(curtimewid), curtext);
+cached_currenttimeval = val;
+
+if(tims.baseline<0)
+	{
+	currenttime=val;
+	reformat_time_blackout(curtext, val, time_dimension);
+	gtk_label_set(GTK_LABEL(curtimewid), curtext);
+	}
 }
 
    
@@ -204,7 +243,6 @@ gtk_label_set(GTK_LABEL(curtimewid), curtext);
 GtkWidget *
 create_time_box(void)
 {
-GtkWidget *label2;
 GtkWidget *mainbox;
 
 max_or_marker_label=(use_maxtime_display)
@@ -223,19 +261,29 @@ if(use_maxtime_display)
 
 maxtimewid=gtk_label_new(maxtext);
 
-label2=gtk_label_new("Current Time");
 curtext=(char *)malloc_2(40);
-reformat_time(curtext, (currenttime=min_time), time_dimension);
-
-curtimewid=gtk_label_new(curtext);
+if(tims.baseline<0)
+	{
+	base_or_curtime_label=gtk_label_new("Current Time");
+	reformat_time(curtext, (currenttime=min_time), time_dimension);
+	curtimewid=gtk_label_new(curtext);
+	}
+	else
+	{
+	base_or_curtime_label=gtk_label_new("Base Marker");
+	reformat_time(curtext, tims.baseline, time_dimension);
+	curtimewid=gtk_label_new(curtext);
+	}
 
 mainbox=gtk_vbox_new(FALSE, 0);
+
 gtk_box_pack_start(GTK_BOX(mainbox), max_or_marker_label, TRUE, FALSE, 0);
 gtk_widget_show(max_or_marker_label);
 gtk_box_pack_start(GTK_BOX(mainbox), maxtimewid, TRUE, FALSE, 0);
 gtk_widget_show(maxtimewid);
-gtk_box_pack_start(GTK_BOX(mainbox), label2, TRUE, FALSE, 0);
-gtk_widget_show(label2);
+
+gtk_box_pack_start(GTK_BOX(mainbox), base_or_curtime_label, TRUE, FALSE, 0);
+gtk_widget_show(base_or_curtime_label);
 gtk_box_pack_start(GTK_BOX(mainbox), curtimewid, TRUE, FALSE, 0);
 gtk_widget_show(curtimewid);
    
@@ -278,3 +326,34 @@ time_trunc_val=1;
 }
 
 
+/*
+ * called by lxt/lxt2/vzt reader inits
+ */
+void exponent_to_time_scale(signed char scale)
+{
+switch(scale)
+        {
+        case 0:         time_dimension = 's'; break;
+
+        case -1:        time_scale = LLDescriptor(100); time_dimension = 'm'; break;
+        case -2:        time_scale = LLDescriptor(10);
+        case -3:                                        time_dimension = 'm'; break;
+
+        case -4:        time_scale = LLDescriptor(100); time_dimension = 'u'; break;
+        case -5:        time_scale = LLDescriptor(10);
+        case -6:                                        time_dimension = 'u'; break;
+
+        case -10:       time_scale = LLDescriptor(100); time_dimension = 'p'; break;
+        case -11:       time_scale = LLDescriptor(10);
+        case -12:                                       time_dimension = 'p'; break;
+
+        case -13:       time_scale = LLDescriptor(100); time_dimension = 'f'; break;
+        case -14:       time_scale = LLDescriptor(10);
+        case -15:                                       time_dimension = 'f'; break;
+
+        case -7:        time_scale = LLDescriptor(100); time_dimension = 'n'; break;
+        case -8:        time_scale = LLDescriptor(10);
+        case -9:
+        default:                                        time_dimension = 'n'; break;
+        }
+}
