@@ -158,9 +158,9 @@ if(ptr)
  * y/on     default to '1'
  * n/nonnum default to '0'
  */
-char *atoi_cont_ptr=NULL;
+const char *atoi_cont_ptr=NULL;
 
-TimeType atoi_64(char *str)
+TimeType atoi_64(const char *str)
 {
 TimeType val=0;
 unsigned char ch, nflag=0;
@@ -248,3 +248,42 @@ if(!disable_tooltips)
 	}
 }
 
+
+char *tmpnam_2(char *s, int *fd)
+{
+#if defined _MSC_VER || defined __MINGW32__
+
+*fd = -1;
+return(tmpnam(s));
+
+#else
+
+char *backpath = "gtkwaveXXXXXX";
+char *tmpspace;
+int len = strlen(P_tmpdir);
+int i;
+
+unsigned char slash = '/';
+for(i=0;i<len;i++)
+	{
+	if((P_tmpdir[i] == '\\') || (P_tmpdir[i] == '/'))
+		{
+		slash = P_tmpdir[i];
+		break;
+		}
+	}
+
+tmpspace = malloc_2(len + 1 + strlen(backpath) + 1);
+sprintf(tmpspace, "%s%c%s", P_tmpdir, slash, backpath);
+*fd = mkstemp(tmpspace);
+if(*fd<0)
+	{
+	fprintf(stderr, "tmpnam_2() could not create tempfile, exiting.\n");
+	perror("Why");
+	exit(255);
+	}
+
+return(tmpspace);
+
+#endif
+}
