@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2004.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,12 +32,16 @@ size_t size;
 #ifdef G_HAVE_GINT64
 	typedef gint64          TimeType; 
 	typedef guint64         UTimeType;
-
         #ifndef _MSC_VER
                 #define LLDescriptor(x) x##LL
                 #define ULLDescriptor(x) x##ULL
-                #define TTFormat "%lld"
-                #define UTTFormat "%llu"
+		#ifdef __MINGW32__
+			#define TTFormat "%I64d"
+			#define UTTFormat "%I64u"
+		#else
+	                #define TTFormat "%lld"
+	                #define UTTFormat "%llu"
+		#endif
         #else
                 #define LLDescriptor(x) x##i64
                 #define ULLDescriptor(x) x##i64
@@ -49,7 +53,6 @@ size_t size;
 #else
 	typedef long            TimeType; 
 	typedef unsigned long   UTimeType;
-
 	#define TTFormat "%d"
 	#define UTTFormat "%u"
 
@@ -92,8 +95,10 @@ void *malloc_2(size_t size);
 void *realloc_2(void *ptr, size_t size);
 void *calloc_2(size_t nmemb, size_t size);
 
-TimeType atoi_64(char *str);
-extern char *atoi_cont_ptr;	/* for unformat_time()'s parse continue for the time unit */
+char *tmpnam_2(char *s, int *fd);  /* mimic functionality of tmpnam() */
+
+TimeType atoi_64(const char *str);
+extern const char *atoi_cont_ptr;  /* for unformat_time()'s parse continue for the time unit */
 
 void gtk_tooltips_set_tip_2(GtkTooltips *tooltips, GtkWidget *widget,
         const gchar *tip_text, const gchar *tip_private);
