@@ -8,6 +8,7 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "debug.h"
 #include <string.h>
 
@@ -18,7 +19,7 @@ static GtkSignalFunc cleanup;
 
 static void enter_callback(GtkWidget *widget, GtkWidget *nothing)
 {
-  gchar *entry_text;
+  G_CONST_RETURN gchar *entry_text;
   int len;
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
   DEBUG(printf("Entry contents: %s\n", entry_text));
