@@ -8,17 +8,18 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "debug.h"
 #include <string.h>
 
 static GtkWidget *fs;
 char **fileselbox_text=NULL;
 char filesel_ok=0;
-static GtkSignalFunc cleanup;
+static GtkSignalFunc cleanup, bad_cleanup;
 
 static void enter_callback(GtkWidget *widget, GtkFileSelection *fw)
 {
-char *allocbuf;
+G_CONST_RETURN char *allocbuf;
 int alloclen;
 
 allocbuf=gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
@@ -41,18 +42,21 @@ static void cancel_callback(GtkWidget *widget, GtkWidget *nothing)
   DEBUG(printf("Filesel Entry Cancel\n"));
   gtk_grab_remove(fs);
   gtk_widget_destroy(fs);
+  if(bad_cleanup) bad_cleanup();
 }
 
 static void destroy_callback(GtkWidget *widget, GtkWidget *nothing)
 {
   DEBUG(printf("Filesel Destroy\n"));
+  if(bad_cleanup) bad_cleanup();
 }
 
-void fileselbox(char *title, char **filesel_path, GtkSignalFunc func)
+void fileselbox(char *title, char **filesel_path, GtkSignalFunc ok_func, GtkSignalFunc notok_func, char *pattn)
 {
 fileselbox_text=filesel_path;
 filesel_ok=0;
-cleanup=func;
+cleanup=ok_func;
+bad_cleanup=notok_func;
 
 fs=gtk_file_selection_new(title);
 gtk_signal_connect(GTK_OBJECT(fs), "destroy", (GtkSignalFunc) destroy_callback, NULL);
@@ -63,6 +67,11 @@ gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
 gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fs));
 if(*fileselbox_text) gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), *fileselbox_text);
 
+/*
+ * XXX: filter on patterns when this feature eventually works (or for a later version of GTK)!
+ * if((pattn)&&(pattn[0])) gtk_file_selection_complete(GTK_FILE_SELECTION(fs), pattn);
+ */
+
 gtk_grab_add(fs);
 gtk_widget_show(fs);
 }
