@@ -3,7 +3,7 @@
 
 #if WAVE_USE_GTK2
 
-#define WAVE_GTKIFE(a,b,c,d,e) a,b,c,d,e,NULL
+#define WAVE_GTKIFE(a,b,c,d,e) {a,b,c,d,e,NULL}
 #define WAVE_GDK_GET_POINTER(a,b,c,bi,ci,d)  gdk_window_get_pointer(a,bi,ci,d)
 #define WAVE_GDK_GET_POINTER_COPY x=xi; y=yi;
 #define WAVE_GTK_SIGFONT gdk_font_load("-*-courier-*-r-*-*-14-*-*-*-*-*-*-*"); \
@@ -14,7 +14,13 @@
 
 #else
 
-#define WAVE_GTKIFE(a,b,c,d,e) a,b,c,d,e
+#if !defined __MINGW32__ && !defined _MSC_VER
+#ifndef G_CONST_RETURN
+#define G_CONST_RETURN
+#endif
+#endif
+
+#define WAVE_GTKIFE(a,b,c,d,e) {a,b,c,d,e}
 #define WAVE_GDK_GET_POINTER(a,b,c,bi,ci,d)  gdk_input_window_get_pointer(a, event->deviceid, b, c, NULL, NULL, NULL, d)
 #define WAVE_GDK_GET_POINTER_COPY
 #define WAVE_GTK_SIGFONT wavearea->style->font
