@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Tony Bybell.
+ * Copyright (c) 2003-2004 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -48,7 +48,7 @@
  * of a big-endian integer.  x86 specific version...
  */
 
-#ifdef __i386__
+#if defined(__i386__) || defined(__x86_64__)
 
 #define lxt2_rd_get_byte(mm,offset)		((unsigned int)(*((unsigned char *)(mm)+(offset))))
 
@@ -824,6 +824,7 @@ if(!(lt->handle=fopen(name, "rb")))
 		lxt2_rd_close(lt);
 	        lt=NULL;
 		}
+	else
 	if(lt->granule_size > LXT2_RD_GRANULE_SIZE)
 		{
 		fprintf(stderr, LXT2_RDLOAD"*** Granule size of %d (>%d) not supported ***\n", lt->granule_size, LXT2_RD_GRANULE_SIZE);
@@ -834,7 +835,7 @@ if(!(lt->handle=fopen(name, "rb")))
 		{
 		int rc;
 		char *m;
-		long pos, fend;
+		off_t pos, fend;
 		int t;
 		struct lxt2_rd_block *b;
 
@@ -847,7 +848,7 @@ if(!(lt->handle=fopen(name, "rb")))
 		fread(&lt->timescale, 1, 1, lt->handle);		/* no swap necessary */
 
 		fprintf(stderr, LXT2_RDLOAD"%d facilities\n", lt->numfacs);
-		pos = ftell(lt->handle);
+		pos = ftello(lt->handle);
 		/* fprintf(stderr, LXT2_RDLOAD"gzip facnames start at pos %d (zsize=%d)\n", pos, lt->zfacnamesize); */
 
 		lt->process_mask = calloc(1, lt->numfacs/8+1);
@@ -875,7 +876,7 @@ if(!(lt->handle=fopen(name, "rb")))
                 lt->faccache->bufcurr = malloc(lt->longestname+1);
                 lt->faccache->bufprev = malloc(lt->longestname+1);
 
-		fseek(lt->handle, pos = pos+lt->zfacnamesize, SEEK_SET);
+		fseeko(lt->handle, pos = pos+lt->zfacnamesize, SEEK_SET);
 		/* fprintf(stderr, LXT2_RDLOAD"seeking to geometry at %d (0x%08x)\n", pos, pos); */
 		lt->zhandle = gzdopen(dup(fileno(lt->handle)), "rb");
 
@@ -910,7 +911,14 @@ if(!(lt->handle=fopen(name, "rb")))
 			lt->lsb[i] = lxt2_rd_get_32(m+i*16, 8);
 			lt->flags[i] = lxt2_rd_get_32(m+i*16, 12);
 
-			lt->len[i] = (lt->msb[i] <= lt->lsb[i]) ? (lt->lsb[i] - lt->msb[i] + 1) : (lt->msb[i] - lt->lsb[i] + 1);
+			if(!(lt->flags[i] & LXT2_RD_SYM_F_INTEGER))
+				{
+				lt->len[i] = (lt->msb[i] <= lt->lsb[i]) ? (lt->lsb[i] - lt->msb[i] + 1) : (lt->msb[i] - lt->lsb[i] + 1);
+				}
+				else
+				{
+				lt->len[i] = 32;
+				}
 			lt->value[i] = calloc(lt->len[i] + 1, sizeof(char));
 			}
 
@@ -931,11 +939,11 @@ if(!(lt->handle=fopen(name, "rb")))
 
 		for(;;)
 			{
-			fseek(lt->handle, 0L, SEEK_END);
-			fend=ftell(lt->handle);
+			fseeko(lt->handle, 0L, SEEK_END);
+			fend=ftello(lt->handle);
 			if(pos>=fend) break;
 
-			fseek(lt->handle, pos, SEEK_SET);
+			fseeko(lt->handle, pos, SEEK_SET);
 			/* fprintf(stderr, LXT2_RDLOAD"seeking to block at %d (0x%08x)\n", pos, pos); */
 
 			b=calloc(1, sizeof(struct lxt2_rd_block));
@@ -944,8 +952,8 @@ if(!(lt->handle=fopen(name, "rb")))
 			fread(&b->compressed_siz, 4, 1, lt->handle);			b->compressed_siz = lxt2_rd_get_32(&b->compressed_siz,0);
 			fread(&b->start, 8, 1, lt->handle);				b->start = lxt2_rd_get_64(&b->start,0);
 			fread(&b->end, 8, 1, lt->handle);				b->end = lxt2_rd_get_64(&b->end,0);
-			pos = ftell(lt->handle);
-			fseek(lt->handle, pos, SEEK_SET);
+			pos = ftello(lt->handle);
+			fseeko(lt->handle, pos, SEEK_SET);
 			/* fprintf(stderr, LXT2_RDLOAD"block gzip start at pos %d (0x%08x)\n", pos, pos); */
 			if(pos>=fend)
 				{
@@ -959,7 +967,7 @@ if(!(lt->handle=fopen(name, "rb")))
 			if((b->uncompressed_siz)&&(b->compressed_siz)&&(b->end))
 				{
 				/* fprintf(stderr, LXT2_RDLOAD"block [%d] %lld / %lld\n", lt->numblocks, b->start, b->end); */
-				fseek(lt->handle, b->compressed_siz, SEEK_CUR);
+				fseeko(lt->handle, b->compressed_siz, SEEK_CUR);
 
 				lt->numblocks++;
 				if(lt->block_curr)
@@ -1186,7 +1194,7 @@ if((lt)&&(facidx<lt->numfacs))
 	}
 	else
 	{
-	return(~0);
+	return(~((lxtint32_t)0));
 	}
 }
 
@@ -1473,16 +1481,16 @@ if(lt)
 
 			processed++;
 
-			fseek(lt->handle, b->filepos, SEEK_SET);
+			fseeko(lt->handle, b->filepos, SEEK_SET);
 			gzid[0]=gzid[1]=0;
 			fread(&gzid, 2, 1, lt->handle);
-			fseek(lt->handle, b->filepos, SEEK_SET);
+			fseeko(lt->handle, b->filepos, SEEK_SET);
 
 			if((striped_kill = (gzid[0]!=0x1f)||(gzid[1]!=0x8b)))
 				{
 				lxtint32_t clen, unclen, iter=0;
 				char *pnt;
-				long fspos = b->filepos;
+				off_t fspos = b->filepos;
 				int rc;
 
 				char *zbuff=NULL;
@@ -1539,7 +1547,7 @@ if(lt)
 						else
 						{
 						fspos += clen;
-						fseek(lt->handle, fspos, SEEK_SET);
+						fseeko(lt->handle, fspos, SEEK_SET);
 						}
 					}
 
