@@ -21,6 +21,12 @@
  */
 
 #include "lxt2_read.h"
+#include <getopt.h>
+
+int flat_earth = 0;
+extern void free_hier(void);
+extern char *output_hier(char *name);
+
 
 /*
  * generate a vcd identifier for a given facindx
@@ -124,17 +130,12 @@ else
 }
 
 
-int main(int argc, char **argv)
+int process_lxt(char *fname)
 {
 struct lxt2_rd_trace *lt;
+char *netname;
 
-if(argc!=2)
-	{
-	fprintf(stderr, "usage:\n------\n%s filename\n", argv[0]);
-	exit(0);
-	}
-
-lt=lxt2_rd_init(argv[1]);
+lt=lxt2_rd_init(fname);
 if(lt)
 	{
 	int i;
@@ -181,14 +182,23 @@ if(lt)
 		struct lxt2_rd_geometry *g = lxt2_rd_get_fac_geometry(lt, i);
  		lxtint32_t newindx = lxt2_rd_get_alias_root(lt, i);
 
+                if(!flat_earth)
+                        {
+                        netname = output_hier(lxt2_rd_get_facname(lt, i));
+                        }
+                        else
+                        {
+                        netname = lxt2_rd_get_facname(lt, i);
+                        }
+
                 if(g->flags & LXT2_RD_SYM_F_DOUBLE)
                 	{
-                        printf("$var real 1 %s %s $end\n", vcdid(newindx), lxt2_rd_get_facname(lt, i));
+                        printf("$var real 1 %s %s $end\n", vcdid(newindx), netname);
                         }
                 else
                 if(g->flags & LXT2_RD_SYM_F_STRING)
                        {
-                       printf("$var real 1 %s %s $end\n", vcdid(newindx), lxt2_rd_get_facname(lt, i));
+                       printf("$var real 1 %s %s $end\n", vcdid(newindx), netname);
                        }
                 else
 			
@@ -197,20 +207,26 @@ if(lt)
                        		{
                                 if((g->msb!=-1)&&(g->msb!=0))
                                 	{
-                                        printf("$var wire 1 %s %s[%d] $end\n", vcdid(newindx), lxt2_rd_get_facname(lt, i), g->msb);
+                                        printf("$var wire 1 %s %s[%d] $end\n", vcdid(newindx), netname, g->msb);
                                         }
                                         else
                                         {
-                                        printf("$var wire 1 %s %s $end\n", vcdid(newindx), lxt2_rd_get_facname(lt, i));
+                                        printf("$var wire 1 %s %s $end\n", vcdid(newindx), netname);
                                         }  
 				}
                                 else
                                 {
-                                printf("$var wire %d %s %s[%d:%d] $end\n", g->len, vcdid(newindx), lxt2_rd_get_facname(lt, i), g->msb, g->lsb);
+                                printf("$var wire %d %s %s[%d:%d] $end\n", g->len, vcdid(newindx), netname, g->msb, g->lsb);
                                 }
                         }
 		}
 
+	if(!flat_earth)
+		{
+	        output_hier(""); /* flush any remaining hierarchy if not back to toplevel */
+	        free_hier();
+		}
+
 	printf("$enddefinitions $end\n");
 	printf("$dumpvars\n");
 
@@ -228,9 +244,115 @@ if(lt)
 	else
 	{
 	fprintf(stderr, "lxt2_rd_init failed\n");
+	return(255);
 	}
 
+return(0);
+}
+
+/*******************************************************************************/
+
+void print_help(char *nam)
+{
+#ifdef __linux__ 
+printf(
+"Usage: %s [OPTION]... [LXT2FILE]\n\n"
+"  -l, --lxtname=FILE         specify LXT2 input filename\n"
+"  -f, --flatearth            emit flattened hierarchies\n"
+"  -h, --help                 display this help then exit\n\n"
+"VCD is emitted to stdout.\n\n"
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
+#else
+printf(
+"Usage: %s [OPTION]... [LXT2FILE]\n\n"
+"  -l                         specify LXT2 input filename\n"
+"  -f                         emit flattened hierarchies\n"
+"  -h                         display this help then exit\n\n"
+
+"VCD is emitted to stdout.\n\n"
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
+#endif
+
 exit(0);
 }
 
-/****************************************************************************/
+
+int main(int argc, char **argv)
+{
+char opt_errors_encountered=0;
+char *lxname=NULL;
+int c;
+int rc;
+
+while (1)
+        {
+        int option_index = 0;
+                        
+#ifdef __linux__
+        static struct option long_options[] =
+                {
+		{"lxtname", 1, 0, 'l'},
+		{"flatearth", 0, 0, 'f'},
+                {"help", 0, 0, 'h'},
+                {0, 0, 0, 0}  
+                };
+                
+        c = getopt_long (argc, argv, "l:fh", long_options, &option_index);
+#else
+        c = getopt      (argc, argv, "l:fh");
+#endif
+                        
+        if (c == -1) break;     /* no more args */
+                        
+        switch (c)
+                {
+		case 'l':
+			if(lxname) free(lxname);
+                        lxname = malloc(strlen(optarg)+1);
+                        strcpy(lxname, optarg);
+			break;
+
+		case 'f': flat_earth=1;
+			break;
+
+                case 'h':
+			print_help(argv[0]);
+                        break;
+                        
+                case '?':
+                        opt_errors_encountered=1;
+                        break;
+                        
+                default:
+                        /* unreachable */
+                        break;
+                }
+        }
+                        
+if(opt_errors_encountered)
+        {
+        print_help(argv[0]);
+        }
+
+if (optind < argc)
+        {               
+        while (optind < argc)
+                {
+                if(!lxname)
+                        {
+                        lxname = malloc(strlen(argv[optind])+1);
+                        strcpy(lxname, argv[optind++]);
+                        }
+                }
+        }
+                        
+if(!lxname)
+        {
+        print_help(argv[0]);
+        }
+
+rc=process_lxt(lxname);
+free(lxname);
+
+return(rc);
+}
