@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-3 Tony Bybell.
+ * Copyright (c) 2001-5 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,6 +22,55 @@
 
 #include "lxt_write.h"
 
+/*
+ * in-place sort to keep chained facs from migrating...
+ */
+static void wave_mergesort(struct lt_symbol **a, struct lt_symbol **b, int lo, int hi)
+{
+int i, j, k;
+
+if (lo<hi)
+	{
+        int mid=(lo+hi)/2;
+
+        wave_mergesort(a, b, lo, mid);
+        wave_mergesort(a, b, mid+1, hi);
+
+    	i=0; j=lo;
+    	while (j<=mid)
+		{
+        	b[i++]=a[j++];
+		}
+
+    	i=0; k=lo;
+    	while ((k<j)&&(j<=hi))
+		{
+        	if (strcmp(b[i]->name, a[j]->name) <= 0)
+			{
+            		a[k++]=b[i++];
+			}
+			else
+			{
+	            	a[k++]=a[j++];
+			}
+		}
+
+    	while (k<j)
+		{
+        	a[k++]=b[i++];
+		}
+	}
+}
+
+static void wave_msort(struct lt_symbol **a, int num)
+{
+struct lt_symbol **b = malloc(((num/2)+1) * sizeof(struct lt_symbol *));
+
+wave_mergesort(a, b, 0, num-1);
+
+free(b);
+}
+
 /************************ splay ************************/
 
 static int dslxt_success;
@@ -561,26 +610,6 @@ strcpy(lt->compress_fac_str, str);
 }
 
 
-/*
- * emit facs in sorted order along with geometry
- * and sync table info
- */
-static int lt_compare(const void *v1, const void *v2)
-{
-struct lt_symbol *s1 = *(struct lt_symbol **)v1;
-struct lt_symbol *s2 = *(struct lt_symbol **)v2;
-int rc = strcmp(s1->name, s2->name);
-if(rc)
-	{
-	return(rc);
-	}
-	else
-	{
-	return(s1->msb - s2->msb);
-	}
-}
-
-
 static void strip_brack(struct lt_symbol *s)
 {
 char *lastch = s->name+s->namlen - 1;
@@ -616,17 +645,17 @@ if((lt)&&(lt->numfacs))
 			if(lt->do_strip_brackets)
 			for(i=0;i<lt->numfacs;i++)
 				{
-				lt->sorted_facs[i] = s;
+				lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 				strip_brack(s);
 				s=s->symchain;
 				}
 			else	
 			for(i=0;i<lt->numfacs;i++)
 				{
-				lt->sorted_facs[i] = s;
+				lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing*/
 				s=s->symchain;
 				}	
-			qsort((void *)lt->sorted_facs, lt->numfacs, sizeof(struct lt_symbol *), lt_compare);
+			wave_msort(lt->sorted_facs, lt->numfacs);
 
 			for(i=0;i<lt->numfacs;i++)
 				{
@@ -655,8 +684,8 @@ if((lt)&&(lt->numfacs))
 		lt->zfacname_predec_size = lt->zpackcount;
 	
 		gzclose(lt->zhandle);
-		fseek(lt->handle, 0L, SEEK_END);
-		lt->position=ftell(lt->handle);
+		fseeko(lt->handle, 0L, SEEK_END);
+		lt->position=ftello(lt->handle);
 		lt->zfacname_size = lt->position - lt->zfacname_size;
 
 		lt->zhandle = gzdopen(dup(fileno(lt->handle)), "wb9");
@@ -681,8 +710,8 @@ if((lt)&&(lt->numfacs))
 			}
 
 		gzclose(lt->zhandle);
-		fseek(lt->handle, 0L, SEEK_END);
-		lt->position=ftell(lt->handle);
+		fseeko(lt->handle, 0L, SEEK_END);
+		lt->position=ftello(lt->handle);
 		lt->zfacgeometry_size = lt->position - lt->facgeometry_offset;
 
 		if(is_interlaced_trace)
@@ -696,8 +725,8 @@ if((lt)&&(lt->numfacs))
 				}
 	
 			gzclose(lt->zhandle); lt->zhandle = NULL;
-			fseek(lt->handle, 0L, SEEK_END);
-			lt->position=ftell(lt->handle);
+			fseeko(lt->handle, 0L, SEEK_END);
+			lt->position=ftello(lt->handle);
 			lt->zsync_table_size = lt->position - lt->sync_table_offset;
 			}
 		}
@@ -706,7 +735,7 @@ if((lt)&&(lt->numfacs))
 
 
 /* 
- * initialize the trace and get back and lt context
+ * initialize the trace and get back an lt context
  */
 struct lt_trace *lt_init(const char *name)
 {
@@ -1002,8 +1031,8 @@ for(i=0;i<lt->num_dict_entries;i++)
 	}
 	
 gzclose(lt->zhandle);
-fseek(lt->handle, 0L, SEEK_END);
-lt->position=ftell(lt->handle);
+fseeko(lt->handle, 0L, SEEK_END);
+lt->position=ftello(lt->handle);
 lt->zdictionary_size = lt->position - lt->zdictionary_size;
 
 free(lt->sorted_dict); lt->sorted_dict = NULL;
@@ -1057,8 +1086,8 @@ if(lt)
 			case LT_ZMODE_BZIP2:	BZ2_bzclose(lt->zhandle); break;
 			}
 		lt->zhandle = NULL;
-		fseek(lt->handle, 0L, SEEK_END);
-		lt->position=ftell(lt->handle);
+		fseeko(lt->handle, 0L, SEEK_END);
+		lt->position=ftello(lt->handle);
 
 		lt_set_zmode(lt, LT_ZMODE_NONE);
 		lt->zchg_table_size = lt->position - lt->change_field_offset;
@@ -1132,8 +1161,8 @@ if(lt)
 			}
 
 		gzclose(lt->zhandle); lt->zhandle = NULL;
-		fseek(lt->handle, 0L, SEEK_END);
-		lt->position=ftell(lt->handle);
+		fseeko(lt->handle, 0L, SEEK_END);
+		lt->position=ftello(lt->handle);
 		lt->ztime_table_size = lt->position - lt->ztime_table_size;
 		}
 
@@ -1467,17 +1496,17 @@ if((lt)&&(!lt->emitted)&&(!lt->sorted_facs))
 		if(lt->do_strip_brackets)
 		for(i=0;i<lt->numfacs;i++)
 			{
-			lt->sorted_facs[i] = s;
+			lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 			strip_brack(s);
 			s=s->symchain;
 			}
 		else	
 		for(i=0;i<lt->numfacs;i++)
 			{
-			lt->sorted_facs[i] = s;
+			lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 			s=s->symchain;
 			}	
-		qsort((void *)lt->sorted_facs, lt->numfacs, sizeof(struct lt_symbol *), lt_compare);
+		wave_msort(lt->sorted_facs, lt->numfacs);
 
 		for(i=0;i<lt->numfacs;i++)
 			{
@@ -1609,7 +1638,7 @@ int i;
 
 len--;
 
-for(i=0;i<len;i++)
+for(i=0;i<=len;i++)
 	{
 	*(p++) = '0' | ((value & (1<<(len-i)))!=0);
 	}
@@ -1642,12 +1671,14 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 	if((len>1)&&(len<=32))
 		{
 		int ivalue = value;
+		int delta1, delta2;
 		s->clk_mask <<= 1;
 		s->clk_mask |= 1;
 
 		if(     ((s->clk_mask&0x1f)==0x1f) && 
-			( ((ivalue         - s->clk_prevval1) & lt_optimask[s->len]) == ((s->clk_prevval1 - s->clk_prevval3) & lt_optimask[s->len]) ) &&
-			( ((s->clk_prevval - s->clk_prevval2) & lt_optimask[s->len]) == ((s->clk_prevval2 - s->clk_prevval4) & lt_optimask[s->len]) )
+			( (delta1=(ivalue         - s->clk_prevval1) & lt_optimask[s->len]) == ((s->clk_prevval1 - s->clk_prevval3) & lt_optimask[s->len]) ) &&
+			( (delta2=(s->clk_prevval - s->clk_prevval2) & lt_optimask[s->len]) == ((s->clk_prevval2 - s->clk_prevval4) & lt_optimask[s->len]) ) &&
+			( (delta1==delta2) || ((!delta1)&&(!delta2)) )
 			)
 			{
 			if(s->clk_prevtrans==ULLDescriptor(~0))
@@ -2118,7 +2149,7 @@ if((s->flags)&LT_SYM_F_STRING)
 	int numbytes;				/* number of bytes to store value minus one */
 	unsigned int last_change_delta = lt->position - s->last_change - 2;
 
-	if(lt->numfacs_bytes)
+	if(!lt->numfacs_bytes)
 		{
 		if(last_change_delta >= 256*65536)
 			{
@@ -2249,6 +2280,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 		int ivalue = 0;
 		int i;
 		char *pnt = value;
+		int delta1, delta2;
 
 		for(i=0;i<len;i++)
 			{
@@ -2274,8 +2306,9 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 		s->clk_mask |= legal;
 
 		if(     ((s->clk_mask&0x1f)==0x1f) && 
-			( ((ivalue         - s->clk_prevval1) & lt_optimask[s->len]) == ((s->clk_prevval1 - s->clk_prevval3) & lt_optimask[s->len]) ) &&
-			( ((s->clk_prevval - s->clk_prevval2) & lt_optimask[s->len]) == ((s->clk_prevval2 - s->clk_prevval4) & lt_optimask[s->len]) )
+			( (delta1=(ivalue         - s->clk_prevval1) & lt_optimask[s->len]) == ((s->clk_prevval1 - s->clk_prevval3) & lt_optimask[s->len]) ) &&
+			( (delta2=(s->clk_prevval - s->clk_prevval2) & lt_optimask[s->len]) == ((s->clk_prevval2 - s->clk_prevval4) & lt_optimask[s->len]) ) &&
+			( (delta1==delta2) || ((!delta1)&&(!delta2)) )
 			)
 			{
 			if(s->clk_prevtrans==ULLDescriptor(~0))
