@@ -139,15 +139,15 @@ facs[indx].ltsym=lt_symbol_add(lt, name, facs[indx].array_height-1, facs[indx].m
  * find the most recent valuechange that corresponds with
  * an arbitrary timevalue.
  */
-static int max_compare_time_tc, max_compare_pos_tc;
+static long max_compare_time_tc, max_compare_pos_tc;
 static int compar_timechain(const void *s1, const void *s2)
 {
-int key, obj, delta;
-int cpos;
+long key, obj, delta;
+long cpos;
 int rv;
 
-key=(int)s1;
-obj=get_32(cpos=(int)s2);
+key=(long)s1;
+obj=get_32(cpos=(long)s2);
 
 if((obj<=key)&&(obj>max_compare_time_tc))
         {
@@ -167,7 +167,7 @@ static int bsearch_timechain(int key)
 {
 max_compare_time_tc=-1; max_compare_pos_tc=-1; 
 
-bsearch((void *)key, (void *)tableposition, total_cycles, 8, compar_timechain);
+bsearch((void *)(long)key, (void *)(long)tableposition, total_cycles, 8, compar_timechain);
 if((!max_compare_pos_tc)||(max_compare_time_tc<0)) 
         {
         max_compare_pos_tc=tableposition; /* aix bsearch fix */
