@@ -31,8 +31,8 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <getopt.h>
 #include "v2l_debug_lxt2.h"
-#include "lxt2_write.h"
 
 #ifndef _MSC_VER
 #include <unistd.h>
@@ -110,7 +110,7 @@ int len;
 struct vcdsymbol
 {
 struct vcdsymbol *next;
-struct lxt2_wr_symbol *ltsym;
+void *ltsym;
 char *name;
 char *id;
 unsigned int nid;
