@@ -219,6 +219,55 @@ v2=*((struct vcdsymbol **)s2);
 return(strcmp(v1->id, v2->id));
 }
 
+/*
+ * alias vs normal symbol adding
+ */
+static void alias_vs_normal_symadd(struct vcdsymbol *v, struct vcdsymbol *root_v)
+{
+if(!root_v) 
+	{
+	if((v->vartype==V_INTEGER)||(v->vartype==V_REAL))
+		{
+		v->ltsym = lt_symbol_add(lt, v->name, 0, v->msi, v->lsi, (v->vartype==V_INTEGER)?LT_SYM_F_INTEGER:((v->vartype==V_REAL)?LT_SYM_F_DOUBLE:LT_SYM_F_BITS));
+		}
+		else
+		{
+		char buf[65537];
+		if(v->msi==v->lsi)
+			{
+			sprintf(buf, "%s[%d]", v->name, v->msi);
+			}
+			else
+			{
+			sprintf(buf, "%s[%d:%d]", v->name, v->msi, v->lsi);
+			}
+		v->ltsym = lt_symbol_add(lt, buf, 0, v->msi, v->lsi, (v->vartype==V_INTEGER)?LT_SYM_F_INTEGER:((v->vartype==V_REAL)?LT_SYM_F_DOUBLE:LT_SYM_F_BITS));
+		}
+	}
+	else
+	{
+	if((v->vartype==V_INTEGER)||(v->vartype==V_REAL))
+		{
+		lt_symbol_alias(lt, root_v->name, v->name, v->msi, v->lsi);
+		}
+		else
+		{
+		char bufold[65537], buf[65537];
+		if(v->msi==v->lsi)
+			{
+			sprintf(bufold, "%s[%d]", root_v->name, root_v->msi);
+			sprintf(buf, "%s[%d]", v->name, v->msi);
+			}
+			else
+			{
+			sprintf(bufold, "%s[%d:%d]", root_v->name, root_v->msi, root_v->lsi);
+			sprintf(buf, "%s[%d:%d]", v->name, v->msi, v->lsi);
+			}
+		lt_symbol_alias(lt, bufold, buf, v->msi, v->lsi);
+		}
+	}
+}
+
 
 /*
  * create sorted (by id) table
@@ -228,37 +277,51 @@ static void create_sorted_table(void)
 struct vcdsymbol *v;
 struct vcdsymbol **pnt;
 unsigned int vcd_distance;
-
+struct vcdsymbol *root_v;
+int i;
+                
 if(numsyms)
-	{
-	vcd_distance = vcd_maxid - vcd_minid + 1;
+        {
+        vcd_distance = vcd_maxid - vcd_minid + 1;
+                         
+        if(vcd_distance <= 4 * 1024 * 1024)
+                {
+                indexed = (struct vcdsymbol **)calloc_2(vcd_distance, sizeof(struct vcdsymbol *));
+                        
+                printf("%d symbols span ID range of %d, using indexing...\n", numsyms, vcd_distance);
+                        
+                v=vcdsymroot;
+                while(v) 
+                        {
+                        if(!(root_v=indexed[v->nid - vcd_minid]))
+                                {
+                                indexed[v->nid - vcd_minid] = v;
+                                }
+                        alias_vs_normal_symadd(v, root_v);
 
-	if(vcd_distance <= 2 * 1024 * 1024)
-		{
-		indexed = (struct vcdsymbol **)calloc_2(vcd_distance, sizeof(struct vcdsymbol *));
-
-		printf("%d symbols span ID range of %d, using indexing...\n", numsyms, vcd_distance);
-
-		v=vcdsymroot;
-		while(v)
-			{
-			if(!indexed[v->nid - vcd_minid]) indexed[v->nid - vcd_minid] = v;
-			v=v->next;
-			}
-		}
-		else
-		{
-		pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
-		v=vcdsymroot;
-		while(v)
-			{
-			*(pnt++)=v;
-			v=v->next;
-			}
-	
-		qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
-		}
-	}
+                        v=v->next;
+                        }
+                }
+                else
+                {
+                pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
+                v=vcdsymroot;
+                while(v)
+                        {
+                        *(pnt++)=v;
+                        v=v->next;
+                        }
+                
+                qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
+                 
+                root_v = NULL;
+                for(i=0;i<numsyms;i++)
+                        {
+                        if(sorted[i] != root_v) root_v = NULL;
+                        alias_vs_normal_symadd(sorted[i], root_v);
+                        }
+                }
+        }
 }
 
 /******************************************************************/
@@ -628,10 +691,13 @@ switch(yytext[0])
 	{
 	case '0':
 	case '1':
-	case 'x':
-	case 'X':
-	case 'z':
-	case 'Z':
+	case 'x': case 'X':
+	case 'z': case 'Z':
+	case 'h': case 'H':
+	case 'u': case 'U':
+	case 'w': case 'W':
+	case 'l': case 'L':
+	case '-':
 		if(yylen>1)
 			{
 			v=bsearch_vcd(yytext+1, yylen-1);	
@@ -825,9 +891,12 @@ switch(yytext[0])
 		double *d;
 
 		d=malloc_2(sizeof(double));
+		*d = 0;
 		sscanf(yytext+1,"%lg",d);
+		errno = 0;
 		
 		get_strtoken();
+
 		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
@@ -1204,6 +1273,7 @@ for(;;)
 				}
 			numsyms++;
 
+#if 0
 			if((v->vartype==V_INTEGER)||(v->vartype==V_REAL))
 				{
 				v->ltsym = lt_symbol_add(lt, v->name, 0, v->msi, v->lsi, (v->vartype==V_INTEGER)?LT_SYM_F_INTEGER:((v->vartype==V_REAL)?LT_SYM_F_DOUBLE:LT_SYM_F_BITS));
@@ -1221,6 +1291,7 @@ for(;;)
 					}
 				v->ltsym = lt_symbol_add(lt, buf, 0, v->msi, v->lsi, (v->vartype==V_INTEGER)?LT_SYM_F_INTEGER:((v->vartype==V_REAL)?LT_SYM_F_DOUBLE:LT_SYM_F_BITS));
 				}
+#endif
 
 			DEBUG(fprintf(stderr,"VAR %s %d %s %s[%d:%d]\n",
 				vartypes[v->vartype], v->size, v->id, v->name, 
