@@ -30,6 +30,7 @@
  */
 
 #include "v2l_analyzer_lxt2.h"
+#include "lxt2_write.h"
 
 #undef VCD_BSEARCH_IS_PERFECT           /* bsearch is imperfect under linux, but OK under AIX */
 
@@ -38,6 +39,12 @@ struct lxt2_wr_trace *lt=NULL;
 int numfacs=0;
 int deadcnt=0;
 
+int opt_depth = 4;
+unsigned long long opt_break_size = 0;
+int opt_partial_mode = -1;
+int opt_checkpoint_disable = 0;
+int opt_maxgranule = 8;
+
 struct symbol **sym=NULL;
 struct symbol **facs=NULL;
 struct symbol *firstnode=NULL;
@@ -688,12 +695,15 @@ int vlen;
 
 switch(yytext[0])
 	{
-	case '0':
-	case '1':
-	case 'x':
-	case 'X':
-	case 'z':
-	case 'Z':
+        case '0':
+        case '1':
+        case 'x': case 'X':
+        case 'z': case 'Z':
+        case 'h': case 'H':
+        case 'u': case 'U':
+        case 'w': case 'W':
+        case 'l': case 'L':
+        case '-':
 		if(yylen>1)
 			{
 			v=bsearch_vcd(yytext+1, yylen-1);	
@@ -887,7 +897,9 @@ switch(yytext[0])
 		double *d;
 
 		d=malloc_2(sizeof(double));
+		*d = 0;
 		sscanf(yytext+1,"%lg",d);
+		errno = 0;
 		
 		get_strtoken();
 		v=bsearch_vcd(yytext, yylen);	
@@ -1533,10 +1545,19 @@ if(!lt)
 	exit(255);
 	}
 
-/* lxt2_wr_set_partial_on(lt, 1); */
-/* lxt2_wr_set_checkpoint_off(lt); */
-lxt2_wr_set_compression_depth(lt, 9);
+if(opt_partial_mode>=0)
+	{
+	lxt2_wr_set_partial_on(lt, opt_partial_mode);
+	}
 
+if((opt_checkpoint_disable)&&(!opt_break_size))
+	{
+	lxt2_wr_set_checkpoint_off(lt);
+	}
+
+lxt2_wr_set_compression_depth(lt, opt_depth);
+lxt2_wr_set_break_size(lt, (off_t)opt_break_size);
+lxt2_wr_set_maxgranule(lt, opt_maxgranule);
 lxt2_wr_symbol_bracket_stripping(lt, 1);	/* this is intentional */
 
 sym=(struct symbol **)calloc_2(SYMPRIME,sizeof(struct symbol *));
@@ -1765,17 +1786,161 @@ if(p<r)
 
 /*******************************************************************************/
 
+void print_help(char *nam)
+{
+#ifdef __linux__ 
+printf(
+"Usage: %s [OPTION]... [VCDFILE] [LXT2FILE]\n\n"
+"  -v, --vcdname=FILE         specify VCD input filename\n"
+"  -l, --lxtname=FILE         specify LXT2 output filename\n"
+"  -d, --depth=value          specify 0..9 compression depth (default = 4)\n"
+"  -m, --maxgranule=value     specify number of granules per section (def = 8)\n"
+"  -b, --break=value          specify break size (default = 0 = off)\n"
+"  -p, --partialmode=mode     specify partial zip mode 0=monolithic/1=separate\n"
+"  -c, --checkpoint=mode      specify checkpoint mode (0 = on [def], 1 = off)\n"
+"  -h, --help                 display this help then exit\n\n"
+
+"VCD files may be compressed with zip or gzip.  Note that VCDFILE and LXTFILE\n"
+"are optional provided the --vcdname and --lxtname options are specified.\n"
+"Use \"-\" as a VCD filename to accept uncompressed input from stdin.\n\n"
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
+#else
+printf(
+"Usage: %s [OPTION]... [VCDFILE] [LXT2FILE]\n\n"
+"  -v FILE                    specify VCD input filename\n"
+"  -l FILE                    specify LXT2 output filename\n"
+"  -d value                   specify 0..9 compression depth (default = 4)\n"
+"  -m value                   specify number of granules per section (def = 8)\n"
+"  -b value                   specify break size (default = 0 = off)\n"
+"  -p mode                    specify partial zip mode 0=monolithic/1=separate\n"
+"  -c mode                    specify checkpoint mode (0 = on [def], 1 = off)\n"
+"  -h                         display this help then exit\n\n"
+
+"VCD files may be compressed with zip or gzip.  Note that VCDFILE and LXTFILE\n"
+"are optional provided the --vcdname and --lxtname options are specified.\n"
+"Use \"-\" as a VCD filename to accept uncompressed input from stdin.\n\n"
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
+#endif
+
+exit(0);
+}
+
+
 int main(int argc, char **argv)
 {
-if(argc==3)
+char opt_errors_encountered=0;
+char *vname=NULL, *lxname=NULL;
+int c;
+
+while (1)
         {
-        vcd_main(argv[1], argv[2]);
+        int option_index = 0;
+                        
+#ifdef __linux__
+        static struct option long_options[] =
+                {
+		{"vcdname", 1, 0, 'v'},
+		{"lxtname", 1, 0, 'l'},
+		{"depth", 1, 0, 'd'},
+		{"maxgranule", 1, 0, 'm'},
+		{"break", 1, 0, 'b'},
+		{"partialmode", 1, 0, 'p'},
+		{"checkpoint", 1, 0, 'c'},
+                {"help", 0, 0, 'h'},
+                {0, 0, 0, 0}  
+                };
+                
+        c = getopt_long (argc, argv, "v:l:d:m:b:p:c:h", long_options, &option_index);
+#else
+        c = getopt      (argc, argv, "v:l:d:m:b:p:c:h");
+#endif
+                        
+        if (c == -1) break;     /* no more args */
+                        
+        switch (c)
+                {
+		case 'v':
+			if(vname) free(vname);
+                        vname = malloc_2(strlen(optarg)+1);
+                        strcpy(vname, optarg);
+			break;
+
+		case 'l':
+			if(lxname) free(lxname);
+                        lxname = malloc_2(strlen(optarg)+1);
+                        strcpy(lxname, optarg);
+			break;
+
+		case 'd':
+			opt_depth = atoi(optarg);
+			if(opt_depth<0) opt_depth = 0;
+			if(opt_depth>9) opt_depth = 9;
+			break;
+
+		case 'm':
+			opt_maxgranule = atoi(optarg);
+			if(opt_maxgranule<1) opt_maxgranule=1;
+			break;
+
+		case 'b':
+			sscanf(optarg, "%lld", &opt_break_size);
+			errno = 0;
+			break;
+
+		case 'p':
+			opt_partial_mode = atoi(optarg);
+			if(opt_depth<0) opt_partial_mode = 0;
+			if(opt_depth>1) opt_partial_mode = 1;
+			break;
+
+		case 'c':
+			opt_checkpoint_disable = atoi(optarg);
+			break;
+
+                case 'h':
+			print_help(argv[0]);
+                        break;
+                        
+                case '?':
+                        opt_errors_encountered=1;
+                        break;
+                        
+                default:
+                        /* unreachable */
+                        break;
+                }
         }
-        else
+                        
+if(opt_errors_encountered)
         {
-        printf("Usage:\n------\n%s filename.vcd[.gz] filename.lxt\n"
-		"Use \"-\" as a filename to accept uncompressed input from stdin.\n",argv[0]);
+        print_help(argv[0]);
         }
 
+if (optind < argc)
+        {               
+        while (optind < argc)
+                {
+                if(!vname)
+                        {
+                        vname = malloc_2(strlen(argv[optind])+1);
+                        strcpy(vname, argv[optind++]);
+                        } 
+                else if(!lxname)
+                        {
+                        lxname = malloc_2(strlen(argv[optind])+1);
+                        strcpy(lxname, argv[optind++]);
+                        }
+                }
+        }
+                        
+if((!vname)||(!lxname))
+        {
+        print_help(argv[0]);
+        }
+
+vcd_main(vname, lxname);
+
+free(vname); free(lxname);
+
 return(0);
 }
