@@ -8,6 +8,7 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "analyzer.h"
 #include "aet.h"
 #include "lx2.h"
@@ -238,7 +239,7 @@ if((tc=treechain))
 
 static void enter_callback_e(GtkWidget *widget, GtkWidget *nothing)
 {
-  gchar *entry_text;
+  G_CONST_RETURN gchar *entry_text;
   int len;
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
   DEBUG(printf("Entry contents: %s\n", entry_text));
@@ -628,7 +629,7 @@ while(tfirst)
         {
         tfirst->flags |= TR_HIGHLIGHT;
         if(tfirst==tlast) break;
-        tfirst=tfirst->next;
+        tfirst=tfirst->t_next;
         }
         
 MaxSignalLength();
@@ -686,7 +687,7 @@ bundle_callback_generic(void)
 {
 if(!autoname_bundles)
 	{
-	entrybox_local("Enter Bundle Name",300,"",128,bundle_cleanup);
+	entrybox_local("Enter Bundle Name",300,"",128,GTK_SIGNAL_FUNC(bundle_cleanup));
 	}
 	else
 	{
