@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2003.
+ * Copyright (c) Tony Bybell 2003-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include "lx2.h"
 
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 
 #include <fcntl.h>
 #include <string.h>
@@ -18,26 +20,20 @@
 #include "aet.h"
 #include "vcd.h"
 #include "lxt.h"
+#include "vzt.h"
 #include "lxt2_read.h"
 #include "debug.h"
 
+
 /*
  * globals
  */
-char is_lx2 = 0;
+unsigned char is_lx2 = LXT2_IS_INACTIVE;
 
 static struct lxt2_rd_trace *lx2=NULL;
 static TimeType first_cycle, last_cycle, total_cycles;
-static TimeType time_scale=1;   /* multiplier is 1, 10, 100 */
 
-struct lx2_entry
-{
-struct HistEnt *histent_head, *histent_curr;
-int numtrans;
-nptr np;
-};
-
-struct lx2_entry *lx2_table = NULL;
+static struct lx2_entry *lx2_table = NULL;
 static struct fac *mvlfacs=NULL;
 
 
@@ -50,6 +46,7 @@ int i;
 struct Node *n;
 struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
 signed char scale;
+unsigned int numalias = 0;
 
 lx2 = lxt2_rd_init(fname);
 if(!lx2)
@@ -58,48 +55,19 @@ if(!lx2)
         exit(0);
         }
 
+/* SPLASH */                            splash_create();
+
 /* lxt2_rd_set_max_block_mem_usage(lx2, 0); */
 
 scale=(signed char)lxt2_rd_get_timescale(lx2);
-switch(scale)
-	{
-        case 0:         time_dimension = 's'; break;
-                
-        case -1:        time_scale = LLDescriptor(100); time_dimension = 'm'; break;
-        case -2:        time_scale = LLDescriptor(10);
-        case -3:                                        time_dimension = 'm'; break;
-
-        case -4:        time_scale = LLDescriptor(100); time_dimension = 'u'; break;
-        case -5:        time_scale = LLDescriptor(10);
-        case -6:                                        time_dimension = 'u'; break;
-
-        case -10:       time_scale = LLDescriptor(100); time_dimension = 'p'; break;
-        case -11:       time_scale = LLDescriptor(10);
-        case -12:                                       time_dimension = 'p'; break;
-        
-        case -13:       time_scale = LLDescriptor(100); time_dimension = 'f'; break;
-        case -14:       time_scale = LLDescriptor(10);
-        case -15:                                       time_dimension = 'f'; break;
-        
-        case -7:        time_scale = LLDescriptor(100); time_dimension = 'n'; break;
-        case -8:        time_scale = LLDescriptor(10);
-        case -9:
-        default:                                        time_dimension = 'n'; break;
-        }
-
+exponent_to_time_scale(scale);
 
 numfacs=lxt2_rd_get_num_facs(lx2);
-mvlfacs=(struct fac *)calloc(numfacs,sizeof(struct fac));
-lx2_table=(struct lx2_entry *)calloc(numfacs, sizeof(struct lx2_entry));
+mvlfacs=(struct fac *)calloc_2(numfacs,sizeof(struct fac));
+lx2_table=(struct lx2_entry *)calloc_2(numfacs, sizeof(struct lx2_entry));
 
 for(i=0;i<numfacs;i++)
 	{
-	char *fnam = lxt2_rd_get_facname(lx2, i);
-	int flen = strlen(fnam);
-
-	mvlfacs[i].name=malloc_2(flen+1);
-	strcpy(mvlfacs[i].name, fnam);
-
 	mvlfacs[i].array_height=lxt2_rd_get_fac_rows(lx2, i);
 	mvlfacs[i].msb=lxt2_rd_get_fac_msb(lx2, i);
 	mvlfacs[i].lsb=lxt2_rd_get_fac_lsb(lx2, i);
@@ -108,9 +76,10 @@ for(i=0;i<numfacs;i++)
 	}
 
 fprintf(stderr, LXT2_RDLOAD"Finished building %d facs.\n", numfacs);
+/* SPLASH */                            splash_sync(1, 5);
 
-first_cycle = (TimeType) lxt2_rd_get_start_time(lx2);
-last_cycle = (TimeType) lxt2_rd_get_end_time(lx2);
+first_cycle = (TimeType) lxt2_rd_get_start_time(lx2) * time_scale;
+last_cycle = (TimeType) lxt2_rd_get_end_time(lx2) * time_scale;
 total_cycles = last_cycle - first_cycle + 1;
 
 /* do your stuff here..all useful info has been initialized by now */
@@ -120,12 +89,36 @@ if(!hier_was_explicitly_set)    /* set default hierarchy split char */
         hier_delimeter='.';
         }
 
+if(numfacs)
+	{
+	char *fnam = lxt2_rd_get_facname(lx2, 0);
+	int flen = strlen(fnam);
+
+	mvlfacs[0].name=malloc_2(flen+1);
+	strcpy(mvlfacs[0].name, fnam);
+	}
+
 for(i=0;i<numfacs;i++)
         {
-	char buf[4096];
+	char buf[65537];
 	char *str;	
 	struct fac *f;
 
+	if(i!=(numfacs-1))
+		{
+		char *fnam = lxt2_rd_get_facname(lx2, i+1);
+		int flen = strlen(fnam);
+
+		mvlfacs[i+1].name=malloc_2(flen+1);
+		strcpy(mvlfacs[i+1].name, fnam);
+		}
+
+	if(i>1)
+		{
+		free_2(mvlfacs[i-2].name);
+		mvlfacs[i-2].name = NULL;
+		}
+
 	if(mvlfacs[i].flags&LXT2_RD_SYM_F_ALIAS)
 		{
 		int alias = mvlfacs[i].array_height;
@@ -135,6 +128,8 @@ for(i=0;i<numfacs;i++)
 			{
 			f=mvlfacs+f->array_height;
 			}
+
+		numalias++;
 		}
 		else
 		{
@@ -153,13 +148,15 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		prevsymroot = prevsym = NULL;
 		}
-	else if ( (f->len==1)&&(!(f->flags&(LXT2_RD_SYM_F_INTEGER|LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING)))&&
-			((i!=numfacs-1)&&(!strcmp(mvlfacs[i].name, mvlfacs[i+1].name))) ||
-			((i!=0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name))) &&
-			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1)
+	else if ( 
+			((f->len==1)&&(!(f->flags&(LXT2_RD_SYM_F_INTEGER|LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING)))&&
+			((i!=numfacs-1)&&(!strcmp(mvlfacs[i].name, mvlfacs[i+1].name))))
+			||
+			(((i!=0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name))) &&
+			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1))
 		)
 		{
 		sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
@@ -172,7 +169,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		if((prevsym)&&(i>0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name)))	/* allow chaining for search functions.. */
 			{
 			prevsym->vec_root = prevsymroot;
@@ -196,7 +193,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, mvlfacs[i].name, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		prevsymroot = prevsym = NULL;
 
 		if(f->flags&LXT2_RD_SYM_F_INTEGER)
@@ -231,55 +228,142 @@ for(i=0;i<numfacs;i++)
 		}
                  
         n->head.time=-1;        /* mark 1st node as negative time */
-        n->head.v.val=1;
+        n->head.v.h_val=AN_X;
         s->n=n;
         }
 
+for(i=0;((i<2)&&(i<numfacs));i++)
+	{
+	if(mvlfacs[i].name)
+		{
+		free_2(mvlfacs[i].name);
+		mvlfacs[i].name = NULL;
+		}
+	}
+
+
+/* SPLASH */                            splash_sync(2, 5);
 facs=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
-curnode=firstnode;
-for(i=0;i<numfacs;i++)
-	{
-	char *subst, ch;
-	int len;
 
-	facs[i]=curnode;
-        if((len=strlen(subst=curnode->name))>longestname) longestname=len;
-	curnode=curnode->nextinaet;
-	while((ch=(*subst)))
-		{	
-		if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
-		subst++;
+if(fast_tree_sort)
+        {
+        curnode=firstnode;
+        for(i=0;i<numfacs;i++)
+                {
+                int len;
+                facs[i]=curnode;
+                if((len=strlen(curnode->name))>longestname) longestname=len;
+                curnode=curnode->nextinaet;
+                }
+
+	if(numalias)
+		{
+		unsigned int idx_lft = 0;
+		unsigned int idx_lftmax = numfacs - numalias;  		
+		unsigned int idx_rgh = numfacs - numalias;  		
+		struct symbol **facs_merge=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
+
+		fprintf(stderr, LXT2_RDLOAD"Merging in %d aliases.\n", numalias);
+
+		for(i=0;i<numfacs;i++)	/* fix possible tail appended aliases by remerging in partial one pass merge sort */
+			{
+			if(strcmp(facs[idx_lft]->name, facs[idx_rgh]->name) <= 0)
+				{
+				facs_merge[i] = facs[idx_lft++];
+
+				if(idx_lft == idx_lftmax)
+					{
+					for(i++;i<numfacs;i++)
+						{
+						facs_merge[i] = facs[idx_rgh++];
+						}
+					}
+				}
+				else
+				{
+				facs_merge[i] = facs[idx_rgh++];
+
+				if(idx_rgh == numfacs)
+					{
+					for(i++;i<numfacs;i++)
+						{
+						facs_merge[i] = facs[idx_lft++];
+						}
+					}
+				}
+			}
+
+		free_2(facs); facs = facs_merge;
 		}
-	}
+                 
+/* SPLASH */                            splash_sync(3, 5);
+        fprintf(stderr, LXT2_RDLOAD"Building facility hierarchy tree.\n");
+        
+        init_tree();
+        for(i=0;i<numfacs;i++)
+                {
+                build_tree_from_name(facs[i]->name, i);
+                }
+/* SPLASH */                            splash_sync(4, 5);
+        treeprune(treeroot);
+        
+        fprintf(stderr, LXT2_RDLOAD"Sorting facility hierarchy tree.\n");
+        treesort(treeroot, NULL);
+/* SPLASH */                            splash_sync(5, 5);
+        order_facs_from_treesort(treeroot, &facs);
+                 
+        facs_are_sorted=1;
+        }
+        else
+	{
+	curnode=firstnode;
+	for(i=0;i<numfacs;i++)
+		{
+		char *subst, ch;
+		int len;
 
-fprintf(stderr, LXT2_RDLOAD"Sorting facilities at hierarchy boundaries.\n");
-wave_heapsort(facs,numfacs);
+		facs[i]=curnode;
+	        if((len=strlen(subst=curnode->name))>longestname) longestname=len;
+		curnode=curnode->nextinaet;
+		while((ch=(*subst)))
+			{	
+			if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
+			subst++;
+			}
+		}
+
+/* SPLASH */                            splash_sync(3, 5);
+	fprintf(stderr, LXT2_RDLOAD"Sorting facilities at hierarchy boundaries.\n");
+	wave_heapsort(facs,numfacs);
 	
-for(i=0;i<numfacs;i++)
-	{
-	char *subst, ch;
-
-	subst=facs[i]->name;
-	while((ch=(*subst)))
-		{	
-		if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
-		subst++;
+	for(i=0;i<numfacs;i++)
+		{
+		char *subst, ch;
+	
+		subst=facs[i]->name;
+		while((ch=(*subst)))
+			{	
+			if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
+			subst++;
+			}
 		}
+
+	facs_are_sorted=1;
+
+/* SPLASH */                            splash_sync(4, 5);
+	fprintf(stderr, LXT2_RDLOAD"Building facility hierarchy tree.\n");
+
+	init_tree();		
+	for(i=0;i<numfacs;i++)	
+		{
+		build_tree_from_name(facs[i]->name, i);
+		}
+/* SPLASH */                            splash_sync(5, 5);
+	treeprune(treeroot);
 	}
 
-facs_are_sorted=1;
-
-fprintf(stderr, LXT2_RDLOAD"Building facility hierarchy tree.\n");
-
-init_tree();		
-for(i=0;i<numfacs;i++)	
-{
-build_tree_from_name(facs[i]->name, i);
-}
-treeprune(treeroot);
-
 min_time = first_cycle; max_time=last_cycle;
-is_lx2 = ~0;
+is_lx2 = LXT2_IS_LXT2;
 
 if(skip_start || skip_end)
 	{
@@ -320,7 +404,7 @@ return(max_time);
 /*
  * lx2 callback (only does bits for now)
  */
-void lx2_callback(struct lxt2_rd_trace **lt, lxtint64_t *time, lxtint32_t *facidx, char **value)
+static void lx2_callback(struct lxt2_rd_trace **lt, lxtint64_t *time, lxtint32_t *facidx, char **value)
 {
 struct HistEnt *htemp = histent_calloc();
 struct lx2_entry *l2e = lx2_table+(*facidx);
@@ -332,18 +416,18 @@ if(!(f->flags&(LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING)))
 	{
 	if(f->len>1)        
 	        {
-	        htemp->v.vector = (char *)malloc_2(f->len);
-		memcpy(htemp->v.vector, *value, f->len);
+	        htemp->v.h_vector = (char *)malloc_2(f->len);
+		memcpy(htemp->v.h_vector, *value, f->len);
 	        }
 	        else
 	        {
 		switch(**value)
 			{
-			case '0':	htemp->v.val = 0; break;
-			case '1':	htemp->v.val = 3; break;
+			case '0':	htemp->v.h_val = AN_0; break;
+			case '1':	htemp->v.h_val = AN_1; break;
 			case 'Z':
-			case 'z':	htemp->v.val = 2; break;
-			default:	htemp->v.val = 1; break;
+			case 'z':	htemp->v.h_val = AN_Z; break;
+			default:	htemp->v.h_val = AN_X; break;
 			}
 	        }
 	}
@@ -351,14 +435,14 @@ else if(f->flags&LXT2_RD_SYM_F_DOUBLE)
 	{
 	double *d = malloc_2(sizeof(double));
 	sscanf(*value, "%lg", d);
-	htemp->v.vector = (char *)d;
+	htemp->v.h_vector = (char *)d;
 	htemp->flags = HIST_REAL;
 	}
 else	/* string */
 	{
 	char *s = malloc_2(strlen(*value)+1);
 	strcpy(s, *value);
-	htemp->v.vector = s;
+	htemp->v.h_vector = s;
 	htemp->flags = HIST_REAL|HIST_STRING;
 	}
 
@@ -400,11 +484,20 @@ np->mvlfac=NULL;
 void import_lx2_trace(nptr np)
 {
 struct HistEnt *htemp, *histent_tail;
-int len, i, j;
+int len, i;
 struct fac *f;
 int txidx;
 nptr nold = np;
 
+switch(is_lx2)
+	{
+#ifdef AET2_IS_PRESENT
+	case LXT2_IS_AET2: import_ae2_trace(np); return;
+#endif
+	case LXT2_IS_VZT:  import_vzt_trace(np); return;
+	default: break; /* fallthrough */
+	}
+
 if(!(f=np->mvlfac)) return;	/* already imported */
 
 txidx = f - mvlfacs;
@@ -435,24 +528,24 @@ if(f->array_height <= 1) /* sorry, arrays not supported, but lx2 doesn't support
 histent_tail = htemp = histent_calloc();
 if(len>1)
 	{
-	htemp->v.vector = (char *)malloc_2(len);
-	for(i=0;i<len;i++) htemp->v.vector[i] = 2;
+	htemp->v.h_vector = (char *)malloc_2(len);
+	for(i=0;i<len;i++) htemp->v.h_vector[i] = AN_Z;
 	}
 	else
 	{
-	htemp->v.val = 2;		/* z */
+	htemp->v.h_val = AN_Z;		/* z */
 	}
 htemp->time = MAX_HISTENT_TIME;
 
 htemp = histent_calloc();
 if(len>1)
 	{
-	htemp->v.vector = (char *)malloc_2(len);
-	for(i=0;i<len;i++) htemp->v.vector[i] = 1;
+	htemp->v.h_vector = (char *)malloc_2(len);
+	for(i=0;i<len;i++) htemp->v.h_vector[i] = AN_X;
 	}
 	else
 	{
-	htemp->v.val = 1;		/* x */
+	htemp->v.h_val = AN_X;		/* x */
 	}
 htemp->time = MAX_HISTENT_TIME-1;
 htemp->next = histent_tail;			
@@ -467,12 +560,12 @@ if(!(f->flags&(LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING)))
         {
 	if(len>1)
 		{
-		np->head.v.vector = (char *)malloc_2(len);
-		for(i=0;i<len;i++) np->head.v.vector[i] = 1;
+		np->head.v.h_vector = (char *)malloc_2(len);
+		for(i=0;i<len;i++) np->head.v.h_vector[i] = AN_X;
 		}
 		else
 		{
-		np->head.v.val = 1;		/* x */
+		np->head.v.h_val = AN_X;	/* x */
 		}
 	}
         else
@@ -502,11 +595,18 @@ if(nold!=np)
  */
 void lx2_set_fac_process_mask(nptr np)
 {
-struct HistEnt *htemp, *histent_tail;
-int len, i, j;
 struct fac *f;
 int txidx;
 
+switch(is_lx2)
+        {
+#ifdef AET2_IS_PRESENT
+        case LXT2_IS_AET2: ae2_set_fac_process_mask(np); return;
+#endif
+        case LXT2_IS_VZT:  vzt_set_fac_process_mask(np); return;
+        default: break; /* fallthrough */
+        }
+
 if(!(f=np->mvlfac)) return;	/* already imported */
 
 txidx = f-mvlfacs;
@@ -527,10 +627,20 @@ if(np->mvlfac->array_height <= 1) /* sorry, arrays not supported, but lx2 doesn'
 }
 
 
-void lx2_import_masked()
+void lx2_import_masked(void)
 {
-int txidx, i, cnt=0;
+int txidx, i, cnt;
 
+switch(is_lx2)
+        {
+#ifdef AET2_IS_PRESENT
+        case LXT2_IS_AET2: ae2_import_masked(); return;
+#endif
+        case LXT2_IS_VZT:  vzt_import_masked(); return;
+        default: break; /* fallthrough */
+        }
+
+cnt = 0;
 for(txidx=0;txidx<numfacs;txidx++)
 	{
 	if(lxt2_rd_get_fac_process_mask(lx2, txidx))
@@ -561,24 +671,24 @@ for(txidx=0;txidx<numfacs;txidx++)
 		histent_tail = htemp = histent_calloc();
 		if(len>1)
 			{
-			htemp->v.vector = (char *)malloc_2(len);
-			for(i=0;i<len;i++) htemp->v.vector[i] = 2;
+			htemp->v.h_vector = (char *)malloc_2(len);
+			for(i=0;i<len;i++) htemp->v.h_vector[i] = AN_Z;
 			}
 			else
 			{
-			htemp->v.val = 2;		/* z */
+			htemp->v.h_val = AN_Z;		/* z */
 			}
 		htemp->time = MAX_HISTENT_TIME;
 			
 		htemp = histent_calloc();
 		if(len>1)
 			{
-			htemp->v.vector = (char *)malloc_2(len);
-			for(i=0;i<len;i++) htemp->v.vector[i] = 1;
+			htemp->v.h_vector = (char *)malloc_2(len);
+			for(i=0;i<len;i++) htemp->v.h_vector[i] = AN_X;
 			}
 			else
 			{
-			htemp->v.val = 1;		/* x */
+			htemp->v.h_val = AN_X;		/* x */
 			}
 		htemp->time = MAX_HISTENT_TIME-1;
 		htemp->next = histent_tail;			
@@ -593,12 +703,12 @@ for(txidx=0;txidx<numfacs;txidx++)
 		        {
 			if(len>1)
 				{
-				np->head.v.vector = (char *)malloc_2(len);
-				for(i=0;i<len;i++) np->head.v.vector[i] = 1;
+				np->head.v.h_vector = (char *)malloc_2(len);
+				for(i=0;i<len;i++) np->head.v.h_vector[i] = AN_X;
 				}
 				else
 				{
-				np->head.v.val = 1;		/* x */
+				np->head.v.h_val = AN_X;	/* x */
 				}
 			}
 		        else
