@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2001-2002.
+ * Copyright (c) Tony Bybell 2001-2004.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -8,11 +8,16 @@
  */
 
 #include <stdio.h>
-#include <zlib.h>
-#include <bzlib.h>
+#include "zlib.h"
+#include "bzlib.h"
 
+#if !defined __MINGW32__ && !defined _MSC_VER
 #include <unistd.h>
 #include <sys/mman.h>
+#else
+#include <windows.h>
+#include <io.h>
+#endif
 
 #include <fcntl.h>
 #include <string.h>
@@ -25,6 +30,7 @@
 #include "bsearch.h"
 #include "debug.h"
 
+
 /****************************************************************************/
 
 /*
@@ -102,24 +108,22 @@ static int version;
 
 static struct fac *mvlfacs=NULL;
 static TimeType first_cycle, last_cycle, total_cycles;
-static TimeType time_scale=1;   /* multiplier is 1, 10, 100 */
 static int maxchange=0, maxindex=0;
-static int trace_start=0;
 static off_t f_len;
 
 static int *positional_information=NULL;
 TimeType *time_information=NULL;
 
-static unsigned int change_field_offset=0;
-static unsigned int facname_offset=0;
-static unsigned int facgeometry_offset=0;
-static unsigned int time_table_offset=0;
-static unsigned int time_table_offset64=0;
-static unsigned int sync_table_offset=0;
-static unsigned int initial_value_offset=0;
-static unsigned int timescale_offset=0;
-static unsigned int double_test_offset=0;
-static unsigned int zdictionary_offset=0;
+static off_t change_field_offset=0;
+static off_t facname_offset=0;
+static off_t facgeometry_offset=0;
+static off_t time_table_offset=0;
+static off_t time_table_offset64=0;
+static off_t sync_table_offset=0;
+static off_t initial_value_offset=0;
+static off_t timescale_offset=0;
+static off_t double_test_offset=0;
+static off_t zdictionary_offset=0;
 
 static unsigned int zfacname_predec_size=0;
 static unsigned int zfacname_size=0;
@@ -130,23 +134,54 @@ static unsigned int zchg_predec_size=0;
 static unsigned int zchg_size=0;
 static unsigned int zdictionary_predec_size=0;
 
-static unsigned char initial_value = 1;
+static unsigned char initial_value = AN_X;
 
 static unsigned int dict_num_entries;
 static unsigned int dict_string_mem_required;
-static unsigned int dict_16_offset;
-static unsigned int dict_24_offset;
-static unsigned int dict_32_offset;
+static off_t dict_16_offset;
+static off_t dict_24_offset;
+static off_t dict_32_offset;
 static unsigned int dict_width;
 static char **dict_string_mem_array=NULL;
 
-static unsigned int exclude_offset=0;
+static off_t exclude_offset=0;
 
 static char *lt_buf=NULL;	/* max memory ever needed for parse_offset() */
 static int lt_len = 0;
 
 static int fd;			/* XXX : win32 requires decompression tempfiles since */
 				/* its mmap() variant doesn't use file descriptors    */
+#if defined __MINGW32__ || defined _MSC_VER
+#define PROT_READ (0)
+#define MAP_SHARED (0)
+
+HANDLE hIn, hInMap;
+char *win_fname = NULL;
+
+void *mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset)
+{
+close(fd);
+hIn = CreateFile(win_fname, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+if(hIn == INVALID_HANDLE_VALUE)
+{
+	fprintf(stderr, "Could not open '%s' with CreateFile(), exiting.\n", win_fname);
+	exit(255);
+}
+
+hInMap = CreateFileMapping(hIn, NULL, PAGE_READONLY, 0, 0, NULL);
+return(MapViewOfFile(hInMap, FILE_MAP_READ, 0, 0, 0));
+}
+
+int munmap(void *start, size_t length)
+{
+UnmapViewOfFile(start);
+CloseHandle(hInMap); hInMap = NULL;
+CloseHandle(hIn); hIn = NULL;
+return(0);
+}
+
+
+#endif
 
 /****************************************************************************/
 
@@ -174,11 +209,11 @@ static int fd;			/* XXX : win32 requires decompression tempfiles since */
  * of a big-endian integer.  x86 specific version...
  */
 
-#ifdef __i386__
+#if defined(__i386__) || defined(__x86_64__)
 
 #define get_byte(offset)        ((unsigned int)(*((unsigned char *)mm+offset)))
 
-inline static unsigned int get_16(int offset)
+inline static unsigned int get_16(off_t offset)
 {
 unsigned short x = *((unsigned short *)((unsigned char *)mm+offset));
 
@@ -189,7 +224,7 @@ unsigned short x = *((unsigned short *)((unsigned char *)mm+offset));
     return (unsigned int) x;
 }
 
-inline static unsigned int get_32(int offset)	/* note that bswap is really 486+ */
+inline static unsigned int get_32(off_t offset)	/* note that bswap is really 486+ */
 {
 unsigned int x = *((unsigned int *)((unsigned char *)mm+offset));
 
@@ -211,7 +246,7 @@ unsigned int x = *((unsigned int *)((unsigned char *)mm+offset));
  */
 #define get_byte(offset) 	((unsigned int)(*((unsigned char *)mm+offset)))
 
-static unsigned int get_16(int offset)
+static unsigned int get_16(off_t offset)
 {
 unsigned char *nn=(unsigned char *)mm+offset;
 unsigned int m1=*((unsigned char *)(nn++));
@@ -219,7 +254,7 @@ unsigned int m2=*((unsigned char *)nn);
 return((m1<<8)|m2);
 }
 
-static unsigned int get_24(int offset)
+static unsigned int get_24(off_t offset)
 {
 unsigned char *nn=(unsigned char *)mm+offset;
 unsigned int m1=*((unsigned char *)(nn++));
@@ -228,7 +263,7 @@ unsigned int m3=*((unsigned char *)nn);
 return((m1<<16)|(m2<<8)|m3);
 }
 
-static unsigned int get_32(int offset)
+static unsigned int get_32(off_t offset)
 {
 unsigned char *nn=(unsigned char *)mm+offset;
 unsigned int m1=*((unsigned char *)(nn++));
@@ -238,7 +273,7 @@ unsigned int m4=*((unsigned char *)nn);
 return((m1<<24)|(m2<<16)|(m3<<8)|m4);
 }
 
-static UTimeType get_64(int offset)
+static UTimeType get_64(off_t offset)
 {
 return(
 (((UTimeType)get_32(offset))<<32)
@@ -251,7 +286,7 @@ return(
 
 /****************************************************************************/
 
-static char double_mask[8]={0,0,0,0,0,0,0,0};
+static unsigned char double_mask[8]={0,0,0,0,0,0,0,0};
 static char double_is_native=0;
 
 static void create_double_endian_mask(int offset)
@@ -359,13 +394,14 @@ return(time_information[max_compare_pos_tc]);
  */
 static void build_dict(void)
 {
-FILE *tmp;
 gzFile zhandle;
-int offs = zdictionary_offset+24;
+off_t offs = zdictionary_offset+24;
 int total_mem, rc, i;
 char *decmem=NULL;
 char *pnt;
-unsigned char *t;
+#if defined __MINGW32__ || defined _MSC_VER
+FILE *tmp;
+#endif
 
 dict_num_entries = get_32(zdictionary_offset+0);
 dict_string_mem_required = get_32(zdictionary_offset+4);
@@ -382,10 +418,19 @@ DEBUG(fprintf(stderr, LXTHDR"dict_16_offset = %d\n", dict_16_offset));
 DEBUG(fprintf(stderr, LXTHDR"dict_24_offset = %d\n", dict_24_offset));
 DEBUG(fprintf(stderr, LXTHDR"dict_32_offset = %d\n", dict_32_offset));
 
-printf(LXTHDR"Dictionary compressed MVL2 change records detected...\n");
+fprintf(stderr, LXTHDR"Dictionary compressed MVL2 change records detected...\n");
 
-if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+{
+unsigned char *t = (char *)mm+offs;
+tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+fwrite(t, zdictionary_predec_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+zhandle = gzdopen(dup(fileno(tmp)), "rb");
+}
+#else
+if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"dict lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 zhandle = gzdopen(dup(fd), "rb");
+#endif
 decmem = malloc_2(total_mem = dict_string_mem_required);
 
 rc=gzread(zhandle, decmem, total_mem);
@@ -403,8 +448,11 @@ for(i=0;i<dict_num_entries;i++)
 	}
 
 gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+fclose(tmp);
+#endif
 
-printf(LXTHDR"...expanded %d entries from %08x into %08x bytes.\n", dict_num_entries, zdictionary_predec_size, dict_string_mem_required);
+fprintf(stderr, LXTHDR"...expanded %d entries from %08x into %08x bytes.\n", dict_num_entries, zdictionary_predec_size, dict_string_mem_required);
 }
 
 
@@ -414,22 +462,32 @@ printf(LXTHDR"...expanded %d entries from %08x into %08x bytes.\n", dict_num_ent
 static void build_facs(char *fname)
 {
 char *buf, *bufprev, *bufcurr;
-int offs=facname_offset+8;
+off_t offs=facname_offset+8;
 int i, j, clone;
 char *pnt;
 int total_mem = get_32(facname_offset+4);
-gzFile zhandle;
+gzFile zhandle = NULL;
 char *decmem=NULL;
 
+#if defined __MINGW32__ || defined _MSC_VER
+FILE *tmp;
+#endif
+
 buf=malloc_2(total_mem);
 pnt=bufprev=buf;
 
 if(zfacname_size)
 	{
 	int rc;
-
-	if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+	unsigned char *t = (char *)mm+offs;
+	tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+	fwrite(t, zfacname_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+	zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
+	if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"zfacname lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 	zhandle = gzdopen(dup(fd), "rb");
+#endif
 
 	mmcache = mm;
 	decmem = malloc_2(total_mem = zfacname_predec_size); mm = decmem;
@@ -442,7 +500,7 @@ if(zfacname_size)
 	offs=0;	/* we're in our new memory region now.. */
 	}
 
-printf(LXTHDR"Building %d facilities.\n", numfacs);
+fprintf(stderr, LXTHDR"Building %d facilities.\n", numfacs);
 for(i=0;i<numfacs;i++)
 	{
         clone=get_16(offs);  offs+=2;
@@ -462,6 +520,9 @@ if(zfacname_size)
 	mm = mmcache;
 	free_2(decmem); decmem = NULL;
 	gzclose(zhandle);
+#if defined __MINGW32__ ||  defined _MSC_VER
+	fclose(tmp);
+#endif
 	}
 
 if(!facgeometry_offset)
@@ -476,9 +537,15 @@ if(zfacgeometry_size)
 	{
 	int rc;
 
-	if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+	unsigned char *t = (char *)mm+offs;
+	tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+	fwrite(t, zfacgeometry_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+	zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
+	if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"zfacgeometry lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 	zhandle = gzdopen(dup(fd), "rb");
-
+#endif
 	mmcache = mm;
 	total_mem = numfacs * 16;
 	decmem = malloc_2(total_mem); mm = decmem;
@@ -512,6 +579,9 @@ if(zfacgeometry_size)
 	mm = mmcache;
 	free_2(decmem); decmem = NULL;
 	gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+	fclose(tmp);
+#endif
 	}
 }
 
@@ -523,14 +593,18 @@ if(zfacgeometry_size)
 static void build_facs2(char *fname)
 {
 int i;
-int offs;
+off_t offs;
 int chg;
 int maxchg=0, maxindx=0;
 int last_position;
 TimeType last_time;
-char *decmem;
+char *decmem = NULL;
 int total_mem;
-gzFile zhandle;
+gzFile zhandle = NULL;
+
+#if defined __MINGW32__ || defined _MSC_VER
+FILE *tmp;
+#endif
 
 if((!time_table_offset)&&(!time_table_offset64))
 	{
@@ -556,9 +630,15 @@ if(time_table_offset)
 		{
 		int rc;
 
-		if((offs+4)!=lseek(fd, offs+4, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+		unsigned char *t = (char *)mm+offs+4;
+		tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+		fwrite(t, ztime_table_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+		zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
+		if((offs+4)!=lseek(fd, offs+4, SEEK_SET)) { fprintf(stderr, LXTHDR"ztime_table lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 		zhandle = gzdopen(dup(fd), "rb");
-
+#endif
 		mmcache = mm;
 		total_mem = 4 + 4 + (total_cycles * 4) + (total_cycles * 4);
 		decmem = malloc_2(total_mem); mm = decmem;
@@ -605,6 +685,9 @@ if(time_table_offset)
 		mm = mmcache;
 		free_2(decmem); decmem = NULL;
 		gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+		fclose(tmp);
+#endif
 		}
 	}
 	else	/* 64-bit read */
@@ -620,9 +703,15 @@ if(time_table_offset)
 		{
 		int rc;
 
-		if((offs+4)!=lseek(fd, offs+4, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+		unsigned char *t = (char *)mm+offs+4;
+		tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+		fwrite(t, ztime_table_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+		zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
+		if((offs+4)!=lseek(fd, offs+4, SEEK_SET)) { fprintf(stderr, LXTHDR"ztime_table lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 		zhandle = gzdopen(dup(fd), "rb");
-
+#endif
 		mmcache = mm;
 		total_mem = 8 + 8 + (total_cycles * 4) + (total_cycles * 8);
 		decmem = malloc_2(total_mem); mm = decmem;
@@ -669,6 +758,9 @@ if(time_table_offset)
 		mm = mmcache;
 		free_2(decmem); decmem = NULL;
 		gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+		fclose(tmp);
+#endif
 		}
 	}
 
@@ -679,10 +771,15 @@ if(sync_table_offset)
 	if(zsync_table_size)
 		{
 		int rc;
-
-		if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", offs); exit(255); }
+#if defined __MINGW32__ || defined _MSC_VER
+		unsigned char *t = (char *)mm+offs;
+		tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+		fwrite(t, zsync_table_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+		zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
+		if(offs!=lseek(fd, offs, SEEK_SET)) { fprintf(stderr, LXTHDR"zsync_table lseek error at offset %08x\n", (unsigned int)offs); exit(255); }
 		zhandle = gzdopen(dup(fd), "rb");
-
+#endif
 		mmcache = mm;
 		decmem = malloc_2(total_mem = numfacs * 4); mm = decmem;
 
@@ -707,6 +804,9 @@ if(sync_table_offset)
 		mm = mmcache;
 		free_2(decmem); decmem = NULL;
 		gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+		fclose(tmp);
+#endif
 		}
 
 	DEBUG(printf(LXTHDR"Maxchange at: %08x for symbol '%s' of len %d\n", maxchg, mvlfacs[maxindx].name,  mvlfacs[maxindx].len));
@@ -717,9 +817,12 @@ if(sync_table_offset)
 
 if(zchg_size)
 	{
-	if(zchg_predec_size > 128*1024*1024)
+	/* we don't implement the tempfile version for windows... */
+#if !defined __MINGW32__ && !defined _MSC_VER
+	if(zchg_predec_size > LXT_MMAP_MALLOC_BOUNDARY)
 		{
-		char *nam = tmpnam(NULL);
+		int fd_dummy;
+		char *nam = tmpnam_2(NULL, &fd_dummy);
 		FILE *tmp = fopen(nam, "wb");
 		unsigned int len=zchg_predec_size;
 		int rc;
@@ -729,14 +832,15 @@ if(zchg_size)
 		char is_bz2;
 	
 		unlink(nam);
-	
-		printf(LXTHDR"Compressed change records detected, making tempfile...\n");
-		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", change_field_offset); exit(255); }
+		if(fd_dummy >=0) close(fd_dummy);
+
+		fprintf(stderr, LXTHDR"Compressed change records detected, making tempfile...\n");
+		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
 		read(fd, &testbyte, 2);
 		is_bz2 = (testbyte[0]=='B')&&(testbyte[1]=='Z');
 
-		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", change_field_offset); exit(255); }
+		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
 		if(is_bz2)
 			{
@@ -747,11 +851,11 @@ if(zchg_size)
 				int siz = (len>32768) ? 32768 : len; 
 				rc = BZ2_bzread(zhandle, buf, siz);	
 				if(rc!=siz) { fprintf(stderr, LXTHDR"gzread error to tempfile %d (act) vs %d (exp), exiting.\n", rc, siz); exit(255); }
-				if(1 != fwrite(buf, siz, 1, tmp)) { fprintf(stderr, LXTHDR"fwrite error to tempfile, exiting.\n", rc, siz); exit(255); };
+				if(1 != fwrite(buf, siz, 1, tmp)) { fprintf(stderr, LXTHDR"fwrite error to tempfile, exiting.\n"); exit(255); };
 				len -= siz;		
 				}
 	
-			printf(LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
+			fprintf(stderr, LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
 			BZ2_bzclose(zhandle);
 			}
 			else
@@ -763,23 +867,25 @@ if(zchg_size)
 				int siz = (len>32768) ? 32768 : len; 
 				rc = gzread(zhandle, buf, siz);	
 				if(rc!=siz) { fprintf(stderr, LXTHDR"gzread error to tempfile %d (act) vs %d (exp), exiting.\n", rc, siz); exit(255); }
-				if(1 != fwrite(buf, siz, 1, tmp)) { fprintf(stderr, LXTHDR"fwrite error to tempfile, exiting.\n", rc, siz); exit(255); };
+				if(1 != fwrite(buf, siz, 1, tmp)) { fprintf(stderr, LXTHDR"fwrite error to tempfile, exiting.\n"); exit(255); };
 				len -= siz;		
 				}
 	
-			printf(LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
+			fprintf(stderr, LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
 			gzclose(zhandle);
 			}
 		munmap(mm, f_len); close(fd);
 		fflush(tmp);
-		fseek(tmp, 0, SEEK_SET);
+		fseeko(tmp, 0, SEEK_SET);
 		fclose(tmp);
 	
 		fd = fd2;
 		mm=mmap(NULL, zchg_predec_size, PROT_READ, MAP_SHARED, fd, 0);	
-		mm-=4; /* because header and version don't exist in packed change records */
+		
+		mm=(void *)((char *)mm-4); /* because header and version don't exist in packed change records */
 		}
 		else
+#endif
 		{
 		unsigned int len=zchg_predec_size;
 		int rc;
@@ -787,19 +893,32 @@ if(zchg_size)
 		char *pnt = buf;
 		char testbyte[2]={0,0};
 		char is_bz2;
-		
-		printf(LXTHDR"Compressed change records detected...\n");
-		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", change_field_offset); exit(255); }
+
+		fprintf(stderr, LXTHDR"Compressed change records detected...\n");
+#if defined __MINGW32__ || defined _MSC_VER
+		{
+		unsigned char *t = (char *)mm+change_field_offset;
+		tmp = tmpfile(); if(!tmp) { fprintf(stderr, LXTHDR"could not open decompression tempfile, exiting.\n"); exit(255); }
+		fwrite(t, zchg_size, 1, tmp); fseek(tmp, 0, SEEK_SET);
+		is_bz2 = (get_byte(change_field_offset)=='B') && (get_byte(change_field_offset+1)=='Z');
+		}
+#else		
+		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
 		read(fd, &testbyte, 2);
 		is_bz2 = (testbyte[0]=='B')&&(testbyte[1]=='Z');
 
-		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", change_field_offset); exit(255); }
+		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
+#endif
 
 		if(is_bz2)
 			{
+#if defined __MINGW32__ || defined _MSC_VER
+			zhandle = BZ2_bzdopen(dup(fileno(tmp)), "rb");			
+#else
 			zhandle = BZ2_bzdopen(dup(fd), "rb");
-	
+#endif
+			
 			while(len)
 				{
 				int siz = (len>32768) ? 32768 : len; 
@@ -809,13 +928,16 @@ if(zchg_size)
 				len -= siz;		
 				}
 	
-			printf(LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
+			fprintf(stderr, LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
 			BZ2_bzclose(zhandle);
 			}
 			else
 			{
+#if defined __MINGW32__ || defined _MSC_VER
+			zhandle = gzdopen(dup(fileno(tmp)), "rb");
+#else
 			zhandle = gzdopen(dup(fd), "rb");
-	
+#endif
 			while(len)
 				{
 				int siz = (len>32768) ? 32768 : len; 
@@ -825,12 +947,18 @@ if(zchg_size)
 				len -= siz;		
 				}
 	
-			printf(LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
+			fprintf(stderr, LXTHDR"...expanded %08x into %08x bytes.\n", zchg_size, zchg_predec_size);
 			gzclose(zhandle);
+#if defined __MINGW32__ || defined _MSC_VER
+			fclose(tmp);
+#endif
 			}
 
-		munmap(mm, f_len); close(fd);
-	
+		munmap(mm, f_len); 
+#if !defined __MINGW32__ && !defined _MSC_VER
+		close(fd);
+#endif
+		
 		fd=-1;
 		mm=buf-4; /* because header and version don't exist in packed change records */
 		}
@@ -839,8 +967,8 @@ if(zchg_size)
 
 if(!sync_table_offset)
 	{
-	unsigned int vlen = zchg_predec_size ? zchg_predec_size+4 : 0;
-	unsigned int offs = zchg_predec_size ? 4 : 0;
+	off_t vlen = zchg_predec_size ? zchg_predec_size+4 : 0;
+	off_t offs = zchg_predec_size ? 4 : 0;
 	unsigned int numfacs_bytes;
 	unsigned int num_records = 0;
 	unsigned int last_change_delta, numbytes;
@@ -855,8 +983,9 @@ if(!sync_table_offset)
 	char *nam;
 	FILE *tmp;
 	int recfd;
+	int fd_dummy;
 	
-	printf(LXTHDR"Linear LXT encountered...\n");
+	fprintf(stderr, LXTHDR"Linear LXT encountered...\n");
 
 	if(!zchg_predec_size)
 		{
@@ -883,19 +1012,20 @@ if(!sync_table_offset)
                	numfacs_bytes = 0;
                	}
 
-	nam = tmpnam(NULL);
+	nam = tmpnam_2(NULL, &fd_dummy);
 	tmp = fopen(nam, "wb");
 	fpos = 4;	/* fake 4 bytes padding */
 	recfd = open(nam, O_RDONLY);
 
 	unlink(nam);
+	if(fd_dummy >=0) close(fd_dummy);
+
 
 	while(offs < vlen)
 		{
-		unsigned int facidx;
+		unsigned int facidx = 0;
 		unsigned char cmd;
-		unsigned int offscache = offs;
-		unsigned int offscache2, offscache3;
+		off_t offscache2, offscache3;
 		unsigned int height;
 		unsigned char cmdkill;
 
@@ -922,7 +1052,7 @@ if(!sync_table_offset)
 
 		if(facidx>numfacs)
 			{
-			fprintf(stderr, LXTHDR"Facidx %d out of range (vs %d) at offset %08x, exiting.\n", facidx, numfacs, offs);
+			fprintf(stderr, LXTHDR"Facidx %d out of range (vs %d) at offset %08x, exiting.\n", facidx, numfacs, (unsigned int)offs);
 			exit(255);
 			}
 
@@ -935,7 +1065,7 @@ if(!sync_table_offset)
 
 			if(cmd>0xf)
 				{
-				fprintf(stderr, LXTHDR"Command byte %02x invalid at offset %08x, exiting.\n", cmd, offs);
+				fprintf(stderr, LXTHDR"Command byte %02x invalid at offset %08x, exiting.\n", cmd, (unsigned int)offs);
 				exit(0);
 				}
 
@@ -1101,7 +1231,7 @@ if(!sync_table_offset)
 
 		if(offs-offscache2)
 			{
-			fpos += fwrite(mm+offscache2, 1, offs-offscache2, tmp);	/* copy rest of relevant info */
+			fpos += fwrite((char *)mm+offscache2, 1, offs-offscache2, tmp);	/* copy rest of relevant info */
 			}
 		}
 
@@ -1110,22 +1240,27 @@ if(!sync_table_offset)
 	dict_32_offset = dict_32_offset_new;
 
         fflush(tmp);
-        fseek(tmp, 0, SEEK_SET);
+        fseeko(tmp, 0, SEEK_SET);
         fclose(tmp);
-	printf(LXTHDR"%d linear records converted into %08x bytes.\n", num_records, fpos-4);
+	fprintf(stderr, LXTHDR"%d linear records converted into %08x bytes.\n", num_records, fpos-4);
 
-	if(zchg_predec_size > 128*1024*1024)
+#if !defined __MINGW32__ && !defined _MSC_VER
+	if(zchg_predec_size > LXT_MMAP_MALLOC_BOUNDARY)
 		{
-	        munmap(mm+4, zchg_predec_size); close(fd);
+		munmap((char *)mm+4, zchg_predec_size); close(fd);
 		}
 		else
+#endif
 		{
-		free(mm+4);
+		free((char *)mm+4);
 		}
 
-	fd = recfd;              
+	fd = recfd;            
+#if defined __MINGW32__ || defined _MSC_VER	
+	win_fname = nam;
+#endif
         mm=mmap(NULL, fpos-4, PROT_READ, MAP_SHARED, recfd, 0);
-        mm-=4; /* because header and version don't exist in packed change records */
+        mm=(void *)((char *)mm-4); /* because header and version don't exist in packed change records */
 	}
 }
 
@@ -1133,7 +1268,7 @@ if(!sync_table_offset)
 /*
  * given a fac+offset, return the binary data for it
  */
-static char *parse_offset(struct fac *which, unsigned int offs)
+static char *parse_offset(struct fac *which, off_t offs)
 {
 int v, v2, j;
 int k, l;
@@ -1191,7 +1326,7 @@ switch(v2)
 					}
 					else
 					{
-					fprintf(stderr, LXTHDR"dict entry at offset %08x [%d] out of range, ignoring!\n", dictpos, offs);
+					fprintf(stderr, LXTHDR"dict entry at offset %08x [%d] out of range, ignoring!\n", dictpos, (unsigned int)offs);
 					for(j=0;j<l;j++)
 						{
 						*(pnt++) = '0';
@@ -1278,7 +1413,7 @@ switch(v2)
 			break;
 
 	default:
-			fprintf(stderr, LXTHDR"Unknown %02x at offset: %08x\n", v, offs);
+			fprintf(stderr, LXTHDR"Unknown %02x at offset: %08x\n", v, (unsigned int)offs);
 			exit(255);
 	}
 
@@ -1295,7 +1430,7 @@ TimeType lxt_main(char *fname)
 int i;
 struct Node *n;
 struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
-unsigned int tagpnt;
+off_t tagpnt;
 int tag;
 
 fd=open(fname, O_RDONLY);
@@ -1305,16 +1440,19 @@ if(fd<0)
         exit(0);
         }
 
-f_len=lseek(fd, 0, SEEK_END);
+f_len=lseek(fd, (off_t)0, SEEK_END);
+#if defined __MINGW32__ || defined _MSC_VER
+win_fname = fname;
+#endif
 mm=mmap(NULL, f_len, PROT_READ, MAP_SHARED, fd, 0);
 
-if((i=get_16(0))!=LT_HDRID)
+if((i=get_16((off_t)0))!=LT_HDRID)
 	{
 	fprintf(stderr, "Not an LXT format AET, exiting.\n");
 	exit(255);
 	}
 
-if((version=get_16(2))>LT_VERSION)
+if((version=get_16((off_t)2))>LT_VERSION)
 	{
 	fprintf(stderr, "Version %d of LXT format AETs not supported, exiting.\n", version);
 	exit(255);
@@ -1329,10 +1467,12 @@ if(get_byte(f_len-1)!=LT_TRLID)
 DEBUG(printf(LXTHDR"Loading LXT '%s'...\n", fname));
 DEBUG(printf(LXTHDR"Len: %d\n", (unsigned int)f_len));
 
+/* SPLASH */                            splash_create();
+
 tagpnt = f_len-2;
 while((tag=get_byte(tagpnt))!=LT_SECTION_END)
 	{
-	int offset = get_32(tagpnt-4);
+	off_t offset = get_32(tagpnt-4);
 	tagpnt-=5;
 
 	switch(tag)
@@ -1366,7 +1506,7 @@ while((tag=get_byte(tagpnt))!=LT_SECTION_END)
 
 if(exclude_offset)
 	{
-	unsigned int offset = exclude_offset;
+	off_t offset = exclude_offset;
 	int i, num_blackouts = get_32(offset);
 	TimeType bs, be;
 	struct blackout_region_t *bt;
@@ -1397,31 +1537,7 @@ if(timescale_offset)
 	signed char scale;
 
 	scale=(signed char)get_byte(timescale_offset);
-	switch(scale)
-		{
-		case 0:		time_dimension = 's'; break;
-
-		case -1:	time_scale = LLDescriptor(100); time_dimension = 'm'; break;
-		case -2:	time_scale = LLDescriptor(10);  
-		case -3:					time_dimension = 'm'; break;
-
-		case -4:	time_scale = LLDescriptor(100);	time_dimension = 'u'; break;
-		case -5:	time_scale = LLDescriptor(10);	
-		case -6:					time_dimension = 'u'; break;
-
-		case -10:	time_scale = LLDescriptor(100); time_dimension = 'p'; break;
-		case -11:	time_scale = LLDescriptor(10);  
-		case -12:					time_dimension = 'p'; break;
-
-		case -13:	time_scale = LLDescriptor(100); time_dimension = 'f'; break;
-		case -14:	time_scale = LLDescriptor(10);  
-		case -15:					time_dimension = 'f'; break;
-
-		case -7:	time_scale = LLDescriptor(100); time_dimension = 'n'; break;
-		case -8:	time_scale = LLDescriptor(10);  
-		case -9:
-		default:					time_dimension = 'n'; break;
-		}
+	exponent_to_time_scale(scale);
 	}
 	else
 	{
@@ -1443,18 +1559,19 @@ if(initial_value_offset)
 	switch(get_byte(initial_value_offset))
 		{
 		case 0:
-		case 7:		initial_value = 0; break;
+		case 7:		initial_value = AN_0; break;
 		case 1:
-		case 4:		initial_value = 3; break;
-		case 2:		initial_value = 2; break;
-		default:	initial_value = 1; break;
+		case 4:		initial_value = AN_1; break;
+		case 2:		initial_value = AN_Z; break;
+		default:	initial_value = AN_X; break;
 		}
 	}
 	else
 	{
-	initial_value = 1;
+	initial_value = AN_X;
 	}
 
+	
 if(zdictionary_offset)
 	{
 	if(zdictionary_predec_size)
@@ -1469,8 +1586,9 @@ if(zdictionary_offset)
 	}
 
 build_facs(fname);
+/* SPLASH */                            splash_sync(1, 5);
 build_facs2(fname);
-
+/* SPLASH */                            splash_sync(2, 5);
 
 /* do your stuff here..all useful info has been initialized by now */
 
@@ -1512,13 +1630,15 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		prevsymroot = prevsym = NULL;
 		}
-	else if ( (f->len==1)&&(!(f->flags&(LT_SYM_F_INTEGER|LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))&&
-			((i!=numfacs-1)&&(!strcmp(mvlfacs[i].name, mvlfacs[i+1].name))) ||
-			((i!=0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name))) &&
-			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1)
+	else if ( 
+			((f->len==1)&&(!(f->flags&(LT_SYM_F_INTEGER|LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))&&
+			((i!=numfacs-1)&&(!strcmp(mvlfacs[i].name, mvlfacs[i+1].name))))
+			||
+			(((i!=0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name))) &&
+			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1))
 		)
 		{
 		sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
@@ -1531,7 +1651,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		if((prevsym)&&(i>0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name)))	/* allow chaining for search functions.. */
 			{
 			prevsym->vec_root = prevsymroot;
@@ -1555,7 +1675,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, mvlfacs[i].name, alt_hier_delimeter);
 			}
-	        s=symadd(str,hash(str));
+	        s=symadd_name_exists(str,hash(str));
 		prevsymroot = prevsym = NULL;
 
 		if(f->flags&LT_SYM_F_INTEGER)
@@ -1589,10 +1709,17 @@ for(i=0;i<numfacs;i++)
 		}
                  
         n->head.time=-1;        /* mark 1st node as negative time */
-        n->head.v.val=1;
+        n->head.v.h_val=AN_X;
         s->n=n;
         }
 
+free(mvlfacs[0].name);		/* the start of the big decompression buffer */
+for(i=0;i<numfacs;i++)
+	{
+	mvlfacs[i].name = NULL;
+	}
+
+/* SPLASH */                            splash_sync(3, 5);
 facs=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
 curnode=firstnode;
 for(i=0;i<numfacs;i++)
@@ -1610,10 +1737,9 @@ for(i=0;i<numfacs;i++)
 		}
 	}
 
-printf(LXTHDR"Sorting facilities at hierarchy boundaries...");
-fflush(stdout);
+fprintf(stderr, LXTHDR"Sorting facilities at hierarchy boundaries...");
 wave_heapsort(facs,numfacs);
-printf("sorted.\n");
+fprintf(stderr, "sorted.\n");
 	
 for(i=0;i<numfacs;i++)
 	{
@@ -1633,22 +1759,23 @@ for(i=0;i<numfacs;i++)
 
 facs_are_sorted=1;
 
-printf(LXTHDR"Building facility hierarchy tree...");
-fflush(stdout);
+fprintf(stderr, LXTHDR"Building facility hierarchy tree...");
+/* SPLASH */                            splash_sync(4, 5);
 init_tree();		
 for(i=0;i<numfacs;i++)	
 {
 build_tree_from_name(facs[i]->name, i);
 }
+/* SPLASH */                            splash_sync(5, 5);
 treeprune(treeroot);
-printf("built.\n\n");
+fprintf(stderr, "built.\n\n");
 
 #ifdef DEBUG_FACILITIES
 treedebug(treeroot,"");
 #endif
 
 min_time = first_cycle*time_scale; max_time=last_cycle*time_scale;
-printf("["TTFormat"] start time.\n["TTFormat"] end time.\n", min_time, max_time);
+fprintf(stderr, "["TTFormat"] start time.\n["TTFormat"] end time.\n", min_time, max_time);
 is_lxt = ~0;
 
 return(max_time);
@@ -1677,9 +1804,8 @@ np->mvlfac=NULL;
  */
 void import_lxt_trace(nptr np)
 {
-unsigned int offs, offsdelta;
+off_t offs, offsdelta;
 int v, w;
-int ary_indx, ary_skip;
 TimeType tmval;
 TimeType prevtmval;
 struct HistEnt *htemp;
@@ -1736,24 +1862,24 @@ len = np->mvlfac->len;
 histent_tail = htemp = histent_calloc();
 if(len>1)
 	{
-	htemp->v.vector = (char *)malloc_2(len);
-	for(i=0;i<len;i++) htemp->v.vector[i] = 2;
+	htemp->v.h_vector = (char *)malloc_2(len);
+	for(i=0;i<len;i++) htemp->v.h_vector[i] = AN_Z;
 	}
 	else
 	{
-	htemp->v.val = 2;		/* z */
+	htemp->v.h_val = AN_Z;		/* z */
 	}
 htemp->time = MAX_HISTENT_TIME;
 
 histent_head = histent_calloc();
 if(len>1)
 	{
-	histent_head->v.vector = (char *)malloc_2(len);
-	for(i=0;i<len;i++) histent_head->v.vector[i] = 1;
+	histent_head->v.h_vector = (char *)malloc_2(len);
+	for(i=0;i<len;i++) histent_head->v.h_vector[i] = AN_X;
 	}
 	else
 	{
-	histent_head->v.val = 1;	/* x */
+	histent_head->v.h_val = AN_X; /* x */
 	}
 histent_head->time = MAX_HISTENT_TIME-1;
 histent_head->next = htemp;	/* x */
@@ -1763,11 +1889,11 @@ np->numhist=2;
 if(f->array_height < 1)	/* sorry, arrays not supported */
 while(offs)
 	{
-	unsigned char val;
+	unsigned char val = 0;
 
 	if( (w=((v=get_byte(offs))&0xF)) >0xb)
 		{
-		unsigned int offsminus1, offsminus2, offsminus3;
+		off_t offsminus1, offsminus2, offsminus3;
 		TimeType delta, time_offsminus1;
 		int skip;
 
@@ -1790,7 +1916,7 @@ while(offs)
 				offsdelta=get_32(offs+1);
 				break;
 			default:
-				fprintf(stderr, "Unknown %02x at offset: %08x\n", v, offs);
+				fprintf(stderr, "Unknown %02x at offset: %08x\n", v, (unsigned int)offs);
 				exit(0);
 			}
 		offsminus1 = offs-offsdelta-2;
@@ -1810,7 +1936,7 @@ while(offs)
 				offsdelta=get_32(offsminus1+1);
 				break;
 			default:
-				fprintf(stderr, "Unknown %02x at offset: %08x\n", get_byte(offsminus1), offsminus1);
+				fprintf(stderr, "Unknown %02x at offset: %08x\n", get_byte(offsminus1), (unsigned int)offsminus1);
 				exit(0);
 			}
 		offsminus2 = offsminus1-offsdelta-2;
@@ -1825,7 +1951,7 @@ while(offs)
 			if(!lxt_clock_compress_to_z)
 				{
 				int val = get_byte(offsminus1)&0xF;
-				int reps;
+				int reps = 0;
 				int rcnt;
 				unsigned int reconstructm1 = 0;
 				unsigned int reconstructm2 = 0;
@@ -1835,7 +1961,7 @@ while(offs)
 	
 				if((val!=0)&&(val!=3)&&(val!=4))
 					{
-					fprintf(stderr, "Unexpected clk compress byte %02x at offset: %08x\n", get_byte(offsminus1), offsminus1);
+					fprintf(stderr, "Unexpected clk compress byte %02x at offset: %08x\n", get_byte(offsminus1), (unsigned int)offsminus1);
 					exit(0);
 					}
 	
@@ -1884,7 +2010,7 @@ while(offs)
 						offsdelta=get_32(offsminus2+1);
 						break;
 					default:
-						fprintf(stderr, "Unknown %02x at offset: %08x\n", get_byte(offsminus2), offsminus2);
+						fprintf(stderr, "Unknown %02x at offset: %08x\n", get_byte(offsminus2), (unsigned int)offsminus2);
 						exit(0);
 					}
 				offsminus3 = offsminus2-offsdelta-2;
@@ -1923,8 +2049,8 @@ while(offs)
 						}
 
 					htemp = histent_calloc();
-					htemp->v.vector = (char *)malloc_2(len);
-					memcpy(htemp->v.vector, parsed, len);
+					htemp->v.h_vector = (char *)malloc_2(len);
+					memcpy(htemp->v.h_vector, parsed, len);
 					htemp->time = tmval;
 					htemp->next = histent_head;
 					histent_head = htemp;
@@ -1938,8 +2064,8 @@ while(offs)
 				int i;	
 
 				htemp = histent_calloc();
-				htemp->v.vector = (char *)malloc_2(len);
-				for(i=0;i<len;i++) { htemp->v.vector[i] = 'z'; }
+				htemp->v.h_vector = (char *)malloc_2(len);
+				for(i=0;i<len;i++) { htemp->v.h_vector[i] = 'z'; }
 				tmval = time_offsminus1 + delta;
 				htemp->time = tmval;
 				htemp->next = histent_head;
@@ -1955,12 +2081,12 @@ while(offs)
 			if(!lxt_clock_compress_to_z)
 				{
 				int val = get_byte(offsminus1)&0xF;
-				int reps;
+				int reps = 0;
 				int rcnt;
 	
 				if((val<3)||(val>4))
 					{
-					fprintf(stderr, "Unexpected clk compress byte %02x at offset: %08x\n", get_byte(offsminus1), offsminus1);
+					fprintf(stderr, "Unexpected clk compress byte %02x at offset: %08x\n", get_byte(offsminus1), (unsigned int)offsminus1);
 					exit(0);
 					}
 	
@@ -1974,15 +2100,15 @@ while(offs)
 	
 				reps++;
 				val = (reps & 1) ^ (val==4);	/* because x3='0', x4='1' */
-				val |= val<<1;			/* because 00='0', 11='1' */
+				val = (val==0) ? AN_0 : AN_1;
 	
 				tmval = time_offsminus1 + (delta * reps);
 				for(rcnt=0;rcnt<reps;rcnt++)
 					{
-					if(val!=histent_head->v.val)
+					if(val!=histent_head->v.h_val)
 						{
 						htemp = histent_calloc();
-						htemp->v.val = val;
+						htemp->v.h_val = val;
 						htemp->time = tmval;
 						htemp->next = histent_head;
 						histent_head = htemp;
@@ -1994,17 +2120,17 @@ while(offs)
 						}
 	
 					tmval-=delta;
-					val=val^3;
+					val= (val==AN_0) ? AN_1: AN_0;
 					}
 				}
 				else
 				{
-				int val=2;
+				int val=AN_Z;
 	
-				if(val!=histent_head->v.val)
+				if(val!=histent_head->v.h_val)
 					{
 					htemp = histent_calloc();
-					htemp->v.val = val;
+					htemp->v.h_val = val;
 					htemp->time = time_offsminus1 + delta;
 					htemp->next = histent_head;
 					histent_head = htemp;
@@ -2033,16 +2159,16 @@ while(offs)
 				{
 				switch(parsed[0])
 					{
-				        case '0':       val = 0; break;
-				        case 'x':	val = 1; break;
-				        case 'z':	val = 2; break;
-				        case '1':       val = 3; break;
+				        case '0':       val = AN_0; break;
+				        case 'x':	val = AN_X; break;
+				        case 'z':	val = AN_Z; break;
+				        case '1':       val = AN_1; break;
 				        }
 	
-				if(val!=histent_head->v.val)
+				if(val!=histent_head->v.h_val)
 					{
 					htemp = histent_calloc();
-					htemp->v.val = val;
+					htemp->v.h_val = val;
 					htemp->time = tmval;
 					htemp->next = histent_head;
 					histent_head = htemp;
@@ -2055,11 +2181,11 @@ while(offs)
 				}
 				else
 				{
-				if(memcmp(parsed, histent_head->v.vector, len))
+				if(memcmp(parsed, histent_head->v.h_vector, len))
 					{
 					htemp = histent_calloc();
-					htemp->v.vector = (char *)malloc_2(len);
-					memcpy(htemp->v.vector, parsed, len);
+					htemp->v.h_vector = (char *)malloc_2(len);
+					memcpy(htemp->v.h_vector, parsed, len);
 					htemp->time = tmval;
 					htemp->next = histent_head;
 					histent_head = htemp;
@@ -2081,13 +2207,13 @@ while(offs)
 
 			if(double_is_native)
 				{
-				htemp->v.vector = ((unsigned char *)mm+offs_dbl);
+				htemp->v.h_vector = ((unsigned char *)mm+offs_dbl);
 				DEBUG(printf(LXTHDR"Added double '%.16g'\n", *((double *)(mm+offs_dbl))));
 				}
 				else
 				{
-				htemp->v.vector = swab_double_via_mask(offs_dbl);
-				DEBUG(printf(LXTHDR"Added bytefixed double '%.16g'\n", *((double *)(htemp->v.vector))));
+				htemp->v.h_vector = swab_double_via_mask(offs_dbl);
+				DEBUG(printf(LXTHDR"Added bytefixed double '%.16g'\n", *((double *)(htemp->v.h_vector))));
 				}
 			htemp->time = tmval;
 			htemp->next = histent_head;
@@ -2101,7 +2227,7 @@ while(offs)
 			htemp = histent_calloc();
 			htemp->flags = HIST_REAL|HIST_STRING;
 
-			htemp->v.vector = ((unsigned char *)mm+offs_str);
+			htemp->v.h_vector = ((unsigned char *)mm+offs_str);
 			DEBUG(printf(LXTHDR"Added string '%s'\n", (unsigned char *)mm+offs_str));
 			htemp->time = tmval;
 			htemp->next = histent_head;
@@ -2132,7 +2258,7 @@ while(offs)
 			break;
 
 		default:
-			fprintf(stderr, "Unknown %02x at offset: %08x\n", v, offs);
+			fprintf(stderr, "Unknown %02x at offset: %08x\n", v, (unsigned int)offs);
 			exit(0);
 		}
 
@@ -2157,12 +2283,12 @@ for(j=0;j>-2;j--)
 				}
 				else
 				{
-				init = 1; /* x if unspecified */
+				init = AN_X; /* x if unspecified */
 				}
 		
 			if(len>1)
 				{
-				char *pnt = htemp->v.vector = (char *)malloc_2(len);	/* zeros */
+				char *pnt = htemp->v.h_vector = (char *)malloc_2(len);	/* zeros */
 				int i;
 				for(i=0;i<len;i++)
 					{
@@ -2171,7 +2297,7 @@ for(j=0;j>-2;j--)
 				}
 				else
 				{
-				htemp->v.val = init;
+				htemp->v.h_val = init;
 				}
 			}
 			else
@@ -2193,12 +2319,12 @@ if(!(f->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 	{
 	if(len>1)
 		{
-		np->head.v.vector = (char *)malloc_2(len);
-		for(i=0;i<len;i++) np->head.v.vector[i] = 1;
+		np->head.v.h_vector = (char *)malloc_2(len);
+		for(i=0;i<len;i++) np->head.v.h_vector[i] = AN_X;
 		}
 		else
 		{
-		np->head.v.val = 1;                     /* 'x' */
+		np->head.v.h_val = AN_X;                     /* 'x' */
 		}
 	}
 	else
