@@ -1,6 +1,19 @@
+/*
+ * Copyright (c) Tony Bybell 2001-2004.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ */
+
 #ifndef WAVE_LXT_H
 #define WAVE_LXT_H
 
+#if defined __MINGW32__ || defined _MSC_VER
+#define fseeko fseek
+#endif
+
 #include "vcd.h"
 
 TimeType lxt_main(char *fname);
@@ -39,4 +52,6 @@ void import_lxt_trace(nptr np);
 
 #define LT_MINDICTWIDTH (16)
 
+#define LXT_MMAP_MALLOC_BOUNDARY		(128*1024*1024)
+
 #endif
