@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2003.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -9,33 +9,52 @@
 
 #include <gtk/gtk.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #ifdef __linux__
 			/* need to find getopt_long for non glibc platforms! */
 #include <getopt.h>
 #else
+#ifndef _MSC_VER
 #include <unistd.h>
+#else
+#include "getopt.c"
+#define strcasecmp _stricmp
+#endif
 #endif
 
 #include "aet.h"
 #include "lx2.h"
+#include "ae2.h"
+#include "vzt.h"
+#include "ghw.h"
 #include "main.h"
 #include "menu.h"
 #include "vcd.h"
 #include "lxt.h"
 #include "lxt2_read.h"
+#include "vzt_read.h"
 #include "pixmaps.h"
 #include "currenttime.h"
 #include "fgetdynamic.h"
 #include "rc.h"
+#include "translate.h"
+#include "ptranslate.h"
 
 char *whoami=NULL;
+struct logfile_chain *logfile=NULL;
 
 GtkWidget *signalwindow;
 GtkWidget *wavewindow;
 char paned_pack_semantics=1;	/* 1 for paned_pack, 0 for paned_add */
 char zoom_was_explicitly_set=0; /* set on '*' encountered in .sav file read  */
-int initial_window_x=700, initial_window_y=400; /* initial window sizes */
+#if defined _MSC_VER || defined __MINGW32__
+int initial_window_x=800, initial_window_y=400; /* initial window sizes */
+#else
+int initial_window_x=740, initial_window_y=400; /* initial window sizes */
+#endif
+
+int num_cpus=1;
 int initial_window_xpos=-1, initial_window_ypos=-1; /* initial window position (-1 == WM choice) */
 char use_scrollbar_only=0;
 char force_toolbars=0;		/* 1 sez use toolbar rendering */
@@ -47,48 +66,100 @@ static void print_help(char *nam)
 
 printf(
 "Usage: %s [OPTION]... [DUMPFILE] [SAVEFILE] [RCFILE]\n\n"
+"  -n, --nocli=DIRPATH        use file requester for dumpfile name\n"
 "  -f, --dump=FILE            specify dumpfile name\n"
 "  -a, --save=FILE            specify savefile name\n"
 "  -r, --rcfile=FILE          specify override .rcfile name\n"
-"  -s, --start=TIME           specify start time for LXT2 block skip\n"    
-"  -e, --end=TIME             specify end time for LXT2 block skip\n"    
+"  -d, --defaultskip          if missing .rcfile, do not use useful defaults\n"
+"  -i, --indirect=FILE        specify indirect facs file name\n"
+"  -l, --logfile=FILE         specify simulation logfile name for time values\n"
+"  -s, --start=TIME           specify start time for LXT2/VZT block skip\n"    
+"  -e, --end=TIME             specify end time for LXT2/VZT block skip\n"    
+"  -c, --cpu=NUMCPUS          specify number of CPUs for parallelizable ops\n"
 "  -v, --vcd                  use stdin as a VCD dumpfile\n"
-"  -h, --help                 display this help and exit\n"
+"  -V, --version              display version banner then exit\n"
+"  -h, --help                 display this help then exit\n"
 "  -x  --exit                 exit after loading trace (for loader benchmarks)\n\n"
 
-"VCD files and save files may be compressed with zip or gzip.  LXT and\n"
-"LXT2 files must remain uncompressed due to their non-linear access.\n"
-"Note that DUMPFILE is optional provided the --dump option is specified.\n"
+"VCD files and save files may be compressed with zip or gzip.\n"
+"Other formats must remain uncompressed due to their non-linear access.\n"
+"Note that DUMPFILE is optional if the --dump or --nocli options are specified.\n"
 "SAVEFILE and RCFILE are always optional.\n\n"
 
-"Report bugs to <bybell@linux-workshop.com>.\n",nam);
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
 
 #else
 
+#if !defined _MSC_VER && !defined __MINGW32__ && !defined __FreeBSD__
+#define WAVE_GETOPT_CPUS "  -c NUMCPUS                 specify number of CPUs for parallelizable ops\n"
+#else
+#define WAVE_GETOPT_CPUS
+#endif
+
 printf(
 "Usage: %s [OPTION]... [DUMPFILE] [SAVEFILE] [RCFILE]\n\n"
+"  -n DIRPATH                 use file requester for dumpfile name\n"
 "  -f FILE                    specify dumpfile name\n"
 "  -a FILE                    specify savefile name\n"
 "  -r FILE                    specify override .rcfile name\n"
-"  -s TIME                    specify start time for LXT2 block skip\n"    
-"  -e TIME                    specify end time for LXT2 block skip\n"    
+"  -i FILE                    specify indirect facs file name\n"
+"  -l FILE                    specify simulation logfile name for time values\n"
+"  -d                         if missing .rcfile, do not use useful defaults\n"
+"  -s TIME                    specify start time for LXT2/VZT block skip\n"    
+"  -e TIME                    specify end time for LXT2/VZT block skip\n"    
+WAVE_GETOPT_CPUS
 "  -v                         use stdin as a VCD dumpfile\n"
-"  -h                         display this help and exit\n"
+"  -V                         display version banner then exit\n"
+"  -h                         display this help then exit\n"
 "  -x                         exit after loading trace (for loader benchmarks)\n\n"
 
-"VCD files and save files may be compressed with zip or gzip.  LXT and\n"
-"LXT2 files must remain uncompressed due to their non-linear access.\n"
-"Note that DUMPFILE is optional provided the -f option is specified.\n"
+"VCD files and save files may be compressed with zip or gzip.\n"
+"Other formats must remain uncompressed due to their non-linear access.\n"
+"Note that DUMPFILE is optional provided the -f or -n options are specified.\n"
 "SAVEFILE and RCFILE are always optional.\n\n"
 
-"Report bugs to <bybell@linux-workshop.com>.\n",nam);
+"Report bugs to <bybell@nc.rr.com>.\n",nam);
 
+#undef WAVE_GETOPT_CPUS
 #endif
 
 exit(0);
 }
 
 
+/*
+ * file selection for -n/--nocli flag 
+ */
+static char *ftext_main = NULL;
+
+static void wave_get_filename_cleanup(GtkWidget *widget, gpointer data) { gtk_main_quit(); /* do nothing but exit gtk loop */ }
+
+static char *wave_get_filename(char *dfile)
+{
+if(dfile) 
+	{
+	int len = strlen(dfile);
+	ftext_main = malloc_2(strlen(dfile)+2);
+	strcpy(ftext_main, dfile);
+#if !defined _MSC_VER && !defined __MINGW32__
+	if((len)&&(dfile[len-1]!='/'))
+		{
+		strcpy(ftext_main + len, "/");
+		}
+#else
+	if((len)&&(dfile[len-1]!='\\'))
+		{
+		strcpy(ftext_main + len, "\\");
+		}
+#endif
+	}
+fileselbox("GTKWave: Select a dumpfile...",&ftext_main,GTK_SIGNAL_FUNC(wave_get_filename_cleanup), GTK_SIGNAL_FUNC(wave_get_filename_cleanup), NULL);
+gtk_main();
+
+return(ftext_main);
+}
+
+
 int main(int argc, char *argv[])
 {
 int i;
@@ -105,6 +176,7 @@ static char *winprefix="GTKWave - ";
 static char *winstd="GTKWave (stdio) ";
 static char *vcd_autosave_name="vcd_autosave.sav";
 FILE *wave;
+char *indirect_fname=NULL;
 
 GtkWidget *window;
 GtkWidget *main_vbox, *top_table, *whole_table;
@@ -119,7 +191,9 @@ GtkWidget *entry;
 GtkWidget *timebox;
 GtkWidget *panedwindow;
 GtkWidget *dummy1, *dummy2;
-GtkWidget *toolbar=NULL, *toolhandle=NULL;
+GtkWidget *toolhandle=NULL;
+
+int splash_disable_rc_override = 0;
 
 char *skip_start=NULL, *skip_end=NULL;
 
@@ -132,6 +206,10 @@ if(!gtk_init_check(&argc, &argv))
 	print_help(argv[0]);
 	}
 
+init_filetrans_data(); /* for file translation splay trees */
+init_proctrans_data(); /* for proc translation structs */
+atexit(remove_all_proc_filters);
+
 while (1)
         {
         int option_index = 0;
@@ -140,25 +218,39 @@ while (1)
         static struct option long_options[] =
                 {
                 {"dump", 1, 0, 'f'},
+                {"nocli", 1, 0, 'n'},
                 {"save", 1, 0, 'a'},
 		{"rcfile", 1, 0, 'r'},
+		{"defaultskip", 0, 0, 'd'},
+		{"indirect", 1, 0, 'i'},
+		{"logfile", 1, 0, 'l'},
                 {"start", 1, 0, 's'},
                 {"end", 1, 0, 'e'},
+                {"cpus", 1, 0, 'c'},
                 {"vcd", 0, 0, 'v'},
+                {"version", 0, 0, 'V'},
                 {"help", 0, 0, 'h'},
                 {"exit", 0, 0, 'x'},
                 {0, 0, 0, 0}
                 };
 
-        c = getopt_long (argc, argv, "f:a:r:s:e:vhx", long_options, &option_index);
+        c = getopt_long (argc, argv, "f:n:a:r:di:l:s:e:c:vVhx", long_options, &option_index);
 #else
-	c = getopt      (argc, argv, "f:a:r:s:e:vhx");
+	c = getopt      (argc, argv, "f:n:a:r:di:l:s:e:c:vVhx");
 #endif
 
         if (c == -1) break;     /* no more args */
 
         switch (c)
                 {
+		case 'V':
+			printf(
+			WAVE_VERSION_INFO"\n\n"
+			"This is free software; see the source for copying conditions.  There is NO\n"
+			"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+			);
+			exit(0);
+
                 case 'v':
 			is_vcd = 1;
 			if(fname) free_2(fname);
@@ -166,14 +258,29 @@ while (1)
 			strcpy(fname, "-vcd");
                         break;
 
+		case 'n':
+			wave_get_filename(optarg);
+			if(filesel_ok)
+				{
+				if(fname) free_2(fname);
+				fname = ftext_main;
+				ftext_main = NULL;
+				}
+			break;
+
                 case 'h':
 			print_help(argv[0]);
                         break;
 
                 case 'x':
 			fast_exit = 1;
+			splash_disable_rc_override = 1;
                         break;
 
+		case 'd':
+			possibly_use_rc_defaults = 0;
+			break;
+
                 case 'f':
 			is_vcd = 0;
 			if(fname) free_2(fname);
@@ -193,18 +300,52 @@ while (1)
 			strcpy(override_rc, optarg);
 			break;
 
+                case 'i':
+			if(indirect_fname) free_2(indirect_fname);
+			indirect_fname = malloc_2(strlen(optarg)+1);
+			strcpy(indirect_fname, optarg);
+			break;
+
                 case 's':
-			if(skip_start) free(skip_start);
+			if(skip_start) free_2(skip_start);
 			skip_start = malloc_2(strlen(optarg)+1);
 			strcpy(skip_start, optarg);
 			break;			
 
                 case 'e':
-			if(skip_end) free(skip_end);
+			if(skip_end) free_2(skip_end);
 			skip_end = malloc_2(strlen(optarg)+1);
 			strcpy(skip_end, optarg);
                         break;
 
+                case 'c':
+#if !defined _MSC_VER && !defined __MINGW32__
+			num_cpus = atoi(optarg);
+			if(num_cpus<1) num_cpus = 1;
+			if(num_cpus>8) num_cpus = 8;
+#endif
+                        break;
+
+		case 'l':
+			{
+			struct logfile_chain *l = calloc_2(1, sizeof(struct logfile_chain));
+			struct logfile_chain *ltraverse;
+			l->name = malloc_2(strlen(optarg)+1);
+			strcpy(l->name, optarg);
+
+			if(logfile)
+				{
+				ltraverse = logfile;
+				while(ltraverse->next) ltraverse = ltraverse->next;
+				ltraverse->next = l;
+				}
+				else
+				{
+				logfile = l;
+				}
+			}
+			break;
+
                 case '?':
 			opt_errors_encountered=1;
                         break;
@@ -250,7 +391,10 @@ if(!fname)
 	}
 
 read_rc_file(override_rc);
-printf("\n%s\n\n",WAVE_VERSION_INFO);
+splash_disable |= splash_disable_rc_override;
+
+
+fprintf(stderr, "\n%s\n\n",WAVE_VERSION_INFO);
 
 if((!wname)&&(make_vcd_save_file))
 	{
@@ -274,7 +418,7 @@ if(is_vcd)
 
 strcat(winname,fname);
 
-if((strlen(fname)>3)&&((!strcmp(fname+strlen(fname)-4,".lxt"))||(!strcmp(fname+strlen(fname)-4,".lx2"))))
+if((strlen(fname)>3)&&((!strcasecmp(fname+strlen(fname)-4,".lxt"))||(!strcasecmp(fname+strlen(fname)-4,".lx2"))))
 	{
 	FILE *f = fopen(fname, "rb");
 	int typ = 0;
@@ -299,22 +443,54 @@ if((strlen(fname)>3)&&((!strcmp(fname+strlen(fname)-4,".lxt"))||(!strcmp(fname+s
 		else
 		{
 		lx2_main(fname, skip_start, skip_end);
-		if(skip_start)
-			{
-			free(skip_start); skip_start=NULL;
-			}
-		if(skip_end)
-			{
-			free(skip_end); skip_end=NULL;
-			}
 		}	
-
+	}
+else
+if((strlen(fname)>3)&&(!strcasecmp(fname+strlen(fname)-4,".vzt")))
+	{
+	vzt_main(fname, skip_start, skip_end);
+	}
+else if ((strlen(fname)>3)&&((!strcasecmp(fname+strlen(fname)-4,".aet"))||(!strcasecmp(fname+strlen(fname)-4,".ae2"))))
+	{
+	ae2_main(fname, skip_start, skip_end, indirect_fname);
+	}
+else if ((strlen(fname)>3)&&(!strcasecmp(fname+strlen(fname)-4,".ghw")))
+	{
+	ghw_main(fname);
+	}
+else if (strlen(fname)>4)	/* case for .aet? type filenames */
+	{
+	char sufbuf[5];
+	memcpy(sufbuf, fname+strlen(fname)-5, 4);
+	sufbuf[4] = 0;
+	if(!strcasecmp(sufbuf, ".aet"))	/* strncasecmp() in windows? */
+		{
+		ae2_main(fname, skip_start, skip_end, indirect_fname);
+		}
+		else
+		{
+		vcd_main(fname);
+		}
 	}
 else	/* nothing else left so default to "something" */
 	{
 	vcd_main(fname);
 	}
 
+if(indirect_fname)
+	{
+	free_2(indirect_fname);
+	indirect_fname=NULL;
+	}
+if(skip_start)
+	{
+	free_2(skip_start); skip_start=NULL;
+	}
+if(skip_end)
+	{
+	free_2(skip_end); skip_end=NULL;
+	}
+
 for(i=0;i<26;i++) named_markers[i]=-1;	/* reset all named markers */
 
 tims.last=max_time;
@@ -322,11 +498,12 @@ tims.end=tims.last;		/* until the configure_event of wavearea */
 tims.first=tims.start=tims.laststart=min_time;
 tims.zoom=tims.prevzoom=0;	/* 1 pixel/ns default */
 tims.marker=tims.lmbcache=-1;	/* uninitialized at first */
+tims.baseline=-1;		/* middle button toggle marker */
 
 if((wname)||(vcd_save_handle))
 	{
 	int wave_is_compressed;
-        char *str;
+        char *str = NULL;
 
 	if(vcd_save_handle)
 		{
@@ -334,8 +511,8 @@ if((wname)||(vcd_save_handle))
 		do_initial_zoom_fit=1;
 		}
 
-	if(((strlen(wname)>2)&&(!strcmp(wname+strlen(wname)-3,".gz")))||
-	   ((strlen(wname)>3)&&(!strcmp(wname+strlen(wname)-4,".zip"))))
+	if(((strlen(wname)>2)&&(!strcasecmp(wname+strlen(wname)-3,".gz")))||
+	   ((strlen(wname)>3)&&(!strcasecmp(wname+strlen(wname)-4,".zip"))))
 	        {
         	int dlen;
         	dlen=strlen(WAVE_DECOMPRESSOR);
@@ -349,6 +526,9 @@ if((wname)||(vcd_save_handle))
 	        {
 	        wave=fopen(wname,"rb");
 	        wave_is_compressed=0;
+
+		filesel_writesave = malloc_2(strlen(wname)+1); /* don't handle compressed files */
+		strcpy(filesel_writesave, wname);
 	        }
 
 	if(!wave)
@@ -368,7 +548,12 @@ if((wname)||(vcd_save_handle))
 				free_2(iline);
 		                }
 
-			lx2_import_masked();
+			switch(is_lx2)
+				{
+				case LXT2_IS_LXT2: lx2_import_masked(); break;
+				case LXT2_IS_AET2: ae2_import_masked(); break;
+				case LXT2_IS_VZT:  vzt_import_masked(); break;
+				}
 
 			if(wave_is_compressed)
 			        {
@@ -418,6 +603,8 @@ if((wname)||(vcd_save_handle))
 	}
 
 savefile_bail:
+current_translate_file = 0;
+
 if(fast_exit)
 	{
 	printf("Exiting early because of --exit request.\n");
@@ -436,6 +623,8 @@ gtk_signal_connect(GTK_OBJECT(window), "destroy",
 			GTK_SIGNAL_FUNC(file_quit_cmd_callback), 
 		       	"WM destroy");
 gtk_widget_show(window);
+
+#if !defined __MINGW32__ && !defined _MSC_VER
 if((initial_window_xpos>=0)||(initial_window_ypos>=0))
 	{
 	if (initial_window_xpos<0) { initial_window_xpos = 0; }
@@ -446,6 +635,8 @@ if((initial_window_xpos>=0)||(initial_window_ypos>=0))
 	gtk_window_reposition(GTK_WINDOW(window), initial_window_xpos, initial_window_ypos);
 #endif
 	}
+#endif
+
 make_pixmaps(window);
 
 main_vbox = gtk_vbox_new(FALSE, 5);
@@ -468,7 +659,7 @@ if(force_toolbars)
 	gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
 	}
 
-top_table = gtk_table_new (1, 256, FALSE);
+top_table = gtk_table_new (1, 284, FALSE);
 
 if(force_toolbars)
 	{
@@ -481,20 +672,20 @@ whole_table = gtk_table_new (256, 16, FALSE);
 
 text1 = create_text ();
 gtk_table_attach (GTK_TABLE (top_table), text1, 0, 141, 0, 1,
-                      	GTK_FILL | GTK_EXPAND,
+                      	GTK_FILL,
                       	GTK_FILL | GTK_SHRINK, 0, 0);
 gtk_widget_set_usize(GTK_WIDGET(text1), 200, -1);
 gtk_widget_show (text1);
 
 dummy1=gtk_label_new("");
 gtk_table_attach (GTK_TABLE (top_table), dummy1, 141, 171, 0, 1,
-                      	GTK_FILL | GTK_EXPAND,
+                      	GTK_FILL,
                       	GTK_SHRINK, 0, 0);
 gtk_widget_show (dummy1);
 
 zoombuttons = create_zoom_buttons ();
 gtk_table_attach (GTK_TABLE (top_table), zoombuttons, 171, 173, 0, 1,
-                      	GTK_FILL | GTK_EXPAND,
+                      	GTK_FILL,
                       	GTK_SHRINK, 0, 0);
 gtk_widget_show (zoombuttons);
 
@@ -502,30 +693,30 @@ if(!use_scrollbar_only)
 	{
 	pagebuttons = create_page_buttons ();
 	gtk_table_attach (GTK_TABLE (top_table), pagebuttons, 173, 174, 0, 1,
-	                      	GTK_FILL | GTK_EXPAND,
+	                      	GTK_FILL,
 	                      	GTK_SHRINK, 0, 0);
 	gtk_widget_show (pagebuttons);
 	fetchbuttons = create_fetch_buttons ();
 	gtk_table_attach (GTK_TABLE (top_table), fetchbuttons, 174, 175, 0, 1,
-	                      	GTK_FILL | GTK_EXPAND,
+	                      	GTK_FILL,
 	                      	GTK_SHRINK, 0, 0);
 	gtk_widget_show (fetchbuttons);
 	discardbuttons = create_discard_buttons ();
 	gtk_table_attach (GTK_TABLE (top_table), discardbuttons, 175, 176, 0, 1,
-	                      	GTK_FILL | GTK_EXPAND,
+	                      	GTK_FILL,
 	                      	GTK_SHRINK, 0, 0);
 	gtk_widget_show (discardbuttons);
 	
 	shiftbuttons = create_shift_buttons ();
 	gtk_table_attach (GTK_TABLE (top_table), shiftbuttons, 176, 177, 0, 1,
-	                      	GTK_FILL | GTK_EXPAND,
+	                      	GTK_FILL,
 	                      	GTK_SHRINK, 0, 0);
 	gtk_widget_show (shiftbuttons);
 	}
 
 dummy2=gtk_label_new("");
 gtk_table_attach (GTK_TABLE (top_table), dummy2, 177, 215, 0, 1,
-                      	GTK_FILL | GTK_EXPAND,
+                      	GTK_FILL,
                       	GTK_SHRINK, 0, 0);
 gtk_widget_show (dummy2);
 
@@ -536,9 +727,9 @@ gtk_table_attach (GTK_TABLE (top_table), entry, 215, 216, 0, 1,
 gtk_widget_show(entry);
 
 timebox = create_time_box();
-gtk_table_attach (GTK_TABLE (top_table), timebox, 216, 256, 0, 1,
+gtk_table_attach (GTK_TABLE (top_table), timebox, 216, 284, 0, 1,
                       	GTK_FILL | GTK_EXPAND,
-                      	GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
+                      	GTK_FILL | GTK_EXPAND | GTK_SHRINK, 20, 0);
 gtk_widget_show (timebox);
 
 wavewindow = create_wavewindow();
@@ -550,7 +741,7 @@ if(do_resize_signals)
                 {
                 int os;
                 os=max_signal_name_pixel_width;
-                os=(os<30)?30:os;
+                os=(os<48)?48:os;
                 gtk_widget_set_usize(GTK_WIDGET(signalwindow),
                                 os+30, -1);
                 }
@@ -587,6 +778,7 @@ gtk_container_add (GTK_CONTAINER (main_vbox), whole_table);
 
 update_markertime(time_trunc(tims.marker));
 
+#if !defined __MINGW32__ && !defined _MSC_VER
 if((initial_window_xpos>=0)||(initial_window_ypos>=0))
 	{
 	if (initial_window_xpos<0) { initial_window_xpos = 0; }
@@ -597,8 +789,25 @@ if((initial_window_xpos>=0)||(initial_window_ypos>=0))
 	gtk_window_reposition(GTK_WINDOW(window), initial_window_xpos, initial_window_ypos);
 #endif
 	}
+#endif
+
+if(logfile) 
+	{
+	struct logfile_chain *lprev;
+	char buf[50];
+	int which = 1;
+	while(logfile)
+		{
+		sprintf(buf, "Logfile viewer [%d]", which++);
+		logbox(buf, 480, logfile->name);
+		lprev = logfile;
+		logfile = logfile->next;
+		free_2(lprev->name);
+		free_2(lprev);
+		}
+	}
 
 gtk_main();
-    
+
 return(0);
 }
