@@ -12,6 +12,12 @@
 
 #define HAVE_PANED_PACK	/* undefine this if you have an older GTK */
 
+struct logfile_chain
+{
+struct logfile_chain *next;
+char *name;
+};
+
 extern char *whoami;
 
 GtkWidget *create_text(void);
@@ -30,5 +36,7 @@ extern GtkWidget *wavewindow;
 extern char paned_pack_semantics;    /* 1 for paned_pack, 0 for paned_add */
 extern int initial_window_x, initial_window_y; /* inital window sizes */
 extern int initial_window_xpos, initial_window_ypos; /* inital window position */
+extern int splash_disable;
+extern int possibly_use_rc_defaults;
 
 #endif
