@@ -9,6 +9,7 @@
 
 #include <gtk/gtk.h>
 #include <string.h>
+#include "gtk12compat.h"
 #include "debug.h"
 #include "analyzer.h"
 #include "currenttime.h"
@@ -25,7 +26,7 @@ static void enter_callback(GtkWidget *widget, gpointer which)
 GtkWidget *entry;
 TimeType *modify;
 TimeType temp;
-gchar *entry_text;
+G_CONST_RETURN gchar *entry_text;
 char buf[49];
 int len, i;
 int ent_idx;
@@ -95,6 +96,7 @@ void markerbox(char *title, GtkSignalFunc func)
     GtkWidget *entry;
     GtkWidget *vbox, *hbox, *vbox_g, *label;
     GtkWidget *button1, *button2, *scrolled_win, *frame, *separator;
+    GtkWidget *table;
     char labtitle[2]={0,0};
     int i;
 
@@ -111,12 +113,18 @@ void markerbox(char *title, GtkSignalFunc func)
                        (GtkSignalFunc) destroy_callback, NULL);
 
     vbox = gtk_vbox_new (FALSE, 0);
-    gtk_container_add (GTK_CONTAINER (window), vbox);
     gtk_widget_show (vbox);
 
     vbox_g = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox_g);
 
+    table = gtk_table_new (256, 1, FALSE);
+    gtk_widget_show (table);
+
+    gtk_table_attach (GTK_TABLE (table), vbox, 0, 1, 0, 255,
+                        GTK_FILL | GTK_EXPAND,
+                        GTK_FILL | GTK_EXPAND | GTK_SHRINK, 1, 1);
+
     frame = gtk_frame_new (NULL);
     gtk_container_border_width (GTK_CONTAINER (frame), 3);
     gtk_widget_show(frame);
@@ -167,9 +175,12 @@ void markerbox(char *title, GtkSignalFunc func)
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_win), vbox_g);
 
     hbox = gtk_hbox_new (FALSE, 1);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
     gtk_widget_show (hbox);
 
+    gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 255, 256,
+                        GTK_FILL | GTK_EXPAND,
+                        GTK_FILL | GTK_EXPAND | GTK_SHRINK, 1, 1);
+
     button1 = gtk_button_new_with_label ("OK");
     gtk_widget_set_usize(button1, 100, -1);
     gtk_signal_connect(GTK_OBJECT (button1), "clicked",
@@ -193,5 +204,6 @@ void markerbox(char *title, GtkSignalFunc func)
     gtk_widget_show (button2);
     gtk_container_add (GTK_CONTAINER (hbox), button2);
 
+    gtk_container_add (GTK_CONTAINER (window), table); /* need this table to keep ok/cancel buttons from stretching! */
     gtk_widget_show(window);
 }
