@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,9 @@ if(helpbox_is_active)
         help_text(
                 " scrolls the display window left one page worth of data."
 		"  The net action is that the data scrolls right a page."
+#ifdef WAVE_USE_GTK2
+                " Scrollwheel Up also performs this function."
+#endif
         );
         return;
         }
@@ -52,6 +55,9 @@ if(helpbox_is_active)
         help_text(
 		" scrolls the display window right one page worth of data."
 		"  The net action is that the data scrolls left a page."
+#ifdef WAVE_USE_GTK2
+                " Scrollwheel Down also performs this function."
+#endif
         );
         return;
         }
@@ -94,9 +100,9 @@ GtkTooltips *tooltips;
 tooltips=gtk_tooltips_new_2();
 gtk_tooltips_set_delay_2(tooltips,1500);
 
-pixmapwid1=gtk_pixmap_new(larrow_pixmap, larrow_mask);
+pixmapwid1=gtk_pixmap_new(prev_page_pixmap, prev_page_mask);
 gtk_widget_show(pixmapwid1);
-pixmapwid2=gtk_pixmap_new(rarrow_pixmap, rarrow_mask);
+pixmapwid2=gtk_pixmap_new(next_page_pixmap, next_page_mask);
 gtk_widget_show(pixmapwid2);
    
 /* Create a table to hold the text widget and scrollbars */
