@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,4 +33,25 @@ extern GdkBitmap *zoom_larrow_mask;
 extern GdkPixmap *zoom_rarrow_pixmap;
 extern GdkBitmap *zoom_rarrow_mask;
 
+GdkPixmap *prev_page_pixmap;
+GdkBitmap *prev_page_mask;
+GdkPixmap *next_page_pixmap;
+GdkBitmap *next_page_mask;
+
+GdkPixmap *wave_info_pixmap;
+GdkBitmap *wave_info_mask;
+GdkPixmap *wave_alert_pixmap;
+GdkBitmap *wave_alert_mask;
+
+#ifdef WAVE_USE_GTK2
+
+#define WAVE_SPLASH_X (512)
+#define WAVE_SPLASH_Y (384)
+
+void make_splash_pixmaps(GtkWidget *window);
+
+GdkPixmap *wave_splash_pixmap;
+GdkBitmap *wave_splash_mask;
+#endif
+
 #endif
