@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Tony Bybell 1999-2001. 
+ * Copyright (c) Tony Bybell 1999-2005. 
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free
@@ -610,9 +610,9 @@ ps_MaxSignalLength(void)
 	t = traces.first;
 	trwhich = 0;
 	while (t) {
-		if ((trwhich < trtarget) && (t->next)) {
+		if ((trwhich < trtarget) && (GiveNextTrace(t))) {
 			trwhich++;
-			t = t->next;
+			t = GiveNextTrace(t);
 		} else {
 			break;
 		}
@@ -632,7 +632,7 @@ ps_MaxSignalLength(void)
 			bufxlen = bufclen = 0;
 		}
 
-		if (t->flags & TR_BLANK) {
+		if (t->flags & (TR_BLANK|TR_ANALOG_BLANK_STRETCH)) {
 			if (t->name) {
 				len = gdk_string_measure(signalfont, t->name) + bufxlen;
 				numchars = strlen(t->name) + bufclen;
@@ -687,9 +687,9 @@ ps_MaxSignalLength(void)
 							str = (char *) calloc_2(1, 3 * sizeof(char));
 							str[0] = '=';
 							if (t->flags & TR_INVERT) {
-								str[1] = "1XZ0"[hptr->v.val];
+								str[1] = AN_STR_INV[hptr->v.h_val];
 							} else {
-								str[1] = "0XZ1"[hptr->v.val];
+								str[1] = AN_STR[hptr->v.h_val];
 							}
 							t->asciivalue = str;
 							vlen = gdk_string_measure(signalfont, str);
@@ -701,14 +701,14 @@ ps_MaxSignalLength(void)
 								if (!(hptr->flags & HIST_STRING)) {
 									str =
 										convert_ascii_real((double *) hptr->
-												   v.vector);
+												   v.h_vector);
 								} else {
 									str =
 										convert_ascii_string((char *) hptr->
-												     v.vector);
+												     v.h_vector);
 								}
 							} else {
-								str = convert_ascii_vec(t, hptr->v.vector);
+								str = convert_ascii_vec(t, hptr->v.h_vector);
 							}
 							if (str) {
 								int             slen;
@@ -748,7 +748,7 @@ ps_MaxSignalLength(void)
 			if (numchars > ps_nummaxchars)
 				ps_nummaxchars = numchars;
 		}
-		t = t->next;
+		t = GiveNextTrace(t);
 	}
 
 	maxlen += 6;		/* endcap padding */
@@ -764,9 +764,11 @@ static void
 pr_renderhash(pr_context * prc, int x, TimeType tim)
 {
 	TimeType        rborder;
-	int             hashoffset;
+	gdouble		dx;
+	gdouble         hashoffset;
 	int             fhminus2;
 	int             rhs;
+	int		iter = 0;
 
 	fhminus2 = fontheight - 2;
 
@@ -788,13 +790,17 @@ pr_renderhash(pr_context * prc, int x, TimeType tim)
 		rhs = rborder;
 	pr_draw_line(prc, x, wavecrosspiece, rhs, wavecrosspiece);
 
-	x += (hashoffset = hashstep);
 
-	while ((hashoffset < pixelsperframe) && (x <= rhs)) {
+	dx = x + (hashoffset=hashstep);
+        x  = dx;
+
+	while ((hashoffset < pixelsperframe) && (x <= rhs) && (iter < 10)) {
 		pr_draw_line(prc, x, wavecrosspiece, x, fhminus2);
 
 		hashoffset += hashstep;
-		x += hashstep;
+	        dx = dx + hashstep;
+	        if((pixelsperframe!=200)||(hashstep!=10.0)) iter++; /* fix any roundoff errors */
+		x = dx;
 	}
 
 }
@@ -843,8 +849,9 @@ if(bt)
 static void
 pr_rendertimes(pr_context * prc)
 {
+        int             lastx = -1000; /* arbitrary */
 	TimeType        tim, rem;
-	int             x, len;
+	int             x, len, lenhalf;
 	char            timebuff[32];
 
 	pr_renderblackout(prc);
@@ -910,15 +917,21 @@ top:
 			strcpy(timebuff, "0");
 		}
 
-		len = gdk_string_measure(wavefont, timebuff);
+		len     = gdk_string_measure(wavefont, timebuff) >> 1;
+		lenhalf = len >> 1;
 
 		if ((prc->gpd == &ps_print_device)
-		|| ((x - (len >> 1) >= 0) && (x + (len >> 1) < wavewidth))) {
+		|| ((x - lenhalf >= 0) && (x + lenhalf < wavewidth))) {
 			pr_setgray(prc, 0.0);
-			pr_draw_string(prc,
-				       x - (len >> 1), wavefont->ascent + 1,
-				       timebuff, len, wavefont->ascent);
+			if((x-lenhalf >= lastx) || (pixelsperframe >= 200))
+				{
+				pr_draw_string(prc,
+					       x - lenhalf, wavefont->ascent + 1,
+					       timebuff, len, wavefont->ascent);
+			}
 		}
+		lastx = x + lenhalf;
+
 		tim += nsperframe;
 		x += pixelsperframe;
 		if ((x >= wavewidth) || (tim > tims.last))
@@ -936,11 +949,14 @@ pr_draw_named_markers(pr_context * prc)
 	int             i;
 	TimeType        t;
 
-	pixstep = ((gdouble) nsperframe) / ((gdouble) pixelsperframe);
-
 	for (i = 0; i < 26; i++) {
 		if ((t = named_markers[i]) != -1) {
 			if ((t >= tims.start) && (t <= tims.last) && (t <= tims.end)) {
+
+		                /* this needs to be here rather than outside the loop as gcc does some
+                		   optimizations that cause it to calculate slightly different from the marker if it's not here */
+				pixstep = ((gdouble) nsperframe) / ((gdouble) pixelsperframe);
+
 				xl = ((gdouble) (t - tims.start)) / pixstep;	/* snap to integer */
 				if ((xl >= 0) && (xl < wavewidth)) {
 					char            nbuff[2];
@@ -975,6 +991,18 @@ pr_draw_marker(pr_context * prc)
 	gdouble         pixstep;
 	gint            xl;
 
+	if (tims.baseline != -1) {
+		if ((tims.baseline >= tims.start) && (tims.baseline <= tims.last)
+		    && (tims.baseline <= tims.end)) {
+			pixstep = ((gdouble) nsperframe) / ((gdouble) pixelsperframe);
+			xl = ((gdouble) (tims.baseline - tims.start)) / pixstep; /* snap to integer */
+			if ((xl >= 0) && (xl < wavewidth)) {
+				pr_setgray(prc, 0.65);
+				pr_draw_line(prc, xl, fontheight - 1, xl, liney_max);
+			}
+		}
+	}
+
 	if (tims.marker != -1) {
 		if ((tims.marker >= tims.start) && (tims.marker <= tims.last)
 		    && (tims.marker <= tims.end)) {
@@ -995,13 +1023,14 @@ pr_draw_marker(pr_context * prc)
  * "excluded" traces 
  */
 static void 
-pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
+pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw, int kill_grid)
 {
 	TimeType        x0, x1, newtime;
-	int             y0, y1, yu, liney;
+	int             y0, y1, yu, liney, ytext, ysiz;
 	TimeType        tim, h2tim;
 	hptr            h2, h3;
 	char            hval, h2val;
+	char 		identifier_str[2];
 
 	tims.start -= shift_timebase;
 	tims.end -= shift_timebase;
@@ -1016,8 +1045,10 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 	}
 
 	yu = (y0 + y1) / 2;
+	ytext=yu-(wavefont->ascent/2)+wavefont->ascent;
+	ysiz = wavefont->ascent - 1;
 
-	if ((display_grid) && (enable_horiz_grid)) {
+	if ((display_grid) && (enable_horiz_grid) && (!kill_grid)) {
 		pr_setgray(prc, 0.75);
 		pr_draw_line(prc,
 			     (tims.start < tims.first) ? (tims.first - tims.start) * pxns : 0, liney,
@@ -1056,16 +1087,20 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 				x1 = wavewidth;
 			}
 			if (x0 != x1) {
-				hval = h->v.val;
-				h2val = h2->v.val;
+				hval = h->v.h_val;
+				h2val = h2->v.h_val;
 
 				switch (hval) {
-				case 0:	/* 0 */
+				case AN_0: /* 0 */
+				case AN_L: /* 0 */
 					pr_draw_line(prc, x0, y0, x1, y0);
 
 					if (h2tim <= tims.end)
 						switch (h2val) {
-						case 2:
+						case AN_0:
+						case AN_L: break;
+
+						case AN_Z:
 							pr_draw_line(prc, x1, y0, x1, yu);
 							break;
 						default:
@@ -1074,11 +1109,40 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 						}
 					break;
 
-				case 1:	/* X */
+				case AN_X: /* X */
+				case AN_W: /* X */
+				case AN_U: /* X */
+				case AN_DASH: /* X */
 					pr_setgray(prc, 0.70);
 					pr_draw_box(prc, x0, y0, x1, y1);
 					pr_setgray(prc, 0.0);
 
+			                identifier_str[1] = 0;
+			                switch(hval)
+			                        {
+			                        case AN_X:      identifier_str[0] = 0; break;
+			                        case AN_W:      identifier_str[0] = 'W'; break;
+			                        case AN_U:      identifier_str[0] = 'U'; break;
+			                        default:        identifier_str[0] = '-'; break;
+			                        }
+
+			                if(identifier_str[0])
+			                        {
+			                        int x0_new = (x0>=0) ? x0 : 0;
+			                        int width;
+						int pixlen =0;			
+
+			                        if((width=x1-x0_new)>vector_padding)
+			                                {
+			                                if((x1>=wavewidth)||((pixlen=gdk_string_measure(wavefont, identifier_str))+vector_padding<=width))
+			                                        {
+			                                        pr_draw_string(prc,
+			                                                x0+2,ytext,
+			                                                identifier_str, pixlen, ysiz);
+			                                        }
+			                                }
+			                        }
+
 					if (x0 >= 0)
 						pr_draw_line(prc, x0, y0, x0, y1);
 
@@ -1088,14 +1152,16 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 						pr_draw_line(prc, x1, y0, x1, y1);
 					break;
 
-				case 2:	/* Z */
+				case AN_Z: /* Z */
 					pr_draw_line(prc, x0, yu, x1, yu);
 					if (h2tim <= tims.end)
 						switch (h2val) {
-						case 0:
+						case AN_0:
+						case AN_L:
 							pr_draw_line(prc, x1, yu, x1, y0);
 							break;
-						case 3:
+						case AN_1:
+						case AN_H:
 							pr_draw_line(prc, x1, yu, x1, y1);
 							break;
 						default:
@@ -1104,21 +1170,26 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 						}
 					break;
 
-				case 3:	/* 1 */
+				case AN_1: /* 1 */
+				case AN_H: /* 1 */
 					pr_draw_line(prc, x0, y1, x1, y1);
 					if (h2tim <= tims.end)
 						switch (h2val) {
-						case 0:
+						case AN_1:
+						case AN_H: break;
+
+						case AN_0:
+						case AN_L:
 							pr_draw_line(prc, x1, y1, x1, y0);
 							break;
-						case 2:
+						case AN_Z:
 							pr_draw_line(prc, x1, y1, x1, yu);
 							break;
 						default:
 							pr_draw_line(prc, x1, y0, x1, y1);
 							break;
 						}
-					/* fallthrough on break */
+					break;
 
 				default:
 					break;
@@ -1144,6 +1215,154 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw)
 /*
  * draw hptr vectors (integer+real) 
  */
+static void 
+pr_draw_hptr_trace_vector_analog(pr_context * prc, Trptr t, hptr h, int which, int num_extension)
+{
+	TimeType        x0, x1, newtime, width;
+	int             y0, y1, yu, liney, ytext, yt0, yt1;
+	TimeType        tim, h2tim;
+	hptr            h2, h3;
+        int             endcnt = 0;
+	int             pixlen, ysiz;
+	int             type;
+	int             lasttype = -1;
+	double          mynan = strtod("NaN", NULL);
+	double          tmin = mynan, tmax = mynan, tv, tv2;
+
+	liney = ((which + 2 + num_extension) * fontheight) - 2;
+	y1 = ((which + 1) * fontheight) + 2;
+	y0 = liney - 2;
+	yu = (y0 + y1) / 2;
+	ytext = yu - (wavefont->ascent / 2) + wavefont->ascent;
+
+	ysiz = wavefont->ascent - 1;
+	if (ysiz < 1)
+		ysiz = 1;
+
+	h2 = h;
+	for(;;) {
+		if (!h2)
+			break;
+		tim = h2->time;
+		if(tim>tims.end) { endcnt++; if(endcnt==2) break; }
+		if(tim>tims.last) break; 
+		x0 = (tim - tims.start) * pxns;
+		if((x0>wavewidth)&&(endcnt==2)) break;
+		h3=h2;
+		h2 = h2->next;
+		if (!h2)
+			break;
+		tim = h2->time;
+		x1 = (tim - tims.start) * pxns;
+		if (x1 < 0)
+			continue;
+		tv = mynan;
+		if(h3->flags&HIST_REAL)	{
+				if(!(h3->flags&HIST_STRING) && h3->v.h_vector)
+					tv = *(double *)h3->v.h_vector;
+		} else {
+			tv=convert_real_vec(t,h3->v.h_vector);
+		}
+		if (!isnan(tv))	{
+			if (isnan(tmin) || tv < tmin)
+				tmin = tv;
+			if (isnan(tmax) || tv > tmax)
+				tmax = tv;
+		}
+	}
+	if (isnan(tmin) || isnan(tmax))
+		tmin = tmax = 0;
+	if ((tmax - tmin) < 1e-20) {
+		tmax = 1;
+		tmin -= 0.5 * (y1 - y0);
+	} else {
+		tmax = (y1 - y0) / (tmax - tmin);
+	}
+
+	pr_setgray(prc, 0.0);
+	for(;;)	{
+		if (!h)
+			break;
+		tim = (h->time);
+		if ((tim > tims.end) || (tim > tims.last))
+			break;
+
+		x0 = (tim - tims.start) * pxns;
+		if (x0 < -1)
+			x0 = -1;
+		else if (x0 > wavewidth)
+			break;
+
+		h2 = h->next;
+		if (!h2)
+			break;
+		h2tim = tim = (h2->time);
+		if (tim > tims.last)
+			tim = tims.last;
+		else if (tim > tims.end + 1)
+			tim = tims.end + 1;
+		x1 = (tim - tims.start) * pxns;
+
+		if (x1 < -1)
+			x1 = -1;
+		else if (x1 > wavewidth)
+			x1 = wavewidth;
+
+
+		/* draw trans */
+		type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.h_vector) : AN_0;
+		tv = tv2 = mynan;
+		if(h->flags&HIST_REAL) {
+			if(!(h->flags&HIST_STRING) && h->v.h_vector)
+				tv = *(double *)h->v.h_vector;
+		} else {
+			tv=convert_real_vec(t,h->v.h_vector);
+		}
+		if(h2->flags&HIST_REAL) {
+			if(!(h2->flags&HIST_STRING) && h2->v.h_vector)
+				tv2 = *(double *)h2->v.h_vector;
+		} else {
+			tv2=convert_real_vec(t,h2->v.h_vector);
+		}
+		if(isnan(tv))
+			yt0 = yu;
+		else
+			yt0 = y0 + (tv - tmin) * tmax;
+		if(isnan(tv2))
+			yt1 = yu;
+		else
+			yt1 = y0 + (tv2 - tmin) * tmax;
+		if (x0 != x1) {
+			if(t->flags & TR_ANALOG_STEP) {
+				pr_draw_line(prc,
+					     x0, yt0,
+					     x1, yt0);
+				pr_draw_line(prc,
+					     x1, yt0,
+					     x1, yt1);
+			} else {
+				pr_draw_line(prc,
+					     x0, yt0,
+					     x1, yt1);
+			}
+		} else {
+			newtime = (((gdouble) (x1 + WAVE_OPT_SKIP)) * nspx) + tims.start + shift_timebase;	/* skip to next pixel */
+			h3 = bsearch_node(t->n.nd, newtime);
+			if (h3->time > h->time) {
+				h = h3;
+				lasttype = type;
+				continue;
+			}
+		}
+
+		h = h->next;
+		lasttype = type;
+	}
+
+	tims.start += shift_timebase;
+	tims.end += shift_timebase;
+}
+
 static void 
 pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 {
@@ -1171,11 +1390,43 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 
 
 	if ((display_grid) && (enable_horiz_grid)) {
-		pr_setgray(prc, 0.75);
-		pr_draw_line(prc,
-			     (tims.start < tims.first) ? (tims.first - tims.start) * pxns : 0, liney,
-			     (tims.last <= tims.end) ? (tims.last - tims.start) * pxns : wavewidth - 1, liney);
+		if(!(t->flags & TR_ANALOGMASK))
+			{
+			pr_setgray(prc, 0.75);
+			pr_draw_line(prc,
+				     (tims.start < tims.first) ? (tims.first - tims.start) * pxns : 0, liney,
+				     (tims.last <= tims.end) ? (tims.last - tims.start) * pxns : wavewidth - 1, liney);
+			}
 	}
+
+	if((t->flags & TR_ANALOGMASK) && (!(h->flags&HIST_STRING) || !(h->flags&HIST_REAL))) {
+	        Trptr te = t->t_next;
+	        int ext = 0;
+		int num_traces_displayable = signalarea->allocation.height / fontheight;
+                          
+	        while(te)
+	                {
+	                if(te->flags & TR_ANALOG_BLANK_STRETCH)
+	                        {
+	                        ext++;
+	                        te = te->t_next;
+	                        }
+	                        else
+	                        {
+	                        break;
+	                        }
+	                }
+
+		if(which + ext > num_traces_displayable - 2)
+			{
+			ext = num_traces_displayable - which - 2;
+			if(ext<0) ext = 0; /* just in case of a one-off */
+			}
+
+		pr_draw_hptr_trace_vector_analog(prc, t, h, which, ext);
+		return;
+	}
+
 	pr_setgray(prc, 0.0);
 	for (;;) {
 		if (!h)
@@ -1206,10 +1457,10 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 			x1 = wavewidth;
 
 		/* draw trans */
-		type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.vector) : 0;
+		type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.h_vector) : AN_0;
 		if (x0 > -1)
 			if (use_roundcaps) {
-				if (type == 2) {
+				if (type == AN_Z) {
 					if (lasttype != -1) {
 						pr_draw_line(prc,
 							     x0 - 1, y0,
@@ -1218,7 +1469,7 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 							     x0, yu,
 							     x0 - 1, y1);
 					}
-				} else if (lasttype == 2) {
+				} else if (lasttype == AN_Z) {
 					pr_draw_line(prc,
 						     x0 + 1, y0,
 						     x0, yu);
@@ -1256,7 +1507,7 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 
 
 		if (x0 != x1) {
-			if (type == 2) {
+			if (type == AN_Z) {
 				if (use_roundcaps) {
 					pr_draw_line(prc,
 						     x0 + 1, yu,
@@ -1294,12 +1545,12 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 				if (width > vector_padding) {
 					if (h->flags & HIST_REAL) {
 						if (!(h->flags & HIST_STRING)) {
-							ascii = convert_ascii_real((double *) h->v.vector);
+							ascii = convert_ascii_real((double *) h->v.h_vector);
 						} else {
-							ascii = convert_ascii_string((char *) h->v.vector);
+							ascii = convert_ascii_string((char *) h->v.h_vector);
 						}
 					} else {
-						ascii = convert_ascii_vec(t, h->v.vector);
+						ascii = convert_ascii_vec(t, h->v.h_vector);
 					}
 
 
@@ -1343,6 +1594,138 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 /*
  * draw vector traces 
  */
+static void 
+pr_draw_vptr_trace_analog(pr_context * prc, Trptr t, vptr v, int which, int num_extension)
+{
+	TimeType        x0, x1, newtime, width;
+	int             y0, y1, yu, liney, ytext, yt0, yt1;
+	TimeType        tim, h2tim;
+	vptr            h, h2, h3;
+	int             pixlen, ysiz;
+	int             type;
+	int             lasttype = -1;
+	double          mynan = strtod("NaN", NULL);
+	double          tmin = mynan, tmax = mynan, tv, tv2;
+
+	h = v;
+	liney = ((which + 2 + num_extension) * fontheight) - 2;
+	y1 = ((which + 1) * fontheight) + 2;
+	y0 = liney - 2;
+	yu = (y0 + y1) / 2;
+	ytext = yu - (wavefont->ascent / 2) + wavefont->ascent;
+
+	ysiz = wavefont->ascent - 1;
+	if (ysiz < 1)
+		ysiz = 1;
+
+	h2 = h;
+	for(;;) {
+		if (!h2)
+			break;
+		tim = h2->time;
+		if ((tim > tims.end) || (tim > tims.last))
+			break;
+		x0 = (tim - tims.start) * pxns;
+		if (x0 > wavewidth)
+			break;
+		h3=h2;
+		h2 = h2->next;
+		if (!h2)
+			break;
+		tim = h2->time;
+		x1 = (tim - tims.start) * pxns;
+		if (x1 < 0)
+			continue;
+		tv = convert_real(t,h3);
+		if (!isnan(tv))	{
+			if (isnan(tmin) || tv < tmin)
+				tmin = tv;
+			if (isnan(tmax) || tv > tmax)
+				tmax = tv;
+		}
+	}
+	if (isnan(tmin) || isnan(tmax))
+		tmin = tmax = 0;
+	if ((tmax - tmin) < 1e-20) {
+		tmax = 1;
+		tmin -= 0.5 * (y1 - y0);
+	} else {
+		tmax = (y1 - y0) / (tmax - tmin);
+	}
+
+	pr_setgray(prc, 0.0);
+	for(;;)	{
+		if (!h)
+			break;
+		tim = (h->time);
+		if ((tim > tims.end) || (tim > tims.last))
+			break;
+
+		x0 = (tim - tims.start) * pxns;
+		if (x0 < -1)
+			x0 = -1;
+		else if (x0 > wavewidth)
+			break;
+
+		h2 = h->next;
+		if (!h2)
+			break;
+		h2tim = tim = (h2->time);
+		if (tim > tims.last)
+			tim = tims.last;
+		else if (tim > tims.end + 1)
+			tim = tims.end + 1;
+		x1 = (tim - tims.start) * pxns;
+
+		if (x1 < -1)
+			x1 = -1;
+		else if (x1 > wavewidth)
+			x1 = wavewidth;
+
+
+		/* draw trans */
+		type = vtype2(t, h);
+		tv = convert_real(t,h);
+		tv2 = convert_real(t,h2);
+		if(isnan(tv))
+			yt0 = yu;
+		else
+			yt0 = y0 + (tv - tmin) * tmax;
+		if(isnan(tv2))
+			yt1 = yu;
+		else
+			yt1 = y0 + (tv2 - tmin) * tmax;
+		if (x0 != x1) {
+			if(t->flags & TR_ANALOG_STEP) {
+				pr_draw_line(prc,
+					     x0, yt0,
+					     x1, yt0);
+				pr_draw_line(prc,
+					     x1, yt0,
+					     x1, yt1);
+			} else {
+				pr_draw_line(prc,
+					     x0, yt0,
+					     x1, yt1);
+			}
+		} else {
+			newtime = (((gdouble) (x1 + WAVE_OPT_SKIP)) * nspx) + tims.start + shift_timebase;	/* skip to next pixel */
+			h3 = bsearch_vector(t->n.vec, newtime);
+			if (h3->time > h->time) {
+				h = h3;
+				lasttype = type;
+				continue;
+			}
+		}
+
+		h = h->next;
+		lasttype = type;
+	}
+
+	tims.start += shift_timebase;
+	tims.end += shift_timebase;
+}
+
 static void 
 pr_draw_vptr_trace(pr_context * prc, Trptr t, vptr v, int which)
 {
@@ -1375,6 +1758,28 @@ pr_draw_vptr_trace(pr_context * prc, Trptr t, vptr v, int which)
 			     (tims.start < tims.first) ? (tims.first - tims.start) * pxns : 0, liney,
 			     (tims.last <= tims.end) ? (tims.last - tims.start) * pxns : wavewidth - 1, liney);
 	}
+
+	if(t->flags & TR_ANALOGMASK) {
+	        Trptr te = t->t_next;
+	        int ext = 0;
+                 
+	        while(te)
+	                {	
+	                if(te->flags & TR_ANALOG_BLANK_STRETCH)
+	                        {
+	                        ext++;
+	                        te = te->t_next;
+	                        }
+	                        else    
+	                        {
+	                        break;
+	                        }
+	                }
+
+		pr_draw_vptr_trace_analog(prc, t, v, which, ext);
+		return;
+	}
+
 	pr_setgray(prc, 0.0);
 	for (;;) {
 		if (!h)
@@ -1547,7 +1952,7 @@ pr_rendertraces(pr_context * prc)
 
 		t = topmost_trace;
 		for (i = 0; ((i < num_traces_displayable) && (t)); i++) {
-			if (!(t->flags & (TR_EXCLUDE | TR_BLANK))) {
+			if (!(t->flags & (TR_EXCLUDE | TR_BLANK | TR_ANALOG_BLANK_STRETCH))) {
 				shift_timebase = t->shift;
 				if (!t->vector) {
 					h = bsearch_node(t->n.nd, tims.start);
@@ -1558,7 +1963,7 @@ pr_rendertraces(pr_context * prc)
 					       "\n", tims.start, (h->time + shift_timebase)));
 
 					if (!t->n.nd->ext) {
-						pr_draw_hptr_trace(prc, t, h, i, 1);
+						pr_draw_hptr_trace(prc, t, h, i, 1, 0);
 					} else {
 						pr_draw_hptr_trace_vector(prc, t, h, i);
 					}
@@ -1573,9 +1978,24 @@ pr_rendertraces(pr_context * prc)
 					pr_draw_vptr_trace(prc, t, v, i);
 				}
 			} else {
-				pr_draw_hptr_trace(prc, NULL, NULL, i, 0);
+	                        int kill_dodraw_grid = t->flags & TR_ANALOG_BLANK_STRETCH;
+                                 
+	                        if(kill_dodraw_grid)
+	                                {
+	                                if(!(t->t_next))
+	                                        {
+	                                        kill_dodraw_grid = 0;
+	                                        }
+	                                else
+	                                if(!(t->t_next->flags & TR_ANALOG_BLANK_STRETCH))
+	                                        {   
+	                                        kill_dodraw_grid = 0;
+	                                        }
+	                                }
+
+				pr_draw_hptr_trace(prc, NULL, NULL, i, 0, kill_dodraw_grid);
 			}
-			t = t->next;
+			t = GiveNextTrace(t);
 		}
 	}
 	pr_draw_named_markers(prc);
@@ -1620,7 +2040,7 @@ pr_RenderSig(pr_context * prc, Trptr t, int i)
 
 	pr_draw_line(prc, 0, liney, pr_signal_fill_width - 1, liney);
 
-	/* if (!(t->flags&TR_BLANK)) */
+	/* if (!(t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH))) */
 	{
 		int             maxwidth = 0;
 
@@ -1684,9 +2104,9 @@ pr_RenderSigs(pr_context * prc, int trtarget)
 	t = traces.first;
 	trwhich = 0;
 	while (t) {
-		if ((trwhich < trtarget) && (t->next)) {
+		if ((trwhich < trtarget) && (GiveNextTrace(t))) {
 			trwhich++;
-			t = t->next;
+			t = GiveNextTrace(t);
 		} else {
 			break;
 		}
@@ -1696,7 +2116,7 @@ pr_RenderSigs(pr_context * prc, int trtarget)
 	if (t) {
 		for (i = 0; (i < num_traces_displayable) && (t); i++) {
 			pr_RenderSig(prc, t, i);
-			t = t->next;
+			t = GiveNextTrace(t);
 		}
 	}
 }
