@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,15 +27,20 @@
 #include "rc.h"
 
 #ifndef _MSC_VER
+#ifndef __MINGW32__
 	#include <unistd.h>
 	#include <pwd.h>
 	static char *rcname=".gtkwaverc";	/* name of environment file--POSIX */
+#else
+	static char *rcname="gtkwave.ini";      /* name of environment file--WIN32 */
+#endif
 #else
 	static char *rcname="gtkwave.ini";      /* name of environment file--WIN32 */
 	#define strcasecmp _stricmp
 #endif
 
 int rc_line_no;
+int possibly_use_rc_defaults = 1;		/* if not -d option, use white screen, etc on no .gtkwaverc */
 
 /*
  * functions that set the individual rc variables..
@@ -127,6 +132,13 @@ do_resize_signals=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_enable_fast_exit(char *str)
+{
+DEBUG(printf("f_enable_fast_exit(\"%s\")\n",str));
+enable_fast_exit=atoi_64(str)?1:0;
+return(0);
+}
+
 static int f_enable_ghost_marker(char *str)
 {
 DEBUG(printf("f_enable_ghost_marker(\"%s\")\n",str));
@@ -155,6 +167,15 @@ enable_vert_grid=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_fontname_logfile(char *str)
+{
+DEBUG(printf("f_fontname_logfile(\"%s\")\n",str));
+if(fontname_logfile) free_2(fontname_logfile);
+fontname_logfile=(char *)malloc_2(strlen(str)+1);
+strcpy(fontname_logfile,str);
+return(0);
+}
+
 static int f_fontname_signals(char *str)
 {
 DEBUG(printf("f_fontname_signals(\"%s\")\n",str));
@@ -310,6 +331,13 @@ display_grid=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_splash_disable(char *str)
+{
+DEBUG(printf("f_splash_disable(\"%s\")\n",str));
+splash_disable=atoi_64(str)?1:0;
+return(0);
+}
+
 static int f_use_big_fonts(char *str)
 {
 DEBUG(printf("f_use_big_fonts(\"%s\")\n",str));
@@ -359,6 +387,20 @@ vcd_explicit_zero_subscripts=atoi_64(str)?0:-1;	/* 0==yes, -1==no */
 return(0);
 }
 
+static int f_vcd_preserve_glitches(char *str)
+{
+DEBUG(printf("f_vcd_preserve_glitches(\"%s\")\n",str));
+vcd_preserve_glitches=atoi_64(str)?0:-1;	/* 0==yes, -1==no */
+return(0);
+}
+
+static int f_vcd_warning_filesize(char *str)
+{
+DEBUG(printf("f_vcd_preserve_glitches(\"%s\")\n",str));
+vcd_warning_filesize=atoi_64(str);
+return(0);
+}
+
 static int f_vector_padding(char *str)
 {
 DEBUG(printf("f_vector_padding(\"%s\")\n",str));
@@ -418,9 +460,12 @@ return(0); \
 }
 
 color_make(back)
+color_make(baseline)
 color_make(grid)
 color_make(high)
 color_make(low)
+color_make(1)
+color_make(0)
 color_make(mark)
 color_make(mid)
 color_make(time)
@@ -432,6 +477,12 @@ color_make(vbox)
 color_make(vtrans)
 color_make(x)
 color_make(xfill)
+color_make(u)
+color_make(ufill)
+color_make(w)
+color_make(wfill)
+color_make(dash)
+color_make(dashfill)
 
 
 /*
@@ -445,7 +496,12 @@ static struct rc_entry rcitems[]=
 { "autocoalesce", f_autocoalesce },
 { "autocoalesce_reversal", f_autocoalesce_reversal },
 { "autoname_bundles", f_autoname_bundles },
+{ "color_0", f_color_0 },
+{ "color_1", f_color_1 },
 { "color_back", f_color_back },
+{ "color_baseline", f_color_baseline },
+{ "color_dash", f_color_dash },
+{ "color_dashfill", f_color_dashfill },
 { "color_grid", f_color_grid },
 { "color_high", f_color_high },
 { "color_low", f_color_low },
@@ -460,16 +516,22 @@ static struct rc_entry rcitems[]=
 { "color_vtrans", f_color_vtrans },
 { "color_x", f_color_x },
 { "color_xfill", f_color_xfill },
+{ "color_u", f_color_u },
+{ "color_ufill", f_color_ufill },
+{ "color_w", f_color_w },
+{ "color_wfill", f_color_wfill },
 { "constant_marker_update", f_constant_marker_update },
 { "convert_to_reals", f_convert_to_reals },
 { "cursor_snap", f_cursor_snap },
 { "disable_tooltips", f_disable_tooltips },
 { "do_initial_zoom_fit", f_do_initial_zoom_fit },
 { "dynamic_resizing", f_dynamic_resizing },
+{ "enable_fast_exit", f_enable_fast_exit },
 { "enable_ghost_marker", f_enable_ghost_marker },
 { "enable_horiz_grid", f_enable_horiz_grid}, 
 { "enable_vcd_autosave", f_enable_vcd_autosave },
 { "enable_vert_grid", f_enable_vert_grid}, 
+{ "fontname_logfile", f_fontname_logfile}, 
 { "fontname_signals", f_fontname_signals}, 
 { "fontname_waves", f_fontname_waves}, 
 { "force_toolbars", f_force_toolbars}, 
@@ -487,6 +549,7 @@ static struct rc_entry rcitems[]=
 { "ps_maxveclen", f_ps_maxveclen },
 { "show_base_symbols", f_show_base_symbols },
 { "show_grid", f_show_grid },
+{ "splash_disable", f_splash_disable },
 { "use_big_fonts", f_use_big_fonts },
 { "use_full_precision", f_use_full_precision },
 { "use_maxtime_display", f_use_maxtime_display },
@@ -494,6 +557,8 @@ static struct rc_entry rcitems[]=
 { "use_roundcaps", f_use_roundcaps },
 { "use_scrollbar_only", f_use_scrollbar_only },
 { "vcd_explicit_zero_subscripts", f_vcd_explicit_zero_subscripts },
+{ "vcd_preserve_glitches", f_vcd_preserve_glitches },
+{ "vcd_warning_filesize", f_vcd_warning_filesize },
 { "vector_padding", f_vector_padding },
 { "wave_scrolling", f_wave_scrolling },
 { "zoom_base", f_zoom_base },
@@ -502,17 +567,57 @@ static struct rc_entry rcitems[]=
 };
 
 
+static void vanilla_rc(void)
+{
+f_splash_disable 	("off");
+f_zoom_pow10_snap	("on");
+f_hier_max_level	("1");
+f_cursor_snap		("8");
+f_use_maxtime_display	("off");
+f_use_roundcaps		("on");
+f_use_nonprop_fonts	("on");
+f_constant_marker_update("on");
+f_show_base_symbols	("off");
+f_color_back   		("000000");
+f_color_baseline 	("ffffff");
+f_color_grid		("202070");
+f_color_high		("79f6f2");
+f_color_low		("5dbebb");
+f_color_1		("00ff00");
+f_color_0		("008000");
+f_color_trans		("00c000");
+f_color_mid		("c0c000");  
+f_color_value		("ffffff");
+f_color_vbox		("00ff00");
+f_color_vtrans		("00c000");
+f_color_x		("00ff00");
+f_color_xfill		("004000");
+f_color_u		("ff0000");
+f_color_ufill		("400000");
+f_color_w		("79f6f2");
+f_color_wfill		("3f817f");
+f_color_dash		("edf508");
+f_color_dashfill	("7d8104");
+f_color_umark		("ff8080");
+f_color_mark		("ffff80");
+f_color_time		("ffffff");
+f_color_timeb		("000000");
+}
+
 
 void read_rc_file(char *override_rc)
 {
 FILE *handle;
 
+/* move defaults first and only go whitescreen if instructed to do so */
+if(possibly_use_rc_defaults) vanilla_rc();
+
 if((override_rc)&&((handle=fopen(override_rc,"rb"))))
 	{
 	/* good, we have a handle */
 	}
 else
-#ifndef _MSC_VER
+#if !defined __MINGW32__ && !defined _MSC_VER
 if(!(handle=fopen(rcname,"rb")))
 	{
 	char *home;
@@ -534,6 +639,7 @@ if(!(handle=fopen(rcname,"rb")))
 if(!(handle=fopen(rcname,"rb")))		/* no concept of ~ in win32 */
 	{
 	errno=0;
+	if(possibly_use_rc_defaults) vanilla_rc();
 	return; /* no .rc file */
 	} 
 #endif
