@@ -16,7 +16,7 @@ char *name;
 int (*func)(char *);
 };
 
-void read_rc_file(void);
+void read_rc_file(char *override_rc);
 int get_rgb_from_name(char *str);
 
 extern int rc_line_no;
