@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -8,10 +8,12 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "analyzer.h"
 #include "aet.h"
 #include "vcd.h"
 #include "lx2.h"
+#include "ghw.h"
 #include "debug.h"
 
 static GtkWidget *window1;
@@ -35,7 +37,7 @@ return(is_active);
 
 static void enter_callback_e(GtkWidget *widget, GtkWidget *nothing)
 {
-  gchar *entry_text;
+  G_CONST_RETURN gchar *entry_text;
   int len;
   char *vname="<Vector>";
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry_a));
@@ -187,7 +189,7 @@ bundle_callback_generic(void)
 DEBUG(printf("Selected_rows: %d\n",selected_rows));
 if(selected_rows>0)
 	{
-	entrybox_local("Enter Bundle Name",300,"",128,bundle_cleanup);
+	entrybox_local("Enter Bundle Name",300,"",128,GTK_SIGNAL_FUNC(bundle_cleanup));
 	}
 }
 
@@ -477,7 +479,7 @@ while(tfirst)
 	{
 	tfirst->flags |= TR_HIGHLIGHT;
 	if(tfirst==tlast) break;
-	tfirst=tfirst->next;
+	tfirst=tfirst->t_next;
 	}
 
 MaxSignalLength();
@@ -635,21 +637,17 @@ selected_rows--;
 static void enter_callback(GtkWidget *widget, GtkWidget *do_warning)
 {
 GtkCList *cl;
-gchar *entry_text;
+G_CONST_RETURN gchar *entry_text;
 char *entry_suffixed;
 int len;
-int i, j, k, row;
-int pixlen, maxpixlen;
+int i, row;
 char *s, *tmp2;
-
 gfloat interval;
 
 if(is_searching_running) return;
 is_searching_running = ~0;
 gtk_grab_add(widget);
 
-pixlen=0; maxpixlen=0;
-
 entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
 DEBUG(printf("Entry contents: %s\n", entry_text));
 /* if(!(len=strlen(entry_text))) searchbox_text=NULL; */
@@ -686,11 +684,11 @@ for(i=0;i<numfacs;i++)
 	pdata->oldvalue = i;
 
 	if(wave_regex_match(facs[i]->name))
+	if((!is_ghw)||(strcmp(WAVE_GHW_DUMMYFACNAME, facs[i]->name)))
 		{
 		if(!facs[i]->vec_root)
 			{
 			row=gtk_clist_append(cl,(gchar **)&(facs[i]->name));
-			pixlen=gdk_string_measure(signalfont,(gchar *)(facs[i]->name));
 			}
 			else
 			{
@@ -712,18 +710,16 @@ for(i=0;i<numfacs;i++)
 				}
 
 			row=gtk_clist_append(cl,(gchar **)&s);
-			pixlen=gdk_string_measure(signalfont,(gchar *)s);
 			free_2(s);
 			}
 
-		maxpixlen=(pixlen>maxpixlen)?pixlen:maxpixlen;		
 		gtk_clist_set_row_data(cl, row,facs[i]); 
 		num_rows++;
 		if(num_rows==WAVE_MAX_CLIST_LENGTH) break;
 		}
 	}
 
-gtk_clist_set_column_width(GTK_CLIST(clist),0,maxpixlen?maxpixlen:1);
+gtk_clist_set_column_width(GTK_CLIST(clist),0,gtk_clist_optimal_column_width(GTK_CLIST(clist),0));
 gtk_clist_thaw(cl);
 
 gtk_progress_set_value (GTK_PROGRESS (pdata->pbar), 0.0);
@@ -736,7 +732,7 @@ if(num_rows>=WAVE_MAX_CLIST_LENGTH)
 	{
 	char buf[256];
 	sprintf(buf, "Limiting results to first %d entries.", num_rows);
-	simplereqbox("Regex Search Warning",300,buf,"OK", NULL, NULL);
+	simplereqbox("Regex Search Warning",300,buf,"OK", NULL, NULL, 1);
 	}
 }
 
