@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -82,7 +82,10 @@ Trptr t;
 gdouble x,y;
 GdkModifierType state;
 int doloop;
+
+#ifdef WAVE_USE_GTK2
 gint xi, yi;
+#endif
 
 if(event->is_hint)
 	{
@@ -100,7 +103,7 @@ top: doloop=0;
 
 /********************* button 1 *********************/
 
-if((traces.total)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&(cachedwhich!=-1))
+if((traces.visible)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&(cachedwhich!=-1))
 	{
 	num_traces_displayable=widget->allocation.height/(fontheight);
 	num_traces_displayable--;   /* for the time trace that is always there */
@@ -108,15 +111,15 @@ if((traces.total)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&
 	which=(int)(y);
 	which=(which/fontheight)-1;
 
-	if(which>=traces.total)
+	if(which>=traces.visible)
 		{
-		which=traces.total-1;
+		which=traces.visible-1;
 		}
 
 
 	wadj=GTK_ADJUSTMENT(wave_vslider);
 
-	if(num_traces_displayable<traces.total)
+	if(num_traces_displayable<traces.visible)
 		{
 		if(which>=num_traces_displayable)
 			{
@@ -125,7 +128,7 @@ if((traces.total)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&
 			target=((int)wadj->value)+1;
 			which=num_traces_displayable-1;
 
-			if(target+which>=(traces.total-1)) target=traces.total-which-1;
+			if(target+which>=(traces.visible-1)) target=traces.visible-which-1;
 			wadj->value=target;
 
 			if(cachedwhich==which) cachedwhich=which-1; /* force update */
@@ -164,10 +167,10 @@ if((traces.total)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&
 	trwhich=0;
 	while(t)
 	        {
-	        if((trwhich<trtarget)&&(t->next))
+	        if((trwhich<trtarget)&&(GiveNextTrace(t)))
 	                {
 	                trwhich++;
-	                t=t->next;
+	                t=GiveNextTrace(t);
 	                }
 	                else
 	                {
@@ -234,12 +237,12 @@ if((traces.total)&&(state&(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK))&&(signalpixmap)&&
 		if(which>cachedwhich)
 			{
 			which--;
-			t=t->prev;
+			t=GivePrevTrace(t);
 			}
 		else if(which<cachedwhich)
 			{
 			which++;
-			t=t->next;
+			t=GiveNextTrace(t);
 			}
 
 		} while((which!=cachedwhich)&&(t));
@@ -313,7 +316,7 @@ int trwhich, trtarget;
 GtkAdjustment *wadj;
 Trptr t;
 
-if((traces.total)&&(signalpixmap))
+if((traces.visible)&&(signalpixmap))
 	{
 	gdk_pointer_grab(widget->window, FALSE,
 		GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON3_MOTION_MASK | 
@@ -325,7 +328,7 @@ if((traces.total)&&(signalpixmap))
 	which=(int)(event->y);
 	which=(which/fontheight)-1;
 
-	if((which>=traces.total)||(which>=num_traces_displayable)||(which<0))
+	if((which>=traces.visible)||(which>=num_traces_displayable)||(which<0))
 		{
 		if(dnd_state==0)cachedwhich=-1;
 		goto check_button_3;	/* off in no man's land, but check 3rd anyways.. */
@@ -339,10 +342,10 @@ if((traces.total)&&(signalpixmap))
 	trwhich=0;
 	while(t)
 	        {
-	        if((trwhich<trtarget)&&(t->next))
+	        if((trwhich<trtarget)&&(GiveNextTrace(t)))
 	                {
 	                trwhich++;
-	                t=t->next;
+	                t=GiveNextTrace(t);
 	                }
 	                else
 	                {
@@ -351,6 +354,21 @@ if((traces.total)&&(signalpixmap))
 	        }
 
 	cachedtrace=t;
+
+	/* frankel add */
+	if((event->state&GDK_CONTROL_MASK)&&(dnd_state==0)&&(event->button==1))
+	if(t)
+		{
+		if(CollapseTrace(t))
+			{
+			signalwindow_width_dirty=1;
+        		MaxSignalLength();
+        		signalarea_configure_event(signalarea, NULL);
+        		wavearea_configure_event(wavearea, NULL);
+			return(TRUE);
+			}
+		}
+
 	if((dnd_state==0)&&(event->button==1))
 	if(t)
 		{
@@ -374,11 +392,11 @@ if((traces.total)&&(signalpixmap))
 						{
 						t2->flags = (t2->flags & (~TR_HIGHLIGHT)) | f;
 						if(t2==t) break;
-						t2=t2->prev;
+						t2=GivePrevTrace(t2);
 						}					
 					goto resync_signalarea;
 					}
-				t2=t2->prev;
+				t2=GivePrevTrace(t2);
 				}
 
 			t2=shift_click_trace;
@@ -392,11 +410,11 @@ if((traces.total)&&(signalpixmap))
 						{
 						t2->flags = (t2->flags & (~TR_HIGHLIGHT)) | f;
 						if(t2==t) break;
-						t2=t2->next;
+						t2=GiveNextTrace(t2);
 						}					
 					goto resync_signalarea;
 					}
-				t2=t2->next;
+				t2=GiveNextTrace(t2);
 				}
 
 			goto normal_button1_press;	/* couldn't find original so make this original... */
@@ -459,6 +477,8 @@ GtkAdjustment *wadj, *hadj;
 int num_traces_displayable;
 int width;
 
+UpdateTracesVisible();
+
 num_traces_displayable=widget->allocation.height/(fontheight);
 num_traces_displayable--;   /* for the time trace that is always there */
 
@@ -505,16 +525,16 @@ wadj=GTK_ADJUSTMENT(wave_vslider);
 wadj->page_size=wadj->page_increment=(gfloat) num_traces_displayable;
 wadj->step_increment=(gfloat)1.0;
 wadj->lower=(gfloat)0.0;
-wadj->upper=(gfloat)traces.total;
+wadj->upper=(gfloat)(traces.visible ? traces.visible : 1);
 
-if(num_traces_displayable>traces.total)
+if(num_traces_displayable>traces.visible)
 	{
 	wadj->value=(gfloat)(trtarget=0);
 	}
 	else
-	if (wadj->value + num_traces_displayable > traces.total)
+	if (wadj->value + num_traces_displayable > traces.visible)
 	{
-	wadj->value=(gfloat)(trtarget=traces.total-num_traces_displayable);
+	wadj->value=(gfloat)(trtarget=traces.visible-num_traces_displayable);
 	}
 
 gtk_signal_emit_by_name (GTK_OBJECT (wadj), "changed");	/* force bar update */
