@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -10,6 +10,7 @@
 #include "gtk12compat.h"
 #include <gtk/gtk.h>
 #include "debug.h"
+#include "pixmaps.h"
 
 static GtkWidget *window;
 static void (*cleanup)(GtkWidget *, gpointer);
@@ -31,18 +32,19 @@ static void destroy_callback(GtkWidget *widget, GtkWidget *nothing)
 }
 
 void simplereqbox(char *title, int width, char *default_text,
-	char *oktext, char *canceltext, GtkSignalFunc func)
+	char *oktext, char *canceltext, GtkSignalFunc func, int is_alert)
 {
     GtkWidget *vbox, *hbox;
     GtkWidget *button1, *button2;
     GtkWidget *label, *separator;
+    GtkWidget *pixmapwid1;
 
     cleanup=WAVE_GTK_SFUNCAST(func);
 
     /* create a new modal window */
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_grab_add(window);
-    gtk_widget_set_usize( GTK_WIDGET (window), width, 55);
+    gtk_widget_set_usize( GTK_WIDGET (window), width, 55 + 52);
     gtk_window_set_title(GTK_WINDOW (window), title);
     gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                        (GtkSignalFunc) destroy_callback, NULL);
@@ -56,6 +58,17 @@ void simplereqbox(char *title, int width, char *default_text,
     gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
     gtk_widget_show (label);
 
+    if(is_alert)
+	{
+	pixmapwid1=gtk_pixmap_new(wave_alert_pixmap, wave_alert_mask);
+	}
+	else
+	{
+	pixmapwid1=gtk_pixmap_new(wave_info_pixmap, wave_info_mask);
+	}
+    gtk_widget_show(pixmapwid1);
+    gtk_container_add (GTK_CONTAINER (vbox), pixmapwid1);
+
     separator = gtk_hseparator_new ();
     gtk_box_pack_start (GTK_BOX (vbox), separator, FALSE, TRUE, 0);
     gtk_widget_show (separator);
