@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2003.
+ * Copyright (c) Tony Bybell 1999-2004.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,12 +7,14 @@
  * of the License, or (at your option) any later version.
  */
 
+#include "gtk12compat.h"
 #include "strace.h"
+#include "currenttime.h"
 
 TimeType *timearray=NULL;
 int timearray_size=0;
 
-static GtkWidget *ptr_mark_count_label=NULL;   /* pointer to mark count label on pattern search dialog */
+static GtkWidget *ptr_mark_count_label=NULL;	/* pointer to mark count label on pattern search dialog	*/
 
 struct strace *straces=NULL;   /* when active, disables certain ops! */
 struct strace *shadow_straces=NULL;
@@ -35,6 +37,78 @@ char shadow_active = 0;
 char shadow_type = 0;
 char *shadow_string = NULL;
 
+signed char mark_idx_start = 0;		/* which start mark conditons were selected	*/
+signed char mark_idx_end   = 0;		/* which end   mark conditons were selected	*/
+signed char shadow_mark_idx_start = 0;
+signed char shadow_mark_idx_end   = 0;
+
+struct item_mark_string {
+   char *str;
+   unsigned char idx;
+};
+
+static struct item_mark_string item_mark_start_strings[]=
+	{
+	   { "Start of Time"  },
+	   { "Named Marker A" },
+	   { "Named Marker B" },
+	   { "Named Marker C" },
+	   { "Named Marker D" },
+	   { "Named Marker E" },
+	   { "Named Marker F" },
+	   { "Named Marker G" },
+	   { "Named Marker H" },
+	   { "Named Marker I" },
+	   { "Named Marker J" },
+	   { "Named Marker K" },
+	   { "Named Marker L" },
+	   { "Named Marker M" },
+	   { "Named Marker N" },
+	   { "Named Marker O" },
+	   { "Named Marker P" },
+	   { "Named Marker Q" },
+	   { "Named Marker R" },
+	   { "Named Marker S" },
+	   { "Named Marker T" },
+	   { "Named Marker U" },
+	   { "Named Marker V" },
+	   { "Named Marker W" },
+	   { "Named Marker X" },
+	   { "Named Marker Y" },
+	   { "Named Marker Z" }
+	};
+
+static struct item_mark_string item_mark_end_strings[]=
+	{
+	   { "End of Time"    },
+	   { "Named Marker A" },
+	   { "Named Marker B" },
+	   { "Named Marker C" },
+	   { "Named Marker D" },
+	   { "Named Marker E" },
+	   { "Named Marker F" },
+	   { "Named Marker G" },
+	   { "Named Marker H" },
+	   { "Named Marker I" },
+	   { "Named Marker J" },
+	   { "Named Marker K" },
+	   { "Named Marker L" },
+	   { "Named Marker M" },
+	   { "Named Marker N" },
+	   { "Named Marker O" },
+	   { "Named Marker P" },
+	   { "Named Marker Q" },
+	   { "Named Marker R" },
+	   { "Named Marker S" },
+	   { "Named Marker T" },
+	   { "Named Marker U" },
+	   { "Named Marker V" },
+	   { "Named Marker W" },
+	   { "Named Marker X" },
+	   { "Named Marker Y" },
+	   { "Named Marker Z" }
+	};
+
 
 /*
  * trap timescale overflows
@@ -89,7 +163,6 @@ while(sd)
 strace_defer_free_head = NULL;
 }
 
-
 /*
  * button/menu/entry activations..
  */
@@ -118,10 +191,31 @@ s->value=b->which;
 DEBUG(printf("Trace %s Search Type: %s\n", s->trace->name, stype[(int)s->value]));
 }
 
+static void start_clicked(GtkWidget *widget, gpointer which)
+{
+mark_idx_start=((struct item_mark_string*)which)->idx;
+if (mark_idx_start<0 || mark_idx_start>=(sizeof(item_mark_start_strings)/sizeof(struct item_mark_string)))
+	{
+	fprintf(stderr, "Internal error: mark_idx_start out of range %d\n",mark_idx_start);
+	exit(255);
+	}
+DEBUG(printf("start: %s\n", ((struct item_mark_string*)which)->str));
+}
+
+static void end_clicked(GtkWidget *widget, gpointer which)
+{
+mark_idx_end=((struct item_mark_string*)which)->idx;
+if (mark_idx_end<0 || mark_idx_end>=(sizeof(item_mark_end_strings)/sizeof(struct item_mark_string)))
+	{
+	fprintf(stderr, "Internal error: mark_idx_end out of range %d\n",mark_idx_end);
+	exit(255);
+	}
+DEBUG(printf("end: %s\n", ((struct item_mark_string*)which)->str));
+}
 
 static void enter_callback(GtkWidget *widget, gpointer strace_tmp)
 {
-gchar *entry_text;
+G_CONST_RETURN gchar *entry_text;
 struct strace *s;
 int i, len;
 
@@ -203,10 +297,7 @@ static void update_mark_count_label()
 if(ptr_mark_count_label)
     {
     char mark_count_buf[64];
-    if (timearray_size < 1)
-	mark_count_buf[0] = 0;
-    else
-	sprintf (mark_count_buf, "Mark Count: %d", timearray_size>=1 ? timearray_size-1 : 0);
+    sprintf (mark_count_buf, "Mark Count: %d", timearray_size);
     gtk_label_set_text (GTK_LABEL(ptr_mark_count_label), mark_count_buf);
     }
 }
@@ -216,7 +307,7 @@ void tracesearchbox(char *title, GtkSignalFunc func)
     GtkWidget *menu, *menuitem, *optionmenu;
     GSList *group; 
     GtkWidget *entry;
-    GtkWidget *vbox, *hbox, *small_hbox, *count_hbox, *vbox_g, *label;
+    GtkWidget *vbox, *hbox, *small_hbox, *vbox_g, *label;
     GtkWidget *button1, *button1a, *button1b, *button1c, *button2, *scrolled_win, *frame, *separator;
     Trptr t;
     int i;
@@ -290,11 +381,11 @@ void tracesearchbox(char *title, GtkSignalFunc func)
     gtk_container_add (GTK_CONTAINER (frame), scrolled_win);
     gtk_container_add (GTK_CONTAINER (vbox), frame);
 
-    for(t=traces.first;t;t=t->next)
+    for(t=traces.first;t;t=t->t_next)
     {
     struct strace *s;
 
-    if ((t->flags&TR_BLANK)||(!(t->flags&TR_HIGHLIGHT))||(!(t->name))) continue;
+    if ((t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH))||(!(t->flags&TR_HIGHLIGHT))||(!(t->name))) continue;
 
     numtraces++;
     if(numtraces==500) 
@@ -359,15 +450,85 @@ void tracesearchbox(char *title, GtkSignalFunc func)
 
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_win), vbox_g);
 
-    do		/* add a horizontal box to display the mark count */
+    do		/* add GUI elements for displaying mark count and mark count start/end */
 	{
-	count_hbox = gtk_hbox_new (TRUE, 0);
-	gtk_widget_show (count_hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), count_hbox, FALSE, FALSE, 0);
+	int idx;
+	GtkWidget *ptr_mark_start_label, *ptr_mark_end_label;
+	GtkWidget *mark_count_hbox_start,  *mark_count_hbox_end; 
+	GtkWidget *count_vbox_left, *count_vbox_right, *count_vbox, *count_hbox;
+	GtkWidget *ptr_mark_count_start, *ptr_mark_count_end;
 
-	ptr_mark_count_label = gtk_label_new ("");
+	count_hbox=gtk_hbox_new (TRUE, 0);
+	gtk_widget_show (count_hbox);
+	gtk_box_pack_start (GTK_BOX(vbox),count_hbox,FALSE,FALSE,0);
+
+		/* add a vertical box to display the mark count GUI elements */
+	count_vbox_left=gtk_vbox_new (TRUE, 0);
+	gtk_widget_show (count_vbox_left);
+	gtk_box_pack_start (GTK_BOX(count_hbox),count_vbox_left,FALSE,FALSE,0);
+	count_vbox=gtk_vbox_new (TRUE, 0);
+	gtk_widget_show (count_vbox);
+	gtk_box_pack_start (GTK_BOX(count_hbox),count_vbox,FALSE,FALSE,0);
+	count_vbox_right=gtk_vbox_new (TRUE, 0);
+	gtk_widget_show (count_vbox_right);
+	gtk_box_pack_start (GTK_BOX(count_hbox),count_vbox_right,FALSE,FALSE,0);
+
+		/* add mark start GUI element */
+	mark_count_hbox_start=gtk_hbox_new (TRUE, 0);
+	gtk_widget_show (mark_count_hbox_start);
+	gtk_box_pack_start (GTK_BOX(count_vbox),mark_count_hbox_start,FALSE,FALSE,0);
+	ptr_mark_start_label=gtk_label_new ("Marking Begins at:");
+	gtk_widget_show (ptr_mark_start_label);
+	gtk_box_pack_start (GTK_BOX (mark_count_hbox_start),ptr_mark_start_label,TRUE,FALSE,0);
+	ptr_mark_count_start = gtk_menu_new ();
+	group=NULL;
+	for(idx=0; idx<sizeof(item_mark_start_strings)/sizeof(struct item_mark_string); ++idx)
+		{
+		item_mark_start_strings[idx].idx=idx;
+    		menuitem = gtk_radio_menu_item_new_with_label (group, item_mark_start_strings[idx].str);
+    		group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
+    		gtk_menu_append (GTK_MENU (ptr_mark_count_start), menuitem);
+    		gtk_widget_show (menuitem);
+        	gtk_signal_connect(GTK_OBJECT (menuitem), "activate",
+                                 GTK_SIGNAL_FUNC(start_clicked),
+                                 &item_mark_start_strings[idx]);
+		}
+	optionmenu = gtk_option_menu_new ();
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), ptr_mark_count_start);
+	gtk_box_pack_start (GTK_BOX (mark_count_hbox_start), optionmenu, TRUE, FALSE, 0);
+	gtk_widget_show (optionmenu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), mark_idx_start);
+
+		/* add mark end GUI element */
+	mark_count_hbox_end=gtk_hbox_new (TRUE, 0);
+	gtk_widget_show (mark_count_hbox_end);
+	gtk_box_pack_start (GTK_BOX(count_vbox),mark_count_hbox_end,FALSE,FALSE,0);
+	ptr_mark_end_label=gtk_label_new ("Marking Stops at:");
+	gtk_widget_show (ptr_mark_end_label);
+	gtk_box_pack_start (GTK_BOX (mark_count_hbox_end),ptr_mark_end_label,TRUE,FALSE,0);
+	ptr_mark_count_end = gtk_menu_new ();
+	group=NULL;
+	for(idx=0; idx<sizeof(item_mark_end_strings)/sizeof(struct item_mark_string); ++idx)
+		{
+		item_mark_end_strings[idx].idx=idx;
+    		menuitem = gtk_radio_menu_item_new_with_label (group, item_mark_end_strings[idx].str);
+    		group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
+    		gtk_menu_append (GTK_MENU (ptr_mark_count_end), menuitem);
+    		gtk_widget_show (menuitem);
+        	gtk_signal_connect(GTK_OBJECT (menuitem), "activate",
+                                 GTK_SIGNAL_FUNC(end_clicked),
+                                 &item_mark_end_strings[idx]);
+		}
+	optionmenu = gtk_option_menu_new ();
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), ptr_mark_count_end);
+	gtk_box_pack_start (GTK_BOX (mark_count_hbox_end), optionmenu, TRUE, FALSE, 0);
+	gtk_widget_show (optionmenu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), mark_idx_end);
+
+		/* add mark count GUI element */
+	ptr_mark_count_label=gtk_label_new ("");
 	gtk_widget_show (ptr_mark_count_label);
-	gtk_box_pack_start (GTK_BOX (count_hbox), ptr_mark_count_label, TRUE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (count_vbox),ptr_mark_count_label,TRUE,FALSE,0);
 	update_mark_count_label ();
 	} while (0);
 
@@ -574,11 +735,11 @@ while(s)
 			}
 		if(t->flags&TR_INVERT)  
                 	{
-                        str[0]="1XZ0"[s->his.h->v.val];
+                        str[0]=AN_STR_INV[s->his.h->v.h_val];
                         }
                         else   
                         {
-                        str[0]="0XZ1"[s->his.h->v.val];
+                        str[0]=AN_STR[s->his.h->v.h_val];
                         }
 		str[1]=0x00;
 
@@ -589,23 +750,23 @@ while(s)
 				
 			case ST_HIGH:
 				totaltraces++;
-				if(str[0]=='1') s->search_result=1;	
+				if((str[0]=='1')||(str[0]=='H')) s->search_result=1;	
 				break;
 
 			case ST_RISE:
 				totaltraces++;
-				if((str[0]=='1')&&(adjust(s->his.h->time,shift_timebase)==maxbase)) 
+				if(((str[0]=='1')||(str[0]=='H'))&&(adjust(s->his.h->time,shift_timebase)==maxbase)) 
 					s->search_result=1;	
 				break;
 
 			case ST_LOW:
 				totaltraces++;
-				if(str[0]=='0') s->search_result=1;
+				if((str[0]=='0')||(str[0]=='L')) s->search_result=1;
 				break;
 
 			case ST_FALL:
 				totaltraces++;
-				if((str[0]=='0')&&(adjust(s->his.h->time,shift_timebase)==maxbase))
+				if(((str[0]=='0')||(str[0]=='L'))&&(adjust(s->his.h->time,shift_timebase)==maxbase))
  					s->search_result=1;
 				break;
 
@@ -632,8 +793,8 @@ while(s)
 				break;
 
 			default:
-				printf("Internal error: st_type of %d\n",s->value);
-				exit(0);
+				fprintf(stderr, "Internal error: st_type of %d\n",s->value);
+				exit(255);
 			}
 
 
@@ -663,11 +824,11 @@ while(s)
 				{
 				if(!(s->his.h->flags&HIST_STRING))
 					{
-					chval=convert_ascii_real((double *)s->his.h->v.vector);
+					chval=convert_ascii_real((double *)s->his.h->v.h_vector);
 					}
 					else
 					{
-					chval=convert_ascii_string((char *)s->his.h->v.vector);
+					chval=convert_ascii_string((char *)s->his.h->v.h_vector);
 					chval2=chval;
 					while((ch=*chval2))	/* toupper() the string */
 						{
@@ -678,7 +839,7 @@ while(s)
 				}
 				else
 				{
-				chval=convert_ascii_vec(t,s->his.h->v.vector);
+				chval=convert_ascii_vec(t,s->his.h->v.h_vector);
 				}
 			}
 
@@ -697,8 +858,7 @@ while(s)
 				if((chval2=chval))
 				while((ch=*(chval2++)))
 					{
-					if((((ch>='1')&&(ch<='9'))||((ch>='A')&&(ch<='F')))
-						&&(ch!='X')&&(ch!='Z'))
+					if(((ch>='1')&&(ch<='9'))||(ch=='H')||((ch>='A')&&(ch<='F')))
 						{
 						s->search_result=1;
 						break;
@@ -713,8 +873,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='1')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='X')||(ch=='Z'))
+					if((ch!='0')&&(ch!='L'))
 						{
 						s->search_result=0;
 						break;
@@ -730,8 +889,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='0')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='X'))
+					if(ch!='Z')
 						{
 						s->search_result=0;
 						break;
@@ -747,8 +905,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='0')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='Z'))
+					if((ch!='X')&&(ch!='W'))
 						{
 						s->search_result=0;
 						break;
@@ -770,8 +927,8 @@ while(s)
 				break;
 
 			default:
-				printf("Internal error: st_type of %d\n",s->value);
-				exit(0);
+				fprintf(stderr, "Internal error: st_type of %d\n",s->value);
+				exit(255);
 			}
 
 		free_2(chval);
@@ -927,11 +1084,11 @@ while(s)
 			}
 		if(t->flags&TR_INVERT)  
                 	{
-                        str[0]="1XZ0"[s->his.h->v.val];
+                        str[0]=AN_STR_INV[s->his.h->v.h_val];
                         }
                         else   
                         {
-                        str[0]="0XZ1"[s->his.h->v.val];
+                        str[0]=AN_STR[s->his.h->v.h_val];
                         }
 		str[1]=0x00;
 
@@ -942,23 +1099,23 @@ while(s)
 				
 			case ST_HIGH:
 				totaltraces++;
-				if(str[0]=='1') s->search_result=1;	
+				if((str[0]=='1')||(str[0]=='H')) s->search_result=1;	
 				break;
 
 			case ST_RISE:
 				totaltraces++;
-				if((str[0]=='1')&&(adjust(s->his.h->time,shift_timebase)==maxbase)) 
+				if(((str[0]=='1')||(str[0]=='H'))&&(adjust(s->his.h->time,shift_timebase)==maxbase)) 
 					s->search_result=1;	
 				break;
 
 			case ST_LOW:
 				totaltraces++;
-				if(str[0]=='0') s->search_result=1;
+				if((str[0]=='0')||(str[0]=='L')) s->search_result=1;
 				break;
 
 			case ST_FALL:
 				totaltraces++;
-				if((str[0]=='0')&&(adjust(s->his.h->time,shift_timebase)==maxbase))
+				if(((str[0]=='0')||(str[0]=='L'))&&(adjust(s->his.h->time,shift_timebase)==maxbase))
  					s->search_result=1;
 				break;
 
@@ -985,8 +1142,8 @@ while(s)
 				break;
 
 			default:
-				printf("Internal error: st_type of %d\n",s->value);
-				exit(0);
+				fprintf(stderr, "Internal error: st_type of %d\n",s->value);
+				exit(255);
 			}
 
 
@@ -1016,11 +1173,11 @@ while(s)
 				{
 				if(!(s->his.h->flags&HIST_STRING))
 					{
-					chval=convert_ascii_real((double *)s->his.h->v.vector);
+					chval=convert_ascii_real((double *)s->his.h->v.h_vector);
 					}
 					else
 					{
-					chval=convert_ascii_string((char *)s->his.h->v.vector);
+					chval=convert_ascii_string((char *)s->his.h->v.h_vector);
 					chval2=chval;
 					while((ch=*chval2))	/* toupper() the string */
 						{
@@ -1031,7 +1188,7 @@ while(s)
 				}
 				else
 				{
-				chval=convert_ascii_vec(t,s->his.h->v.vector);
+				chval=convert_ascii_vec(t,s->his.h->v.h_vector);
 				}
 			}
 
@@ -1050,8 +1207,7 @@ while(s)
 				if((chval2=chval))
 				while((ch=*(chval2++)))
 					{
-					if((((ch>='1')&&(ch<='9'))||((ch>='A')&&(ch<='F')))
-						&&(ch!='X')&&(ch!='Z'))
+					if(((ch>='1')&&(ch<='9'))||(ch=='H')||((ch>='A')&&(ch<='F')))
 						{
 						s->search_result=1;
 						break;
@@ -1066,8 +1222,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='1')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='X')||(ch=='Z'))
+					if((ch!='0')&&(ch!='L'))
 						{
 						s->search_result=0;
 						break;
@@ -1083,8 +1238,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='0')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='X'))
+					if(ch!='Z')
 						{
 						s->search_result=0;
 						break;
@@ -1100,8 +1254,7 @@ while(s)
 				s->search_result=1;
 				while((ch=*(chval2++)))
 					{
-					if(((ch>='0')&&(ch<='9'))||((ch>='A')&&(ch<='F'))
-						||(ch=='Z'))
+					if((ch!='X')&&(ch!='W'))
 						{
 						s->search_result=0;
 						break;
@@ -1123,8 +1276,8 @@ while(s)
 				break;
 
 			default:
-				printf("Internal error: st_type of %d\n",s->value);
-				exit(0);
+				fprintf(stderr, "Internal error: st_type of %d\n",s->value);
+				exit(255);
 			}
 
 		free_2(chval);
@@ -1188,8 +1341,9 @@ return(maxbase);
 void strace_maketimetrace(int mode)
 {
 TimeType basetime=tims.first;
+TimeType endtime =MAX_HISTENT_TIME;
 int i, notfirst=0;
-struct timechain *tchead=NULL, *tc=NULL;
+struct timechain *tchead=NULL, *tc=NULL, *tcnext;
 TimeType *t;
 
 if(timearray)
@@ -1207,31 +1361,79 @@ if((!mode)&&(!shadow_active))
 	return;	/* merely free stuff up */
 	}
 
-do
+if(mark_idx_start>0)
+	{
+	if(named_markers[mark_idx_start-1]>=0)
+		basetime=named_markers[mark_idx_start-1];
+	else
+		{
+		status_text(item_mark_start_strings[mark_idx_start].str);
+		status_text(" not in use.\n");
+		}
+	}
+
+if(mark_idx_end>0)
+	{
+	if(named_markers[mark_idx_end-1]>=0)
+		endtime=named_markers[mark_idx_end-1];
+	else
+		{
+		status_text(item_mark_end_strings[mark_idx_end].str);
+		status_text(" not in use.\n");
+		}
+	}
+
+if(basetime>endtime)
+	{
+	TimeType tmp=basetime;
+	basetime    =endtime;
+	endtime     =tmp;
+	}
+
+while(1)
 	{
 	basetime=strace_timetrace(basetime, notfirst);
 	notfirst=1;
 
+	if(endtime==MAX_HISTENT_TIME)
+		{
+		if(basetime==MAX_HISTENT_TIME) break;
+		}
+		else
+	   	{
+		if(basetime>=endtime) break;
+		}
+
+	timearray_size++;
+
 	if(!tc)
 		{
-		tchead=tc=wave_alloca(sizeof(struct timechain));
+		tchead=tc=malloc_2(sizeof(struct timechain));	/* formerly wave_alloca but that dies > 4MB on linux x86! */
 		}
 		else
 		{
-		tc->next=wave_alloca(sizeof(struct timechain));
+		tc->next=malloc_2(sizeof(struct timechain));	/* formerly wave_alloca but that dies > 4MB on linux x86! */
 		tc=tc->next;
 		}
 
 	tc->t=basetime;
 	tc->next=NULL;
-	timearray_size++;
-	} while(basetime!=MAX_HISTENT_TIME);
+	}
 
-timearray=t=malloc_2(sizeof(TimeType)*timearray_size);
-for(i=0;i<timearray_size;i++)
+if(timearray_size)
 	{
-	*(t++)=tchead->t;
-	tchead=tchead->next;
+	timearray=t=malloc_2(sizeof(TimeType)*timearray_size);
+	for(i=0;i<timearray_size;i++)
+		{
+		*(t++)=tchead->t;
+		tcnext=tchead->next;
+		free_2(tchead);					/* need to explicitly free this up now */
+		tchead=tcnext;
+		}
+	}
+	else
+	{
+	timearray = NULL;
 	}
 
 if(!shadow_active) update_mark_count_label();
@@ -1245,6 +1447,7 @@ void swap_strace_contexts(void)
 {
 struct strace *stemp;
 char logical_mutex_temp[6];
+char  mark_idx_start_temp, mark_idx_end_temp;
 
 stemp = straces;
 straces = shadow_straces;
@@ -1253,6 +1456,14 @@ shadow_straces = stemp;
 memcpy(logical_mutex_temp, logical_mutex, 6);
 memcpy(logical_mutex, shadow_logical_mutex, 6);
 memcpy(shadow_logical_mutex, logical_mutex_temp, 6);
+
+mark_idx_start_temp   = mark_idx_start;
+mark_idx_start        = shadow_mark_idx_start;
+shadow_mark_idx_start = mark_idx_start_temp;
+
+mark_idx_end_temp   = mark_idx_end;
+mark_idx_end        = shadow_mark_idx_end;
+shadow_mark_idx_end = mark_idx_end_temp;
 }
 
 
@@ -1270,6 +1481,9 @@ for(i=0;i<6;i++)
 	shadow_logical_mutex[i] = 0;
 	}
 
+shadow_mark_idx_start = 0;
+shadow_mark_idx_end   = 0;
+
 strace_cache = straces;	/* so the trace actually deletes */
 straces=NULL;
 
@@ -1356,7 +1570,6 @@ if(mprintf_buff_head)
 void cache_actual_pattern_mark_traces(void)
 {
 Trptr t;
-int i;
 unsigned int def=0;
 TimeType prevshift=LLDescriptor(0);
 struct strace *st;
@@ -1365,7 +1578,7 @@ delete_mprintf();
 
 if(timearray)
 	{
-	mprintf("!%d%d%d%d%d%d\n", logical_mutex[0], logical_mutex[1], logical_mutex[2], logical_mutex[3], logical_mutex[4], logical_mutex[5]);
+	mprintf("!%d%d%d%d%d%d%c%c\n", logical_mutex[0], logical_mutex[1], logical_mutex[2], logical_mutex[3], logical_mutex[4], logical_mutex[5], '@'+mark_idx_start, '@'+mark_idx_end);
 	st=straces;
 
 	while(st)
@@ -1392,7 +1605,7 @@ if(timearray)
 			}
 		prevshift=t->shift;
 
-		if(!(t->flags&TR_BLANK))	
+		if(!(t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH)))	
 			{
 			if(t->vector)
 				{
