@@ -8,6 +8,7 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "aet.h"
 #include "debug.h"
 
@@ -28,7 +29,7 @@ gtk_signal_emit_by_name (GTK_OBJECT (GTK_ADJUSTMENT(wave_hslider)), "value_chang
 static void
 callback(GtkWidget *widget, GtkWidget *entry)
 {
-gchar *entry_text;
+G_CONST_RETURN gchar *entry_text;
 TimeType newlo;
 char fromstr[40];
 
@@ -64,7 +65,7 @@ if(newlo<(tims.last))
 static void
 callback2(GtkWidget *widget, GtkWidget *entry)
 {
-gchar *entry_text;
+G_CONST_RETURN gchar *entry_text;
 TimeType newhi;
 char tostr[40];
 
