@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2001.
+ * Copyright (c) Tony Bybell 1999-2004.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -96,7 +96,7 @@ while(s)
 
 		t2=(struct tree *)calloc_2(1,sizeof(struct tree));
 		t->child=t2;			
-		t2->parent=t;
+		/* t2->parent=t; */
 		t=t2;
 
 		continue;
@@ -123,7 +123,7 @@ while(s)
 
 		t3=(struct tree *)calloc_2(1,sizeof(struct tree));
 		t->child=t3;
-		t3->parent=t;
+		/* t3->parent=t; */
 		t=t3;		
 
 		continue;
@@ -179,7 +179,9 @@ while(t)
 		{
 		treedebug(t->child, s2);
 		}
-		else
+
+	if(t->which>=0) /* for when valid netnames like A.B.C, A.B.C.D exist (not legal excluding texsim) */
+			/* otherwise this would be an 'else' */
 		{
 		printf("%d) %s\n", t->which, s2);
 		}
@@ -192,8 +194,6 @@ while(t)
 
 static GtkCTreeNode *maketree_nodes(GtkCTreeNode *subtree, struct tree *t2, GtkCTreeNode *sibling, int mode)
 {
-GtkWidget *titem;
-GtkWidget *added;
 char *tmp, *tmp2, *tmp3;
 gchar *text [1];
 
@@ -327,7 +327,7 @@ char ch;
 if(nam)
 	{
 	t=nam;
-	while(ch=*(t++))
+	while((ch=*(t++)))
 		{
 		if(ch==hier_delimeter) pnt=t;
 		}
@@ -335,3 +335,122 @@ if(nam)
 
 return(pnt?pnt:nam);
 }
+
+/**********************************/
+/* Experimental treesorting code  */
+/* (won't directly work with lxt2 */
+/* because alias hier is after    */
+/* fac hier so fix with partial   */
+/* mergesort...)                  */
+/**********************************/
+
+/*
+ * sort the hier tree..should be faster than
+ * moving numfacs longer strings around
+ */
+int fast_tree_sort = 1;
+
+static int tree_qsort_cmp(const void *v1, const void *v2)
+{
+struct tree *t1 = *(struct tree **)v1;
+struct tree *t2 = *(struct tree **)v2;
+
+return(sigcmp(t2->name, t1->name));	/* because list must be in rvs */
+}
+
+void treesort(struct tree *t, struct tree *p)
+{
+struct tree *it;
+struct tree **srt;
+int cnt;
+int i;
+
+if(t->next)
+	{
+	it = t;
+	cnt = 0;
+	do	{
+		cnt++;
+		it=it->next;
+		} while(it);
+	
+	srt = wave_alloca(cnt * sizeof(struct tree *));
+	for(i=0;i<cnt;i++)
+		{
+		srt[i] = t;
+		t=t->next;
+		}
+
+	qsort((void *)srt, cnt, sizeof(struct tree *), tree_qsort_cmp);
+
+	if(p)
+		{
+		p->child = srt[0];
+		}
+		else
+		{
+		treeroot = srt[0];
+		}
+
+	for(i=0;i<(cnt-1);i++)
+		{
+		srt[i]->next = srt[i+1];
+		if(srt[i]->child)
+			{
+			treesort(srt[i]->child, srt[i]);
+			}
+		}
+	srt[i]->next = NULL;
+	if(srt[i]->child)
+		{
+		treesort(srt[i]->child, srt[i]);
+		}
+	}
+else if (t->child)
+	{
+	treesort(t->child, t);
+	}
+}
+
+static struct symbol **facs2;
+static int facs2_pos;
+
+void order_facs_from_treesort_2(struct tree *t)
+{
+while(t)
+	{
+	if(t->child)
+		{
+		order_facs_from_treesort_2(t->child);
+		}
+
+	if(t->which>=0) /* for when valid netnames like A.B.C, A.B.C.D exist (not legal excluding texsim) */
+			/* otherwise this would be an 'else' */
+		{
+		facs2[facs2_pos--] = facs[t->which];
+		}
+
+	t=t->next;
+	}
+}
+
+
+void order_facs_from_treesort(struct tree *t, void *v)
+{
+struct symbol ***f = (struct symbol ***)v; /* eliminate compiler warning in tree.h as aet.h refs tree.h */
+
+facs2=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
+facs2_pos = numfacs-1;
+order_facs_from_treesort_2(t);
+
+if(facs2_pos>=0)
+	{
+	fprintf(stderr, "Internal Error: facs2_pos = %d\n", facs2_pos);
+	fprintf(stderr, "[This is usually the result of multiply defined facilities.]\n");
+	exit(255);
+	}
+
+free_2(*f);
+*f = facs2;
+facs2 = NULL;
+}
