@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,11 +23,13 @@
 
 extern char hier_delimeter, hier_was_explicitly_set, alt_hier_delimeter;
 extern char hier_grouping;
-extern struct tree *selectedtree;
 
 struct tree
 {
+#if 0
 struct tree *parent; /* null if toplevel */
+#endif
+
 struct tree *next;
 struct tree *child;
 char *name;
@@ -43,6 +45,8 @@ struct treechain *next;
 
 
 extern struct tree *treeroot;
+extern int fast_tree_sort; /* won't work with lxt2 because of how aliasing stacks */
+			   /* names at the end of the main hierarchy 010104ajb    */
 
 void init_tree(void);
 void build_tree_from_name(char *s, int which);
@@ -51,5 +55,7 @@ void treedebug(struct tree *t, char *s);
 void maketree(GtkCTreeNode *subtree, struct tree *t);
 char *leastsig_hiername(char *nam);
 
-#endif
+void treesort(struct tree *t, struct tree *p);
+void order_facs_from_treesort(struct tree *t, void *v);
 
+#endif
