@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2004.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -8,6 +8,7 @@
  */
 
 #include <gtk/gtk.h>
+#include "gtk12compat.h"
 #include "analyzer.h"
 #include "tree.h"
 #include "aet.h"
@@ -57,7 +58,7 @@ return(is_active);
 
 static void enter_callback_e(GtkWidget *widget, GtkWidget *nothing)
 {
-  gchar *entry_text;
+  G_CONST_RETURN gchar *entry_text;
   int len;
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry_a));
   DEBUG(printf("Entry contents: %s\n", entry_text));
@@ -139,7 +140,6 @@ static void entrybox_local(char *title, int width, char *default_text, int maxch
 /***************************************************************************/
 
 static GtkWidget *window;
-static GtkWidget *entry;
 static GtkWidget *tree;
 static char bundle_direction=0;
 static GtkSignalFunc cleanup;
@@ -189,20 +189,20 @@ while(t)
 void fetchvex(struct tree *t, char direction)
 {
 if(t)
-if(t->child)
 	{
-	fetchvex2(t, direction, 0);
-	}
-	else
-	{
-	add_vector_range(NULL, fetchlow(t)->which, 
-		fetchhigh(t)->which, direction);
+	if(t->child)
+		{
+		fetchvex2(t, direction, 0);
+		}
+		else
+		{
+		add_vector_range(NULL, fetchlow(t)->which, 
+			fetchhigh(t)->which, direction);
+		}
 	}
 }
 
 
-
-
 /* call cleanup() on ok/insert functions */
 
 static void
@@ -255,7 +255,7 @@ if(selectedtree)
 	{
 	if(!autoname_bundles)
 	        {
-	        entrybox_local("Enter Bundle Name",300,"",128,bundle_cleanup);
+	        entrybox_local("Enter Bundle Name",300,"",128,GTK_SIGNAL_FUNC(bundle_cleanup));
 	        }
 	        else
 	        {
@@ -497,7 +497,7 @@ while(tfirst)
 	{
 	tfirst->flags |= TR_HIGHLIGHT;
 	if(tfirst==tlast) break;
-	tfirst=tfirst->next;
+	tfirst=tfirst->t_next;
 	}
 
 MaxSignalLength();
