@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2001.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,12 +27,14 @@
 
 #undef VCD_BSEARCH_IS_PERFECT		/* bsearch is imperfect under linux, but OK under AIX */
 
+int vcd_warning_filesize = -1;		/* -1 means use VCD_SIZE_WARN, 0 means disable, any other size is a compare in MB */
 char autocoalesce=1, autocoalesce_reversal=0;
 
 int vcd_explicit_zero_subscripts=-1;  /* 0=yes, -1=no */
 char convert_to_reals=0;
 char atomic_vectors=1;
 char make_vcd_save_file=0;
+char vcd_preserve_glitches=0;
 FILE *vcd_save_handle=NULL;
 
 static FILE *vcd_handle=NULL;
@@ -44,7 +46,7 @@ static void vcd_build_symbols(void);
 static void vcd_cleanup(void);
 static void evcd_strcpy(char *dst, char *src);
 
-static int vcdbyteno=0;		/* really should be size_t, but this is only used for debugging mangled traces */
+static off_t vcdbyteno=0;
 static int error_count=0;	/* should always be zero */
 
 static int header_over=0;
@@ -52,7 +54,6 @@ static int dumping_off=0;
 static TimeType start_time=-1;
 static TimeType end_time=-1;
 static TimeType current_time=-1;
-static TimeType time_scale=1;	/* multiplier is 1, 10, 100 */
 
 static int num_glitches=0;
 static int num_glitch_regions=0;
@@ -242,7 +243,7 @@ if(sorted)
 	static int err = 0;
 	if(!err)
 		{
-		fprintf(stderr, "Near byte %d, VCD search table NULL..is this a VCD file?\n", vcdbyteno+(vst-vcdbuf));
+		fprintf(stderr, "Near byte %d, VCD search table NULL..is this a VCD file?\n", (int)(vcdbyteno+(vst-vcdbuf)));
 		err=1;
 		}
 	return(NULL);
@@ -272,7 +273,6 @@ static void create_sorted_table(void)
 struct vcdsymbol *v;
 struct vcdsymbol **pnt;
 unsigned int vcd_distance;
-int i;
 
 if(sorted) 
 	{
@@ -335,17 +335,25 @@ vcdbuf=vst=vend=NULL;
 }
 
 
+static off_t vcd_fsiz = 0;
+
 static int getch_fetch(void)
 {
 size_t rd;
 
-if(feof(vcd_handle)||errno) return(-1);
+errno = 0;
+if(feof(vcd_handle)) return(-1);
 
 vcdbyteno+=(vend-vcdbuf);
 rd=fread(vcdbuf, sizeof(char), VCD_BSIZ, vcd_handle);
 vend=(vst=vcdbuf)+rd;
 
-if(!rd) return(-1);
+if(vcd_fsiz)
+	{
+	splash_sync(vcdbyteno, vcd_fsiz);
+	}
+
+if((!rd)||(errno)) return(-1);
 
 return((int)(*(vst++)));
 }
@@ -700,16 +708,19 @@ switch(yytext[0])
 	{
 	case '0':
 	case '1':
-	case 'x':
-	case 'X':
-	case 'z':
-	case 'Z':
+	case 'x': case 'X':
+	case 'z': case 'Z':
+	case 'h': case 'H':
+	case 'u': case 'U':
+	case 'w': case 'W':
+	case 'l': case 'L':
+	case '-':
 		if(yylen>1)
 			{
 			v=bsearch_vcd(yytext+1, yylen-1);	
 			if(!v)
 				{
-				fprintf(stderr,"Near byte %d, Unknown VCD identifier: '%s'\n",vcdbyteno+(vst-vcdbuf),yytext+1);
+				fprintf(stderr,"Near byte %d, Unknown VCD identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)),yytext+1);
 				}
 				else
 				{
@@ -736,7 +747,7 @@ switch(yytext[0])
 			}
 			else
 			{
-			fprintf(stderr,"Near byte %d, Malformed VCD identifier\n", vcdbyteno+(vst-vcdbuf));
+			fprintf(stderr,"Near byte %d, Malformed VCD identifier\n", (int)(vcdbyteno+(vst-vcdbuf)));
 			}
 		break;
 
@@ -752,7 +763,7 @@ switch(yytext[0])
 		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
-			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 			free_2(vector);
 			}
 			else
@@ -773,7 +784,7 @@ switch(yytext[0])
 			
 				if(!v)
 					{
-					fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+					fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 					free_2(d);
 					}
 					else
@@ -846,7 +857,7 @@ switch(yytext[0])
 		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
-			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 			free_2(vector);
 			}
 			else
@@ -867,7 +878,7 @@ switch(yytext[0])
 			
 				if(!v)
 					{
-					fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+					fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 					free_2(d);
 					}
 					else
@@ -939,7 +950,7 @@ switch(yytext[0])
 		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
-			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 			free_2(d);
 			}
 			else
@@ -963,7 +974,7 @@ switch(yytext[0])
 		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
-			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
+			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",(int)(vcdbyteno+(vst-vcdbuf)), yytext);
 			free_2(d);
 			}
 			else
@@ -1129,7 +1140,8 @@ for(;;)
 		case T_VAR:
 			if((header_over)&&(0))
 			{
-			fprintf(stderr,"$VAR encountered after $ENDDEFINITIONS near byte %d.  VCD is malformed, exiting.\n",vcdbyteno+(vst-vcdbuf));
+			fprintf(stderr,"$VAR encountered after $ENDDEFINITIONS near byte %d.  VCD is malformed, exiting.\n",
+				(int)(vcdbyteno+(vst-vcdbuf)));
 			exit(0);
 			}
 			else
@@ -1376,7 +1388,7 @@ for(;;)
 						}
 					v->narray[0]=(struct Node *)calloc_2(1,sizeof(struct Node));
 					v->narray[0]->head.time=-1;
-					v->narray[0]->head.v.val=1;
+					v->narray[0]->head.v.h_val=AN_X;
 					}
 					else
 					{
@@ -1386,7 +1398,7 @@ for(;;)
 	
 						v->narray[i]=(struct Node *)calloc_2(1,sizeof(struct Node));
 						v->narray[i]->head.time=-1;
-						v->narray[i]->head.v.val=1;
+						v->narray[i]->head.v.h_val=AN_X;
 						}
 					}
 				}
@@ -1481,12 +1493,12 @@ for(;;)
 				error_count++;
 				if(v->name) 
 					{
-					fprintf(stderr, "Near byte %d, $VAR parse error encountered with '%s'\n", vcdbyteno+(vst-vcdbuf), v->name);
+					fprintf(stderr, "Near byte %d, $VAR parse error encountered with '%s'\n", (int)(vcdbyteno+(vst-vcdbuf)), v->name);
 					free_2(v->name);
 					}
 					else
 					{
-					fprintf(stderr, "Near byte %d, $VAR parse error encountered\n", vcdbyteno+(vst-vcdbuf));
+					fprintf(stderr, "Near byte %d, $VAR parse error encountered\n", (int)(vcdbyteno+(vst-vcdbuf)));
 					}
 				if(v->id) free_2(v->id);
 				if(v->value) free_2(v->value);
@@ -1601,7 +1613,7 @@ if(!n->curr)
 	{
 	he=histent_calloc();
         he->time=-1;
-        he->v.val=1;
+        he->v.h_val=AN_X;
 
 	n->curr=he;
 	n->head.next=he;
@@ -1612,18 +1624,23 @@ if(!n->curr)
 	{
 	if(regadd) { time*=(time_scale); }
 
-        if(ch=='0') heval=0; else
-        if(ch=='1') heval=3; else
-        if((ch=='x')||(ch=='X')) heval=1; else
-        heval=2;
+	if(ch=='0')              heval=AN_0; else
+	if(ch=='1')              heval=AN_1; else
+        if((ch=='x')||(ch=='X')) heval=AN_X; else
+        if((ch=='z')||(ch=='Z')) heval=AN_Z; else
+        if((ch=='h')||(ch=='H')) heval=AN_H; else
+        if((ch=='u')||(ch=='U')) heval=AN_U; else
+        if((ch=='w')||(ch=='W')) heval=AN_W; else
+        if((ch=='l')||(ch=='L')) heval=AN_L; else
+        /* if(ch=='-') */        heval=AN_DASH;		/* default */
 	
-	if((n->curr->v.val!=heval)||(time==start_time)) /* same region == go skip */ 
+	if((n->curr->v.h_val!=heval)||(time==start_time)||(vcd_preserve_glitches)) /* same region == go skip */ 
         	{
 		if(n->curr->time==time)
 			{
 			DEBUG(printf("Warning: Glitch at time ["TTFormat"] Signal [%p], Value [%c->%c].\n",
-				time, n, "0XZ1"[n->curr->v.val], ch));
-			n->curr->v.val=heval;		/* we have a glitch! */
+				time, n, AN_STR[n->curr->v.h_val], ch));
+			n->curr->v.h_val=heval;		/* we have a glitch! */
 
 			num_glitches++;
 			if(!(n->curr->flags&HIST_GLITCH))
@@ -1636,7 +1653,7 @@ if(!n->curr)
 			{
                 	he=histent_calloc();
                 	he->time=time;
-                	he->v.val=heval;
+                	he->v.h_val=heval;
 
                 	n->curr->next=he;
 			n->curr=he;
@@ -1656,7 +1673,7 @@ switch(ch)
 		he=histent_calloc();
 		he->flags=(HIST_STRING|HIST_REAL);
 	        he->time=-1;
-	        he->v.vector=NULL;
+	        he->v.h_vector=NULL;
 	
 		n->curr=he;
 		n->head.next=he;
@@ -1671,8 +1688,8 @@ switch(ch)
 				{
 				DEBUG(printf("Warning: String Glitch at time ["TTFormat"] Signal [%p].\n",
 					time, n));
-				if(n->curr->v.vector) free_2(n->curr->v.vector);
-				n->curr->v.vector=vector;		/* we have a glitch! */
+				if(n->curr->v.h_vector) free_2(n->curr->v.h_vector);
+				n->curr->v.h_vector=vector;		/* we have a glitch! */
 	
 				num_glitches++;
 				if(!(n->curr->flags&HIST_GLITCH))
@@ -1686,7 +1703,7 @@ switch(ch)
 	                	he=histent_calloc();
 				he->flags=(HIST_STRING|HIST_REAL);
 	                	he->time=time;
-	                	he->v.vector=vector;
+	                	he->v.h_vector=vector;
 	
 	                	n->curr->next=he;
 				n->curr=he;
@@ -1702,7 +1719,7 @@ switch(ch)
 		he=histent_calloc();
 		he->flags=HIST_REAL;
 	        he->time=-1;
-	        he->v.vector=NULL;
+	        he->v.h_vector=NULL;
 	
 		n->curr=he;
 		n->head.next=he;
@@ -1714,17 +1731,18 @@ switch(ch)
 		if(regadd) { time*=(time_scale); }
 	
 		if(
-		  (n->curr->v.vector&&vector&&(*(double *)n->curr->v.vector!=*(double *)vector))
+		  (n->curr->v.h_vector&&vector&&(*(double *)n->curr->v.h_vector!=*(double *)vector))
 			||(time==start_time)
-			||(!n->curr->v.vector)
+			||(!n->curr->v.h_vector)
+			||(vcd_preserve_glitches)
 			) /* same region == go skip */ 
 	        	{
 			if(n->curr->time==time)
 				{
 				DEBUG(printf("Warning: Real number Glitch at time ["TTFormat"] Signal [%p].\n",
 					time, n));
-				if(n->curr->v.vector) free_2(n->curr->v.vector);
-				n->curr->v.vector=vector;		/* we have a glitch! */
+				if(n->curr->v.h_vector) free_2(n->curr->v.h_vector);
+				n->curr->v.h_vector=vector;		/* we have a glitch! */
 	
 				num_glitches++;
 				if(!(n->curr->flags&HIST_GLITCH))
@@ -1738,7 +1756,7 @@ switch(ch)
 	                	he=histent_calloc();
 				he->flags=HIST_REAL;
 	                	he->time=time;
-	                	he->v.vector=vector;
+	                	he->v.h_vector=vector;
 	
 	                	n->curr->next=he;
 				n->curr=he;
@@ -1758,7 +1776,7 @@ switch(ch)
 		{
 		he=histent_calloc();
 	        he->time=-1;
-	        he->v.vector=NULL;
+	        he->v.h_vector=NULL;
 	
 		n->curr=he;
 		n->head.next=he;
@@ -1770,17 +1788,18 @@ switch(ch)
 		if(regadd) { time*=(time_scale); }
 	
 		if(
-		  (n->curr->v.vector&&vector&&(strcmp(n->curr->v.vector,vector)))
+		  (n->curr->v.h_vector&&vector&&(strcmp(n->curr->v.h_vector,vector)))
 			||(time==start_time)
-			||(!n->curr->v.vector)
+			||(!n->curr->v.h_vector)
+			||(vcd_preserve_glitches)
 			) /* same region == go skip */ 
 	        	{
 			if(n->curr->time==time)
 				{
 				DEBUG(printf("Warning: Glitch at time ["TTFormat"] Signal [%p], Value [%c->%c].\n",
-					time, n, "0XZ1"[n->curr->v.val], ch));
-				if(n->curr->v.vector) free_2(n->curr->v.vector);
-				n->curr->v.vector=vector;		/* we have a glitch! */
+					time, n, AN_STR[n->curr->v.h_val], ch));
+				if(n->curr->v.h_vector) free_2(n->curr->v.h_vector);
+				n->curr->v.h_vector=vector;		/* we have a glitch! */
 	
 				num_glitches++;
 				if(!(n->curr->flags&HIST_GLITCH))
@@ -1793,7 +1812,7 @@ switch(ch)
 				{
 	                	he=histent_calloc();
 	                	he->time=time;
-	                	he->v.vector=vector;
+	                	he->v.h_vector=vector;
 	
 	                	n->curr->next=he;
 				n->curr=he;
@@ -1815,7 +1834,7 @@ switch(ch)
 
 static void add_tail_histents(void)
 {
-int i,j;
+int j;
 struct vcdsymbol *v;
 
 /* dump out any pending events 1st */
@@ -1890,12 +1909,13 @@ while(v)
 
 static void vcd_build_symbols(void)
 {
-int i,j;
+int j;
 int max_slen=-1;
 struct sym_chain *sym_chain=NULL, *sym_curr=NULL;
 int duphier=0;
 char hashdirty;
 struct vcdsymbol *v, *vprime;
+char *str = NULL;
 
 v=vcdsymroot;
 while(v)
@@ -1904,7 +1924,6 @@ while(v)
 	int delta;
 
 		{
-		char *str;
 		int slen;
 		int substnode;
 
@@ -1936,7 +1955,7 @@ while(v)
 
 		if(((v->size==1)||(!atomic_vectors))&&(v->vartype!=V_REAL))
 			{
-			struct symbol *s;
+			struct symbol *s = NULL;
 	
 			for(j=0;j<v->size;j++)
 				{
@@ -2169,7 +2188,6 @@ treeprune(treeroot);
 
 static void vcd_cleanup(void)
 {
-int i;
 struct slist *s, *s2;
 struct vcdsymbol *v, *vt;
 
@@ -2262,6 +2280,25 @@ if (((flen>2)&&(!strcmp(fname+flen-3,".gz")))||
 	if(strcmp("-vcd",fname))
 		{
 		vcd_handle=fopen(fname,"rb");
+
+		if(vcd_handle)
+			{
+			fseeko(vcd_handle, 0, SEEK_END);	/* do status bar for vcd load */
+			vcd_fsiz = ftello(vcd_handle);
+			fseeko(vcd_handle, 0, SEEK_SET);
+			}
+
+		if(vcd_warning_filesize < 0) vcd_warning_filesize = VCD_SIZE_WARN;
+
+		if(vcd_warning_filesize)
+		if(vcd_fsiz > (vcd_warning_filesize * (1024 * 1024)))
+			{
+			fprintf(stderr, "Warning! File size is %d MB.  This might fail to load.\n"
+					"Consider converting it to lxt, lxt2, or vzt database formats instead.  (See\n"
+					"the vcd2lxt(1), vcd2lxt2(1), and vzt2vzt(1) manpages for more information.)\n"
+					"To disable this warning, set rc variable vcd_warning_filesize to zero.\n\n",
+						(int)(vcd_fsiz/(1024*1024)));
+			}
 		}
 		else
 		{
@@ -2277,9 +2314,12 @@ if(!vcd_handle)
 	exit(1);
 	}
 
+/* SPLASH */				splash_create(); 
+
 getch_alloc();		/* alloc membuff for vcd getch buffer */
 
 build_slisthier();
+
 vcd_parse();
 if((!sorted)&&(!indexed))
 	{
@@ -2290,11 +2330,17 @@ add_tail_histents();
 
 if(vcd_save_handle) fclose(vcd_save_handle);
 
-printf("["TTFormat"] start time.\n["TTFormat"] end time.\n", start_time*time_scale, end_time*time_scale);
-if(num_glitches) printf("Warning: encountered %d glitch%s across %d glitch region%s.\n", 
+fprintf(stderr, "["TTFormat"] start time.\n["TTFormat"] end time.\n", start_time*time_scale, end_time*time_scale);
+if(num_glitches) fprintf(stderr, "Warning: encountered %d glitch%s across %d glitch region%s.\n", 
 		num_glitches, (num_glitches!=1)?"es":"",
 		num_glitch_regions, (num_glitch_regions!=1)?"s":"");
 
+if(vcd_fsiz)
+        {
+        splash_sync(vcd_fsiz, vcd_fsiz);
+	vcd_fsiz = 0;
+        }
+
 vcd_build_symbols();
 vcd_sortfacs();
 vcd_cleanup();
