@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,11 @@
 #include <unistd.h>
 #endif
 
+#if defined _MSC_VER || defined __MINGW32__
+#define fseeko fseek
+#define ftello ftell
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -24,7 +29,9 @@
 #include "aet.h"
 #include "alloca.h"
 #include "debug.h"
+#include "tree.h"
 
+#define VCD_SIZE_WARN (128)	/* number of MB size where converter warning message appears */
 #define VCD_BSIZ 32768	/* size of getch() emulation buffer--this val should be ok */
 #define VCD_INDEXSIZ  (2 * 1024 * 1024)
 
@@ -70,9 +77,11 @@ struct vcdsymbol *sym;
 TimeType last_event_time;    /* make +1 == 0 if there's not an event there too */
 };
 
+extern int vcd_warning_filesize;
 extern char autocoalesce;
 extern char autocoalesce_reversal;
 extern int vcd_explicit_zero_subscripts;  /* 0=yes, -1=no */
+extern char vcd_preserve_glitches;
 extern char convert_to_reals;
 extern char atomic_vectors;
 extern char make_vcd_save_file;
