@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2005.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -88,6 +88,9 @@ if(helpbox_is_active)
         help_text_bold("\n\nZoom Out");
         help_text(
 		" is used to decrease the zoom factor."
+#ifdef WAVE_USE_GTK2
+		" Alt + Scrollwheel Up also performs this function."
+#endif
         );
         return;
         }
@@ -139,6 +142,9 @@ if(helpbox_is_active)
         help_text_bold("\n\nZoom In");
         help_text(
                 " is used to increase the zoom factor."  
+#ifdef WAVE_USE_GTK2
+		" Alt + Scrollwheel Down also performs this function."
+#endif
         );
         return;
         }
@@ -224,6 +230,50 @@ int fixedwidth;
 if(helpbox_is_active)
         {
         help_text_bold("\n\nZoom Best Fit");
+        help_text(
+		" attempts a \"best fit\" to get the whole trace onscreen."
+		"  Note that the trace may be more or less than a whole screen since"
+		" this isn't a \"perfect fit.\" Also, if the middle button baseline"
+		" marker is nailed down, the zoom instead of getting the whole trace"
+		" onscreen will use the part of the trace between the primary"
+		" marker and the baseline marker."
+        );
+        return;
+        }
+
+if((tims.baseline>=0)&&(tims.marker>=0))
+	{
+	service_dragzoom(tims.baseline, tims.marker); /* new semantics added to zoom between the two */
+	}
+	else
+	{
+	if(wavewidth>4) { fixedwidth=wavewidth-4; } else { fixedwidth=wavewidth; }
+	estimated=-log(((gdouble)(tims.last-tims.first+1))/((gdouble)fixedwidth)*((gdouble)200.0))/log(zoombase);
+	if(estimated>((gdouble)(0.0))) estimated=((gdouble)(0.0));
+	
+	tims.prevzoom=tims.zoom;
+	tims.timecache=0;
+	
+	calczoom(estimated);
+	tims.zoom=estimated;
+
+	fix_wavehadj();
+	
+	gtk_signal_emit_by_name (GTK_OBJECT (GTK_ADJUSTMENT(wave_hslider)), "changed"); /* force zoom update */
+	gtk_signal_emit_by_name (GTK_OBJECT (GTK_ADJUSTMENT(wave_hslider)), "value_changed"); /* force zoom update */
+	}
+	
+DEBUG(printf("Zoombuttons Fit\n"));
+}
+
+void service_zoom_full(GtkWidget *text, gpointer data)
+{
+gdouble estimated;
+int fixedwidth;
+
+if(helpbox_is_active)
+        {
+        help_text_bold("\n\nZoom Full");
         help_text(
 		" attempts a \"best fit\" to get the whole trace onscreen."
 		"  Note that the trace may be more or less than a whole screen since"
@@ -235,19 +285,19 @@ if(helpbox_is_active)
 if(wavewidth>4) { fixedwidth=wavewidth-4; } else { fixedwidth=wavewidth; }
 estimated=-log(((gdouble)(tims.last-tims.first+1))/((gdouble)fixedwidth)*((gdouble)200.0))/log(zoombase);
 if(estimated>((gdouble)(0.0))) estimated=((gdouble)(0.0));
-
+	
 tims.prevzoom=tims.zoom;
 tims.timecache=0;
-
+	
 calczoom(estimated);
 tims.zoom=estimated;
 
 fix_wavehadj();
-
+	
 gtk_signal_emit_by_name (GTK_OBJECT (GTK_ADJUSTMENT(wave_hslider)), "changed"); /* force zoom update */
 gtk_signal_emit_by_name (GTK_OBJECT (GTK_ADJUSTMENT(wave_hslider)), "value_changed"); /* force zoom update */
-
-DEBUG(printf("Zoombuttons Fit\n"));
+	
+DEBUG(printf("Zoombuttons Full\n"));
 }
 
 
@@ -275,17 +325,20 @@ if(time2>time1)	/* ensure at least 1 tick */
 	tims.prevzoom=tims.zoom;
 	tims.timecache=tims.laststart=tims.start=time_trunc(time1);
 
-        for(t=traces.first;t;t=t->next)	/* have to nuke string refs so printout is ok! */
+        for(t=traces.first;t;t=t->t_next) /* have to nuke string refs so printout is ok! */
                 {
                 if(t->asciivalue) { free_2(t->asciivalue); t->asciivalue=NULL; }
                 }
 
-        for(t=traces.buffer;t;t=t->next)
+        for(t=traces.buffer;t;t=t->t_next)
                 {
                 if(t->asciivalue) { free_2(t->asciivalue); t->asciivalue=NULL; }
                 }
 
-        update_markertime(tims.marker=-1);
+	if(!((tims.baseline>=0)&&(tims.marker>=0)))
+		{
+	        update_markertime(tims.marker=-1);
+		}
         signalwindow_width_dirty=1;
         MaxSignalLength();
 
